@@ -19,26 +19,12 @@
 #ifndef _SW_ASYNC_H_
 #define _SW_ASYNC_H_
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+SW_EXTERN_C_BEGIN
 
 #ifndef O_DIRECT
 #define O_DIRECT         040000
 #endif
 
-enum swAioOpcode
-{
-    SW_AIO_RAW,
-    SW_AIO_READ = 1,
-    SW_AIO_WRITE,
-    SW_AIO_GETHOSTBYNAME,
-    SW_AIO_GETADDRINFO,
-    SW_AIO_FGETS,
-    SW_AIO_READ_FILE,
-    SW_AIO_WRITE_FILE,
-};
-
 enum swAioFlag
 {
     SW_AIO_WRITE_FSYNC = 1u << 1,
@@ -48,17 +34,27 @@ enum swAioFlag
 typedef struct _swAio_event
 {
     int fd;
-    int task_id;
-    uint8_t type;
+    size_t task_id;
     uint8_t lock;
     uint8_t canceled;
+    /**
+     * input & output
+     */
     uint16_t flags;
     off_t offset;
     size_t nbytes;
     void *buf;
     void *req;
+    /**
+     * output
+     */
     int ret;
     int error;
+    /**
+     * reserved by system
+     */
+    int pipe_fd;
+    double timestamp;
     void *object;
     void (*handler)(struct _swAio_event *event);
     void (*callback)(struct _swAio_event *event);
@@ -66,21 +62,15 @@ typedef struct _swAio_event
 
 typedef void (*swAio_handler)(swAio_event *event);
 
-typedef struct
-{
-    uint8_t init;
-    uint16_t min_thread_count;
-    uint16_t max_thread_count;
-    uint32_t task_num;
-    swLock lock;
-} swAsyncIO;
-
-extern swAsyncIO SwooleAIO;
-
-void swAio_free(void);
-int swAio_dispatch(const swAio_event *request);
+ssize_t swAio_dispatch(const swAio_event *request);
 swAio_event* swAio_dispatch2(const swAio_event *request);
 int swAio_cancel(int task_id);
+int swAio_callback(swReactor *reactor, swEvent *_event);
+size_t swAio_thread_count();
+
+#ifdef SW_DEBUG
+void swAio_notify_one();
+#endif
 
 void swAio_handler_read(swAio_event *event);
 void swAio_handler_write(swAio_event *event);
@@ -90,8 +80,6 @@ void swAio_handler_fgets(swAio_event *event);
 void swAio_handler_read_file(swAio_event *event);
 void swAio_handler_write_file(swAio_event *event);
 
-#ifdef __cplusplus
-}
-#endif
+SW_EXTERN_C_END
 
 #endif /* _SW_ASYNC_H_ */
