@@ -55,14 +55,17 @@ enum swErrorCode
     SW_ERROR_SESSION_INVALID_ID,
     SW_ERROR_SESSION_DISCARD_TIMEOUT_DATA,
     SW_ERROR_OUTPUT_BUFFER_OVERFLOW,
+    SW_ERROR_OUTPUT_SEND_YIELD,
     SW_ERROR_SSL_NOT_READY,
     SW_ERROR_SSL_CANNOT_USE_SENFILE,
     SW_ERROR_SSL_EMPTY_PEER_CERTIFICATE,
     SW_ERROR_SSL_VEFIRY_FAILED,
     SW_ERROR_SSL_BAD_CLIENT,
     SW_ERROR_SSL_BAD_PROTOCOL,
+    SW_ERROR_SSL_RESET,
 
     SW_ERROR_PACKAGE_LENGTH_TOO_LARGE = 1201,
+    SW_ERROR_PACKAGE_LENGTH_NOT_FOUND,
     SW_ERROR_DATA_LENGTH_TOO_LARGE,
 
     /**
@@ -70,6 +73,7 @@ enum swErrorCode
      */
     SW_ERROR_TASK_PACKAGE_TOO_BIG = 2001,
     SW_ERROR_TASK_DISPATCH_FAIL,
+    SW_ERROR_TASK_TIMEOUT,
 
     /**
      * http2 protocol error
@@ -83,6 +87,7 @@ enum swErrorCode
      */
     SW_ERROR_AIO_BAD_REQUEST = 4001,
     SW_ERROR_AIO_CANCELED,
+    SW_ERROR_AIO_TIMEOUT,
 
     /**
      * Client & Socket
@@ -128,6 +133,7 @@ enum swErrorCode
      */
     SW_ERROR_CO_OUT_OF_COROUTINE = 10001,
     SW_ERROR_CO_HAS_BEEN_BOUND,
+    SW_ERROR_CO_HAS_BEEN_DISCARDED,
 
     SW_ERROR_CO_MUTEX_DOUBLE_UNLOCK,
     SW_ERROR_CO_BLOCK_OBJECT_LOCKED,
