@@ -17,56 +17,45 @@
 #ifndef SW_SERVER_H_
 #define SW_SERVER_H_
 
-#include "swoole.h"
+#include "swoole_api.h"
 #include "buffer.h"
 #include "connection.h"
 
-#if defined(__sun) && !defined(s6_addr32)
-#define s6_addr32                  _S6_un._S6_u32
-#endif
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+SW_EXTERN_C_BEGIN
 
 #define SW_REACTOR_NUM             SW_CPU_NUM
 #define SW_WORKER_NUM              (SW_CPU_NUM*2)
 
 enum swServer_event_type
 {
-    //networking socket
-    SW_EVENT_TCP,
-    SW_EVENT_UDP,
-    SW_EVENT_TCP6,
-    SW_EVENT_UDP6,
-    //tcp event
-    SW_EVENT_CLOSE,
-    SW_EVENT_CONNECT,
-    //timer
-    SW_EVENT_TIMER,
+    //data payload
+    SW_SERVER_EVENT_SEND_DATA,
+    SW_SERVER_EVENT_SEND_FILE,
+    SW_SERVER_EVENT_SNED_DGRAM,
+    //connection event
+    SW_SERVER_EVENT_CLOSE,
+    SW_SERVER_EVENT_CONNECT,
     //task
-    SW_EVENT_TASK,
-    SW_EVENT_FINISH,
-    //sendfile
-    SW_EVENT_SENDFILE,
-    //dgram
-    SW_EVENT_UNIX_DGRAM,
-    SW_EVENT_UNIX_STREAM,
+    SW_SERVER_EVENT_TASK,
+    SW_SERVER_EVENT_FINISH,
     //pipe
-    SW_EVENT_PIPE_MESSAGE,
+    SW_SERVER_EVENT_PIPE_MESSAGE,
     //proxy
-    SW_EVENT_PROXY_START,
-    SW_EVENT_PROXY_END,
-    SW_EVENT_CONFIRM,
+    SW_SERVER_EVENT_PROXY_START,
+    SW_SERVER_EVENT_PROXY_END,
     //event operate
-    SW_EVENT_PAUSE_RECV,
-    SW_EVENT_RESUME_RECV,
+    SW_SERVER_EVENT_CONFIRM,
+    SW_SERVER_EVENT_PAUSE_RECV,
+    SW_SERVER_EVENT_RESUME_RECV,
     //buffer event
-    SW_EVENT_BUFFER_FULL,
-    SW_EVENT_BUFFER_EMPTY,
+    SW_SERVER_EVENT_BUFFER_FULL,
+    SW_SERVER_EVENT_BUFFER_EMPTY,
+    //process message
+    SW_SERVER_EVENT_INCOMING,
+    SW_SERVER_EVENT_SHUTDOWN,
 };
 
-enum swIPCType
+enum swIPC_type
 {
     SW_IPC_NONE     = 0,
     SW_IPC_UNIXSOCK = 1,
@@ -74,7 +63,7 @@ enum swIPCType
     SW_IPC_SOCKET   = 3,
 };
 
-enum swTaskIPCMode
+enum swTask_ipc_mode
 {
     SW_TASK_IPC_UNIXSOCK    = 1,
     SW_TASK_IPC_MSGQUEUE    = 2,
@@ -82,15 +71,7 @@ enum swTaskIPCMode
     SW_TASK_IPC_STREAM      = 4,
 };
 
-enum swResponseType
-{
-    SW_RESPONSE_SMALL = 0,
-    SW_RESPONSE_SHM = 1,
-    SW_RESPONSE_TMPFILE,
-    SW_RESPONSE_EXIT,
-};
-
-enum swWorkerPipeType
+enum swWorker_pipe_type
 {
     SW_PIPE_WORKER     = 0,
     SW_PIPE_MASTER     = 1,
@@ -100,7 +81,7 @@ enum swWorkerPipeType
 /**
  * use swDataHead->from_fd, 1 byte 8 bit
  */
-enum swTaskType
+enum swTask_type
 {
     SW_TASK_TMPFILE    = 1,  //tmp file
     SW_TASK_SERIALIZE  = 2,  //php serialize
@@ -112,11 +93,31 @@ enum swTaskType
     SW_TASK_NOREPLY    = 128, //don't reply
 };
 
+enum swFactory_dispatch_mode
+{
+    SW_DISPATCH_ROUND    = 1,
+    SW_DISPATCH_FDMOD    = 2,
+    SW_DISPATCH_QUEUE    = 3,
+    SW_DISPATCH_IPMOD    = 4,
+    SW_DISPATCH_UIDMOD   = 5,
+    SW_DISPATCH_USERFUNC = 6,
+    SW_DISPATCH_STREAM   = 7,
+};
+
+enum swFactory_dispatch_result
+{
+    SW_DISPATCH_RESULT_DISCARD_PACKET    = -1,
+    SW_DISPATCH_RESULT_CLOSE_CONNECTION  = -2,
+    SW_DISPATCH_RESULT_USERFUNC_FALLBACK = -3,
+};
+
 typedef struct _swReactorThread
 {
     pthread_t thread_id;
     swReactor reactor;
     int notify_pipe;
+    uint32_t pipe_num;
+    void *send_buffers;
 } swReactorThread;
 
 typedef struct _swListenPort
@@ -146,7 +147,7 @@ typedef struct _swListenPort
     uint32_t buffer_high_watermark;
     uint32_t buffer_low_watermark;
 
-    uint8_t type;
+    enum swSocket_type type;
     uint8_t ssl;
     int port;
     int sock;
@@ -231,74 +232,66 @@ typedef struct _swUserWorker_node
     swWorker *worker;
 } swUserWorker_node;
 
-typedef struct {
-	char *filename;
-	uint16_t name_len;
-	int fd;
-	size_t length;
-	off_t offset;
+typedef struct _swTask_sendfile
+{
+    char *filename;
+    uint16_t name_len;
+    int fd;
+    size_t length;
+    off_t offset;
 } swTask_sendfile;
 
-typedef struct
-{
-    uint16_t num;
-} swUserWorker;
-
-typedef struct
+typedef struct _swWorkerStopMessage
 {
     pid_t pid;
     uint16_t worker_id;
 } swWorkerStopMessage;
 
-//-----------------------------------Factory--------------------------------------------
-typedef struct
+//------------------------------------Packet-------------------------------------------
+typedef struct _swPacket_task
+{
+    size_t length;
+    char tmpfile[SW_TASK_TMPDIR_SIZE + sizeof(SW_TASK_TMP_FILE)];
+} swPacket_task;
+
+typedef struct _swPacket_response
+{
+    int length;
+    int worker_id;
+} swPacket_response;
+
+typedef struct _swPacket_ptr
 {
     swDataHead info;
     swString data;
-} swPackagePtr;
-
+} swPacket_ptr;
+//-----------------------------------Factory--------------------------------------------
 struct _swFactory
 {
     void *object;
     void *ptr; //server object
-    int last_from_id;
-
-    swReactor *reactor; //reserve for reactor
 
     int (*start)(struct _swFactory *);
     int (*shutdown)(struct _swFactory *);
     int (*dispatch)(struct _swFactory *, swSendData *);
+    /**
+     * Returns the number of bytes sent
+     */
     int (*finish)(struct _swFactory *, swSendData *);
     int (*notify)(struct _swFactory *, swDataHead *);    //send a event notify
     int (*end)(struct _swFactory *, int fd);
+    void (*free)(struct _swFactory *);
 };
 
-typedef struct _swFactoryProcess
-{
-    swPipe *pipes;
-} swFactoryProcess;
-
 typedef int (*swServer_dispatch_function)(swServer *, swConnection *, swSendData *);
 
 int swFactory_create(swFactory *factory);
-int swFactory_start(swFactory *factory);
-int swFactory_shutdown(swFactory *factory);
-int swFactory_dispatch(swFactory *factory, swSendData *req);
 int swFactory_finish(swFactory *factory, swSendData *_send);
-int swFactory_notify(swFactory *factory, swDataHead *event);
-int swFactory_end(swFactory *factory, int fd);
 int swFactory_check_callback(swFactory *factory);
 
-int swFactoryProcess_create(swFactory *factory, int worker_num);
+int swFactoryProcess_create(swFactory *factory, uint32_t worker_num);
 
 //------------------------------------Server-------------------------------------------
-enum swServer_callback_type
-{
-    SW_SERVER_CALLBACK_onConnect = 1,
-    SW_SERVER_CALLBACK_onReceive,
-    SW_SERVER_CALLBACK_onClose,
-};
-
 enum swServer_hook_type
 {
     SW_SERVER_HOOK_MASTER_START,
@@ -318,7 +311,7 @@ enum swServer_hook_type
     SW_SERVER_HOOK_PROCESS_TIMER,
 };
 
-typedef struct
+typedef struct _swServerStats
 {
     time_t start_time;
     sw_atomic_t connection_num;
@@ -328,13 +321,14 @@ typedef struct
     sw_atomic_long_t request_count;
 } swServerStats;
 
-typedef struct
+typedef struct _swServerGS
 {
     pid_t master_pid;
     pid_t manager_pid;
 
     uint32_t session_round :24;
-    sw_atomic_t start;  //after swServer_start will set start=1
+    sw_atomic_t start;
+    sw_atomic_t shutdown;
 
     time_t now;
 
@@ -354,7 +348,7 @@ struct _swServer
     /**
      * worker process num
      */
-    uint16_t worker_num;
+    uint32_t worker_num;
     /**
      * The number of pipe per reactor maintenance
      */
@@ -386,6 +380,7 @@ struct _swServer
      * worker process max request
      */
     uint32_t max_request;
+    uint32_t max_request_grace;
 
     int udp_socket_ipv4;
     int udp_socket_ipv6;
@@ -423,14 +418,24 @@ struct _swServer
      * discard the timeout request
      */
     uint32_t discard_timeout_request :1;
+    /**
+     * parse cookie header
+     */
+    uint32_t http_parse_cookie :1;
     /**
      * parse x-www-form-urlencoded data
      */
     uint32_t http_parse_post :1;
+    /**
+     * parse multipart/form-data files to match $_FILES
+     */
+    uint32_t http_parse_files :1;
+#ifdef SW_HAVE_ZLIB
     /**
      * http content compression
      */
     uint32_t http_compression :1;
+#endif
     /**
      * handle static files
      */
@@ -447,14 +452,14 @@ struct _swServer
      * asynchronous reloading
      */
     uint32_t reload_async :1;
+    /**
+     * use task object
+     */
+    uint32_t task_use_object :1;
     /**
      * enable coroutine in task worker
      */
     uint32_t task_enable_coroutine :1;
-    /**
-     * slowlog
-     */
-    uint32_t trace_event_worker :1;
     /**
      * yield coroutine when the output buffer is full
      */
@@ -467,6 +472,11 @@ struct _swServer
      * disable multi-threads
      */
     uint32_t single_thread :1;
+    /**
+     * server status
+     */
+    uint32_t running :1;
+
     /**
      *  heartbeat check time
      */
@@ -476,16 +486,22 @@ struct _swServer
     int *cpu_affinity_available;
     int cpu_affinity_available_num;
 
+    swPipeBuffer **pipe_buffers;
     double send_timeout;
 
     uint16_t listen_port_num;
     time_t reload_time;
     time_t warning_time;
+    long timezone;
+    swTimer_node *master_timer;
+    swTimer_node *heartbeat_timer;
 
     /* buffer output/input setting*/
     uint32_t buffer_output_size;
     uint32_t buffer_input_size;
 
+    uint32_t ipc_max_size;
+
     void *ptr2;
     void *private_data_3;
 
@@ -496,16 +512,17 @@ struct _swServer
     /**
      *  task process
      */
-    uint16_t task_worker_num;
+    uint32_t task_worker_num;
     uint8_t task_ipc_mode;
-    uint16_t task_max_request;
+    uint32_t task_max_request;
+    uint32_t task_max_request_grace;
     swPipe *task_notify;
     swEventData *task_result;
 
     /**
      * user process
      */
-    uint16_t user_worker_num;
+    uint32_t user_worker_num;
     swUserWorker_node *user_worker_list;
     swHashMap *user_worker_map;
     swWorker *user_workers;
@@ -513,6 +530,7 @@ struct _swServer
     swReactorThread *reactor_threads;
     swWorker *workers;
 
+    swLock lock;
     swChannel *message_box;
 
     swServerStats *stats;
@@ -561,14 +579,6 @@ struct _swServer
      * message queue key
      */
     uint64_t message_queue_key;
-    /**
-     * slow request log
-     */
-    uint8_t request_slowlog_timeout;
-    FILE *request_slowlog_file;
-
-    swReactor *reactor_ptr; //Main Reactor
-    swFactory *factory_ptr; //Factory
 
     swLinkedList *hooks[SW_MAX_HOOK_TYPE];
 
@@ -600,7 +610,7 @@ struct _swServer
      * Server method
      */
     int (*send)(swServer *serv, int session_id, void *data, uint32_t length);
-    int (*sendfile)(swServer *serv, int session_id, char *file, uint32_t l_file, off_t offset, size_t length);
+    int (*sendfile)(swServer *serv, int session_id, const char *file, uint32_t l_file, off_t offset, size_t length);
     int (*sendwait)(swServer *serv, int session_id, void *data, uint32_t length);
     int (*close)(swServer *serv, int session_id, int reset);
     int (*notify)(swServer *serv, swConnection *conn, int event);
@@ -609,18 +619,6 @@ struct _swServer
     int (*dispatch_func)(swServer *, swConnection *, swSendData *);
 };
 
-typedef struct
-{
-    size_t length;
-    char tmpfile[SW_TASK_TMPDIR_SIZE + sizeof(SW_TASK_TMP_FILE)];
-} swPackage_task;
-
-typedef struct
-{
-	int length;
-	int worker_id;
-} swPackage_response;
-
 int swServer_master_onAccept(swReactor *reactor, swEvent *event);
 void swServer_master_onTimer(swTimer *timer, swTimer_node *tnode);
 int swServer_master_send(swServer *serv, swSendData *_send);
@@ -631,7 +629,7 @@ int swServer_onFinish2(swFactory *factory, swSendData *resp);
 void swServer_init(swServer *serv);
 void swServer_signal_init(swServer *serv);
 int swServer_start(swServer *serv);
-swListenPort* swServer_add_port(swServer *serv, int type, const char *host, int port);
+swListenPort* swServer_add_port(swServer *serv, enum swSocket_type type, const char *host, int port);
 void swServer_close_port(swServer *serv, enum swBool_type only_stream_port);
 int swServer_add_worker(swServer *serv, swWorker *worker);
 int swServer_add_systemd_socket(swServer *serv);
@@ -639,42 +637,31 @@ int swServer_add_hook(swServer *serv, enum swServer_hook_type type, swCallback f
 void swServer_call_hook(swServer *serv, enum swServer_hook_type type, void *arg);
 
 int swServer_create(swServer *serv);
-int swServer_free(swServer *serv);
 int swServer_shutdown(swServer *serv);
 
-static sw_inline swString *swServer_get_buffer(swServer *serv, int fd)
-{
-    swString *buffer = serv->connection_list[fd].recv_buffer;
-    if (buffer == NULL)
-    {
-        buffer = swString_new(SW_BUFFER_SIZE_STD);
-        //alloc memory failed.
-        if (!buffer)
-        {
-            return NULL;
-        }
-        serv->connection_list[fd].recv_buffer = buffer;
-    }
-    return buffer;
-}
-
-static sw_inline void swServer_free_buffer(swServer *serv, int fd)
-{
-    swString *buffer = serv->connection_list[fd].recv_buffer;
-    if (buffer)
-    {
-        swString_free(buffer);
-        serv->connection_list[fd].recv_buffer = NULL;
-    }
-}
-
 static sw_inline swListenPort* swServer_get_port(swServer *serv, int fd)
 {
-    sw_atomic_t server_fd = serv->connection_list[fd].from_fd;
+    sw_atomic_t server_fd = serv->connection_list[fd].server_fd;
     return (swListenPort*) serv->connection_list[server_fd].object;
 }
 
-int swServer_udp_send(swServer *serv, swSendData *resp);
+static sw_inline void swServer_lock(swServer *serv)
+{
+    if (serv->single_thread)
+    {
+        return;
+    }
+    serv->lock.lock(&serv->lock);
+}
+
+static sw_inline void swServer_unlock(swServer *serv)
+{
+    if (serv->single_thread)
+    {
+        return;
+    }
+    serv->lock.unlock(&serv->lock);
+}
 
 #define SW_MAX_SESSION_ID             0x1000000
 
@@ -682,9 +669,7 @@ static sw_inline int swEventData_is_dgram(uint8_t type)
 {
     switch (type)
     {
-    case SW_EVENT_UDP:
-    case SW_EVENT_UDP6:
-    case SW_EVENT_UNIX_DGRAM:
+    case SW_SERVER_EVENT_SNED_DGRAM:
         return SW_TRUE;
     default:
         return SW_FALSE;
@@ -695,15 +680,13 @@ static sw_inline int swEventData_is_stream(uint8_t type)
 {
     switch (type)
     {
-    case SW_EVENT_TCP:
-    case SW_EVENT_TCP6:
-    case SW_EVENT_UNIX_STREAM:
-    case SW_EVENT_CONNECT:
-    case SW_EVENT_CLOSE:
-    case SW_EVENT_PAUSE_RECV:
-    case SW_EVENT_RESUME_RECV:
-    case SW_EVENT_BUFFER_FULL:
-    case SW_EVENT_BUFFER_EMPTY:
+    case SW_SERVER_EVENT_SEND_DATA:
+    case SW_SERVER_EVENT_CONNECT:
+    case SW_SERVER_EVENT_CLOSE:
+    case SW_SERVER_EVENT_PAUSE_RECV:
+    case SW_SERVER_EVENT_RESUME_RECV:
+    case SW_SERVER_EVENT_BUFFER_FULL:
+    case SW_SERVER_EVENT_BUFFER_EMPTY:
         return SW_TRUE;
     default:
         return SW_FALSE;
@@ -729,20 +712,20 @@ int swTaskWorker_onTask(swProcessPool *pool, swEventData *task);
 int swTaskWorker_onFinish(swReactor *reactor, swEvent *event);
 void swTaskWorker_onStart(swProcessPool *pool, int worker_id);
 void swTaskWorker_onStop(swProcessPool *pool, int worker_id);
-int swTaskWorker_large_pack(swEventData *task, void *data, int data_len);
-int swTaskWorker_finish(swServer *serv, char *data, int data_len, int flags, swEventData *current_task);
+int swTaskWorker_large_pack(swEventData *task, const void *data, size_t data_len);
+int swTaskWorker_finish(swServer *serv, const char *data, size_t data_len, int flags, swEventData *current_task);
 
-#define swTask_type(task)                  ((task)->info.from_fd)
+#define swTask_type(task)                  ((task)->info.server_fd)
 
 static sw_inline swString* swTaskWorker_large_unpack(swEventData *task_result)
 {
-    swPackage_task _pkg;
+    swPacket_task _pkg;
     memcpy(&_pkg, task_result->data, sizeof(_pkg));
 
     int tmp_file_fd = open(_pkg.tmpfile, O_RDONLY);
     if (tmp_file_fd < 0)
     {
-        swSysError("open(%s) failed.", _pkg.tmpfile);
+        swSysWarn("open(%s) failed", _pkg.tmpfile);
         return NULL;
     }
     if (SwooleTG.buffer_stack->size < _pkg.length && swString_extend_align(SwooleTG.buffer_stack, _pkg.length) < 0)
@@ -764,17 +747,13 @@ static sw_inline swString* swTaskWorker_large_unpack(swEventData *task_result)
     return SwooleTG.buffer_stack;
 }
 
-#define swPackage_data(task) ((task->info.type==SW_EVENT_PACKAGE_END)?SwooleWG.buffer_input[task->info.from_id]->str:task->data)
-#define swPackage_length(task) ((task->info.type==SW_EVENT_PACKAGE_END)?SwooleWG.buffer_input[task->info.from_id]->length:task->info.len)
-
 #define SW_SERVER_MAX_FD_INDEX          0 //max connection socket
 #define SW_SERVER_MIN_FD_INDEX          1 //min listen socket
-#define SW_SERVER_TIMER_FD_INDEX        2 //for timerfd
 
-//使用connection_list[0]表示最大的FD
+// connection_list[0] => the largest fd
 #define swServer_set_maxfd(serv,maxfd) (serv->connection_list[SW_SERVER_MAX_FD_INDEX].fd=maxfd)
 #define swServer_get_maxfd(serv) (serv->connection_list[SW_SERVER_MAX_FD_INDEX].fd)
-//使用connection_list[1]表示最小的FD
+// connection_list[1] => the smallest fd
 #define swServer_set_minfd(serv,maxfd) (serv->connection_list[SW_SERVER_MIN_FD_INDEX].fd=maxfd)
 #define swServer_get_minfd(serv) (serv->connection_list[SW_SERVER_MIN_FD_INDEX].fd)
 
@@ -782,7 +761,7 @@ static sw_inline swString* swTaskWorker_large_unpack(swEventData *task_result)
 
 static sw_inline swConnection* swServer_connection_get(swServer *serv, int fd)
 {
-    if (fd <= 2 || (uint32_t) fd > serv->max_connection)
+    if ((uint32_t) fd > serv->max_connection)
     {
         return NULL;
     }
@@ -811,14 +790,14 @@ static sw_inline swWorker* swServer_get_worker(swServer *serv, uint16_t worker_i
     }
 
     //Task Worker
-    uint16_t task_worker_max = serv->task_worker_num + serv->worker_num;
+    uint32_t task_worker_max = serv->task_worker_num + serv->worker_num;
     if (worker_id < task_worker_max)
     {
         return &(serv->gs->task_workers.workers[worker_id - serv->worker_num]);
     }
 
     //User Worker
-    uint16_t user_worker_max = task_worker_max + serv->user_worker_num;
+    uint32_t user_worker_max = task_worker_max + serv->user_worker_num;
     if (worker_id < user_worker_max)
     {
         return &(serv->user_workers[worker_id - task_worker_max]);
@@ -829,7 +808,16 @@ static sw_inline swWorker* swServer_get_worker(swServer *serv, uint16_t worker_i
 
 static sw_inline int swServer_worker_schedule(swServer *serv, int fd, swSendData *data)
 {
-    uint32_t key;
+    uint32_t key = 0;
+
+    if (serv->dispatch_func)
+    {
+        int id = serv->dispatch_func(serv, swServer_connection_get(serv, fd), data);
+        if (id != SW_DISPATCH_RESULT_USERFUNC_FALLBACK)
+        {
+            return id;
+        }
+    }
 
     //polling mode
     if (serv->dispatch_mode == SW_DISPATCH_ROUND)
@@ -861,7 +849,7 @@ static sw_inline int swServer_worker_schedule(swServer *serv, int fd, swSendData
 #ifdef HAVE_KQUEUE
             key = *(((uint32_t *) &conn->info.addr.inet_v6.sin6_addr) + 3);
 #elif defined(_WIN32)
-			key = conn->info.addr.inet_v6.sin6_addr.u.Word[3];
+            key = conn->info.addr.inet_v6.sin6_addr.u.Word[3];
 #else
             key = conn->info.addr.inet_v6.sin6_addr.s6_addr32[3];
 #endif
@@ -879,16 +867,11 @@ static sw_inline int swServer_worker_schedule(swServer *serv, int fd, swSendData
             key = conn->uid;
         }
     }
-    //schedule by dispatch function
-    else if (serv->dispatch_mode == SW_DISPATCH_USERFUNC)
-    {
-        return serv->dispatch_func(serv, swServer_connection_get(serv, fd), data);
-    }
     //Preemptive distribution
     else
     {
-        int i;
-        int found = 0;
+        uint32_t i;
+        uint8_t found = 0;
         for (i = 0; i < serv->worker_num + 1; i++)
         {
             key = sw_atomic_fetch_add(&serv->worker_round_id, 1) % serv->worker_num;
@@ -898,7 +881,7 @@ static sw_inline int swServer_worker_schedule(swServer *serv, int fd, swSendData
                 break;
             }
         }
-        if (unlikely(found == 0))
+        if (sw_unlikely(found == 0))
         {
             serv->scheduler_warning = 1;
         }
@@ -933,18 +916,18 @@ static sw_inline swString *swWorker_get_buffer(swServer *serv, int reactor_id)
     }
 }
 
-static sw_inline size_t swWorker_get_data(swEventData *req, char **data_ptr)
+static sw_inline size_t swWorker_get_data(swServer *serv, swEventData *req, char **data_ptr)
 {
     size_t length;
     if (req->info.flags & SW_EVENT_DATA_PTR)
     {
-        swPackagePtr *task = (swPackagePtr *) req;
+        swPacket_ptr *task = (swPacket_ptr *) req;
         *data_ptr = task->data.str;
         length = task->data.length;
     }
     else if (req->info.flags & SW_EVENT_DATA_END)
     {
-        swString *worker_buffer = swWorker_get_buffer(SwooleG.serv, req->info.from_id);
+        swString *worker_buffer = swWorker_get_buffer(serv, req->info.reactor_id);
         *data_ptr = worker_buffer->str;
         length = worker_buffer->length;
     }
@@ -980,7 +963,7 @@ static sw_inline swConnection *swServer_connection_verify(swServer *serv, int se
     {
         return NULL;
     }
-    if (conn->ssl && conn->ssl_state != SW_SSL_STATE_READY)
+    if (conn->ssl && !conn->ssl_ready)
     {
         swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SSL_NOT_READY, "SSL not ready");
         return NULL;
@@ -989,30 +972,41 @@ static sw_inline swConnection *swServer_connection_verify(swServer *serv, int se
     return conn;
 }
 
-void swPort_init(swListenPort *port);
-void swPort_free(swListenPort *port);
-void swPort_set_protocol(swListenPort *ls);
-int swPort_listen(swListenPort *ls);
+static sw_inline int swServer_connection_incoming(swServer *serv, swReactor *reactor, swConnection *conn)
+{
 #ifdef SW_USE_OPENSSL
-int swPort_enable_ssl_encrypt(swListenPort *ls);
+    if (conn->socket->ssl)
+    {
+        return reactor->add(reactor, conn->fd, SW_FD_SESSION | SW_EVENT_READ);
+    }
 #endif
-void swPort_clear_protocol(swListenPort *ls);
+    //delay receive, wait resume command.
+    if (serv->enable_delay_receive)
+    {
+        conn->socket->listen_wait = 1;
+        return SW_OK;
+    }
+    if (reactor->add(reactor, conn->fd, SW_FD_SESSION | SW_EVENT_READ) < 0)
+    {
+        return SW_ERR;
+    }
+    //notify worker process
+    if (serv->onConnect)
+    {
+        return serv->notify(serv, conn, SW_SERVER_EVENT_CONNECT);
+    }
+    else
+    {
+        return SW_OK;
+    }
+}
 
-void swWorker_free(swWorker *worker);
-void swWorker_onStart(swServer *serv);
-void swWorker_onStop(swServer *serv);
-void swWorker_try_to_exit();
-int swWorker_loop(swFactory *factory, int worker_pti);
-int swWorker_send2reactor(swServer *serv, swEventData *ev_data, size_t sendn, int fd);
-int swWorker_send2worker(swWorker *dst_worker, void *buf, int n, int flag);
-void swWorker_signal_handler(int signo);
-void swWorker_signal_init(void);
-void swWorker_clean(void);
+void swServer_connection_each(swServer *serv, void (*callback)(swConnection *conn));
 
 /**
  * reactor_id: The fd in which the reactor.
  */
-static sw_inline int swWorker_get_send_pipe(swServer *serv, int session_id, int reactor_id)
+static sw_inline int swServer_get_send_pipe(swServer *serv, int session_id, int reactor_id)
 {
     int pipe_index = session_id % serv->reactor_pipe_num;
     /**
@@ -1023,25 +1017,56 @@ static sw_inline int swWorker_get_send_pipe(swServer *serv, int session_id, int
     return worker->pipe_worker;
 }
 
+static sw_inline uint8_t swServer_support_unsafe_events(swServer *serv)
+{
+    return !serv->enable_unsafe_event && serv->dispatch_mode != SW_DISPATCH_ROUND && serv->dispatch_mode != SW_DISPATCH_QUEUE && serv->dispatch_mode != SW_DISPATCH_STREAM;
+}
+
+static sw_inline uint8_t swServer_dispatch_mode_is_mod(swServer *serv)
+{
+    return serv->dispatch_mode == SW_DISPATCH_FDMOD || serv->dispatch_mode == SW_DISPATCH_IPMOD;
+}
+
+#define swServer_support_send_yield swServer_dispatch_mode_is_mod
+
+//------------------------------------Listen Port-------------------------------------------
+void swPort_init(swListenPort *port);
+void swPort_free(swListenPort *port);
+int swPort_listen(swListenPort *ls);
+void swPort_set_protocol(swServer *serv, swListenPort *ls);
+#ifdef SW_USE_OPENSSL
+int swPort_enable_ssl_encrypt(swListenPort *ls);
+#endif
+void swPort_clear_protocol(swListenPort *ls);
+//------------------------------------Worker Process-------------------------------------------
+void swWorker_onStart(swServer *serv);
+void swWorker_onStop(swServer *serv);
+int swWorker_loop(swServer *serv, int worker_pti);
+void swWorker_clean_pipe_buffer(swServer *serv);
+int swWorker_send2reactor(swServer *serv, swEventData *ev_data, size_t sendn, int session_id);
+int swWorker_send2worker(swWorker *dst_worker, const void *buf, int n, int flag);
+void swWorker_signal_handler(int signo);
+void swWorker_signal_init(void);
+
 int swReactorThread_create(swServer *serv);
 int swReactorThread_start(swServer *serv);
 void swReactorThread_set_protocol(swServer *serv, swReactor *reactor);
+void swReactorThread_join(swServer *serv);
 void swReactorThread_free(swServer *serv);
 int swReactorThread_close(swReactor *reactor, int fd);
-int swReactorThread_dispatch(swConnection *conn, char *data, uint32_t length);
+int swReactorThread_dispatch(swProtocol *proto, swSocket *_socket, char *data, uint32_t length);
 int swReactorThread_send2worker(swServer *serv, swWorker *worker, void *data, int len);
 
 int swReactorProcess_create(swServer *serv);
 int swReactorProcess_start(swServer *serv);
+void swReactorProcess_free(swServer *serv);
 
-int swManager_start(swFactory *factory);
+int swManager_start(swServer *serv);
 pid_t swManager_spawn_user_worker(swServer *serv, swWorker* worker);
 pid_t swManager_spawn_task_worker(swServer *serv, swWorker* worker);
 int swManager_wait_other_worker(swProcessPool *pool, pid_t pid, int status);
 void swManager_kill_user_worker(swServer *serv);
 
-#ifdef __cplusplus
-}
-#endif
+SW_EXTERN_C_END
 
 #endif /* SW_SERVER_H_ */
