@@ -17,13 +17,10 @@
 #ifndef SW_WEBSOCKET_H_
 #define SW_WEBSOCKET_H_
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
-
 #include "http.h"
 
+SW_EXTERN_C_BEGIN
+
 #define SW_WEBSOCKET_GUID "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"
 #define SW_WEBSOCKET_HEADER_LEN  2
 #define SW_WEBSOCKET_MASK_LEN    4
@@ -43,6 +40,7 @@ extern "C"
 
 enum swWebsocketStatus
 {
+    WEBSOCKET_STATUS_NONE = 0,
     WEBSOCKET_STATUS_CONNECTION = 1,
     WEBSOCKET_STATUS_HANDSHAKE = 2,
     WEBSOCKET_STATUS_ACTIVE = 3,
@@ -70,7 +68,9 @@ typedef struct
     char *payload;
 } swWebSocket_frame;
 
-enum swWebsocketCode
+#define WEBSOCKET_VERSION    13
+
+enum swWebsocket_opcode
 {
     WEBSOCKET_OPCODE_CONTINUATION = 0x0,
     WEBSOCKET_OPCODE_TEXT = 0x1,
@@ -78,7 +78,10 @@ enum swWebsocketCode
     WEBSOCKET_OPCODE_CLOSE = 0x8,
     WEBSOCKET_OPCODE_PING = 0x9,
     WEBSOCKET_OPCODE_PONG = 0xa,
+};
 
+enum swWebsocket_close_reason
+{
     WEBSOCKET_CLOSE_NORMAL = 1000,
     WEBSOCKET_CLOSE_GOING_AWAY = 1001,
     WEBSOCKET_CLOSE_PROTOCOL_ERROR = 1002,
@@ -91,18 +94,16 @@ enum swWebsocketCode
     WEBSOCKET_CLOSE_EXTENSION_MISSING = 1010,
     WEBSOCKET_CLOSE_SERVER_ERROR = 1011,
     WEBSOCKET_CLOSE_TLS = 1015,
-    WEBSOCKET_VERSION = 13,
 };
 
-int swWebSocket_get_package_length(swProtocol *protocol, swConnection *conn, char *data, uint32_t length);
-void swWebSocket_encode(swString *buffer, char *data, size_t length, char opcode, uint8_t finish, uint8_t mask);
+void swWebSocket_encode(swString *buffer, const char *data, size_t length, char opcode, uint8_t finish, uint8_t mask);
 void swWebSocket_decode(swWebSocket_frame *frame, swString *data);
 int swWebSocket_pack_close_frame(swString *buffer, int code, char* reason, size_t length, uint8_t mask);
 void swWebSocket_print_frame(swWebSocket_frame *frame);
-int swWebSocket_dispatch_frame(swConnection *conn, char *data, uint32_t length);
 
-#ifdef __cplusplus
-}
-#endif
+ssize_t swWebSocket_get_package_length(swProtocol *protocol, swSocket *conn, char *data, uint32_t length);
+int swWebSocket_dispatch_frame(swProtocol *protocol, swSocket *conn, char *data, uint32_t length);
+
+SW_EXTERN_C_END
 
 #endif /* SW_WEBSOCKET_H_ */
