@@ -14,37 +14,40 @@
  +----------------------------------------------------------------------+
  */
 
-#include "swoole.h"
+#include "swoole_api.h"
 #include "atomic.h"
 #include "async.h"
 #include "coroutine_c_api.h"
 
 #include <stdarg.h>
 
-#ifndef _WIN32
 #include <sys/stat.h>
 #include <sys/resource.h>
 #include <sys/ioctl.h>
-#endif
 
 #ifdef HAVE_EXECINFO
 #include <execinfo.h>
 #endif
 
-#ifdef __sun
-#include <sys/filio.h>
-#endif
-
 swGlobal_t SwooleG;
-swGlobalS_t *SwooleGS;
-__thread swThreadGlobal_t SwooleTG;
 swWorkerGlobal_t SwooleWG;
+__thread swThreadGlobal_t SwooleTG;
+
+#ifdef __MACH__
+static __thread char _sw_error_buf[SW_ERROR_MSG_SIZE];
+char* sw_error_()
+{
+    return _sw_error_buf;
+}
+#else
+__thread char sw_error[SW_ERROR_MSG_SIZE];
+#endif
 
 static void swoole_fatal_error(int code, const char *format, ...);
 
 void swoole_init(void)
 {
-    if (SwooleG.running)
+    if (SwooleG.init)
     {
         return;
     }
@@ -54,25 +57,19 @@ void swoole_init(void)
     bzero(sw_error, SW_ERROR_MSG_SIZE);
 
     SwooleG.running = 1;
+    SwooleG.init = 1;
     SwooleG.enable_coroutine = 1;
 
     SwooleG.log_fd = STDOUT_FILENO;
     SwooleG.write_log = swLog_put;
     SwooleG.fatal_error = swoole_fatal_error;
 
-#ifdef _WIN32
-    SYSTEM_INFO info;
-    GetSystemInfo(&info);
-    SwooleG.cpu_num =  info.dwNumberOfProcessors;
-    SwooleG.pagesize = info.dwPageSize;
-#else
-    SwooleG.cpu_num = sysconf(_SC_NPROCESSORS_ONLN);
+    SwooleG.cpu_num = SW_MAX(1, sysconf(_SC_NPROCESSORS_ONLN));
     SwooleG.pagesize = getpagesize();
     //get system uname
     uname(&SwooleG.uname);
     //random seed
     srandom(time(NULL));
-#endif
 
     SwooleG.pid = getpid();
 
@@ -87,23 +84,17 @@ void swoole_init(void)
     SwooleG.memory_pool = swMemoryGlobal_new(SW_GLOBAL_MEMORY_PAGESIZE, 1);
     if (SwooleG.memory_pool == NULL)
     {
-        printf("[Master] Fatal Error: global memory allocation failure");
+        printf("[Core] Fatal Error: global memory allocation failure");
         exit(1);
     }
-    SwooleGS = SwooleG.memory_pool->alloc(SwooleG.memory_pool, sizeof(swGlobalS_t));
-    if (SwooleGS == NULL)
+
+    if (swMutex_create(&SwooleG.lock, 0) < 0)
     {
-        printf("[Master] Fatal Error: failed to allocate memory for SwooleGS");
-        exit(2);
+        printf("[Core] mutex init failure");
+        exit(1);
     }
 
-    //init global lock
-    swMutex_create(&SwooleGS->lock, 1);
-    swMutex_create(&SwooleGS->lock_2, 1);
-    swMutex_create(&SwooleG.lock, 0);
-
     SwooleG.max_sockets = 1024;
-#ifndef _WIN32
     struct rlimit rlmt;
     if (getrlimit(RLIMIT_NOFILE, &rlmt) < 0)
     {
@@ -114,9 +105,14 @@ void swoole_init(void)
         SwooleG.max_sockets = MAX((uint32_t) rlmt.rlim_cur, 1024);
         SwooleG.max_sockets = MIN((uint32_t) rlmt.rlim_cur, SW_SESSION_LIST_SIZE);
     }
-#endif
 
     SwooleG.socket_buffer_size = SW_SOCKET_BUFFER_SIZE;
+    SwooleG.socket_array = swArray_new(1024, sizeof(swSocket));
+    if (!SwooleG.socket_array)
+    {
+        swSysWarn("[Core] Fatal Error: socket array memory allocation failure");
+        exit(1);
+    }
 
     SwooleTG.buffer_stack = swString_new(SW_STACK_BUFFER_SIZE);
     if (SwooleTG.buffer_stack == NULL)
@@ -151,65 +147,86 @@ void swoole_init(void)
 
 void swoole_clean(void)
 {
+    if (SwooleG.task_tmpdir)
+    {
+        sw_free(SwooleG.task_tmpdir);
+    }
+    if (SwooleTG.timer)
+    {
+        swoole_timer_free();
+    }
+    if (SwooleTG.reactor)
+    {
+        swoole_event_free();
+    }
+    if (SwooleG.socket_array)
+    {
+        swArray_free(SwooleG.socket_array);
+    }
     //free the global memory
     if (SwooleG.memory_pool != NULL)
     {
-        if (SwooleG.timer.initialized)
-        {
-            swTimer_free(&SwooleG.timer);
-        }
-        if (SwooleG.task_tmpdir)
-        {
-            sw_free(SwooleG.task_tmpdir);
-        }
-        if (SwooleG.main_reactor)
-        {
-            SwooleG.main_reactor->free(SwooleG.main_reactor);
-        }
         SwooleG.memory_pool->destroy(SwooleG.memory_pool);
-        bzero(&SwooleG, sizeof(SwooleG));
     }
+    bzero(&SwooleG, sizeof(SwooleG));
 }
 
-pid_t swoole_fork()
+pid_t swoole_fork(int flags)
 {
-    if (swoole_coroutine_is_in())
+    if (!(flags & SW_FORK_EXEC))
     {
-        swFatalError(SW_ERROR_OPERATION_NOT_SUPPORT, "must be forked outside the coroutine");
-        return -1;
-    }
-    if (SwooleAIO.init)
-    {
-        swError("can not create server after using async file operation");
-        return -1;
+        if (swoole_coroutine_is_in())
+        {
+            swFatalError(SW_ERROR_OPERATION_NOT_SUPPORT, "must be forked outside the coroutine");
+            return -1;
+        }
+        if (SwooleTG.aio_init)
+        {
+            swError("can not create server after using async file operation");
+            return -1;
+        }
     }
+
     pid_t pid = fork();
     if (pid == 0)
     {
         /**
          * [!!!] All timers and event loops must be cleaned up after fork
          */
-        if (SwooleG.timer.initialized)
+        if (SwooleTG.timer)
         {
-            swTimer_free(&SwooleG.timer);
+            swoole_timer_free();
         }
-        /**
-         * reset SwooleG.memory_pool
-         */
-        SwooleG.memory_pool = swMemoryGlobal_new(SW_GLOBAL_MEMORY_PAGESIZE, 1);
-        if (SwooleG.memory_pool == NULL)
+        if (!(flags & SW_FORK_EXEC))
         {
-            printf("[Worker] Fatal Error: global memory allocation failure");
-            exit(1);
+            /**
+             * reset SwooleG.memory_pool
+             */
+            SwooleG.memory_pool = swMemoryGlobal_new(SW_GLOBAL_MEMORY_PAGESIZE, 1);
+            if (SwooleG.memory_pool == NULL)
+            {
+                printf("[Worker] Fatal Error: global memory allocation failure");
+                exit(1);
+            }
+            /**
+             * reopen log file
+             */
+            swLog_reopen(0);
+            /**
+             * reset eventLoop
+             */
+            if (SwooleTG.reactor)
+            {
+                swoole_event_free();
+                swTraceLog(SW_TRACE_REACTOR, "reactor has been destroyed");
+            }
         }
-        /**
-         * reset eventLoop
-         */
-        if (SwooleG.main_reactor)
+        else
         {
-            SwooleG.main_reactor->free(SwooleG.main_reactor);
-            SwooleG.main_reactor = NULL;
-            swTraceLog(SW_TRACE_PHP, "destroy reactor");
+            /**
+             * close log fd
+             */
+            swLog_free();
         }
         /**
          * reset signal handler
@@ -236,7 +253,7 @@ uint64_t swoole_hash_key(char *str, int str_len)
     return hash;
 }
 
-void swoole_dump_ascii(char *data, int size)
+void swoole_dump_ascii(const char *data, size_t size)
 {
     int i;
     for (i = 0; i < size; i++)
@@ -246,7 +263,7 @@ void swoole_dump_ascii(char *data, int size)
     printf("\n");
 }
 
-void swoole_dump_bin(char *data, char type, int size)
+void swoole_dump_bin(const char *data, char type, size_t size)
 {
     int i;
     int type_size = swoole_type_size(type);
@@ -263,7 +280,7 @@ void swoole_dump_bin(char *data, char type, int size)
     printf("\n");
 }
 
-void swoole_dump_hex(char *data, int outlen)
+void swoole_dump_hex(const char *data, size_t outlen)
 {
     long i;
     for (i = 0; i < outlen; ++i)
@@ -423,7 +440,7 @@ size_t swoole_sync_writefile(int fd, const void *data, size_t len)
             {
                 continue;
             }
-            swSysWarn("write(%d, %d) failed", fd, towrite);
+            swSysWarn("write(%d, %zu) failed", fd, towrite);
             break;
         }
     }
@@ -599,7 +616,7 @@ long swoole_file_get_size(FILE *fp)
     return size;
 }
 
-long swoole_file_size(char *filename)
+long swoole_file_size(const char *filename)
 {
     struct stat file_stat;
     if (lstat(filename, &file_stat) < 0)
@@ -616,7 +633,7 @@ long swoole_file_size(char *filename)
     return file_stat.st_size;
 }
 
-swString* swoole_file_get_contents(char *filename)
+swString* swoole_file_get_contents(const char *filename)
 {
     long filesize = swoole_file_size(filename);
     if (filesize < 0)
@@ -674,7 +691,7 @@ swString* swoole_file_get_contents(char *filename)
     return content;
 }
 
-int swoole_file_put_contents(char *filename, char *content, size_t length)
+int swoole_file_put_contents(const char *filename, const char *content, size_t length)
 {
     if (length <= 0)
     {
@@ -828,25 +845,18 @@ uint32_t swoole_common_multiple(uint32_t u, uint32_t v)
     return u * v / n_cup;
 }
 
-/**
- * for GDB
- */
-void swBreakPoint() { }
-
 size_t sw_snprintf(char *buf, size_t size, const char *format, ...)
 {
-    size_t retval;
     va_list args;
-
     va_start(args, format);
-    retval = vsnprintf(buf, size, format, args);
+    int retval = vsnprintf(buf, size, format, args);
     va_end(args);
-    if (unlikely(retval < 0))
+    if (sw_unlikely(retval < 0))
     {
         retval = 0;
         buf[0] = '\0';
     }
-    else if (unlikely(retval >= size))
+    else if (sw_unlikely(retval >= size))
     {
         retval = size - 1;
         buf[retval] = '\0';
@@ -856,14 +866,13 @@ size_t sw_snprintf(char *buf, size_t size, const char *format, ...)
 
 size_t sw_vsnprintf(char *buf, size_t size, const char *format, va_list args)
 {
-    size_t retval;
-    retval = vsnprintf(buf, size, format, args);
-    if (unlikely(retval < 0))
+    int retval = vsnprintf(buf, size, format, args);
+    if (sw_unlikely(retval < 0))
     {
         retval = 0;
         buf[0] = '\0';
     }
-    else if (unlikely(retval >= size))
+    else if (sw_unlikely(retval >= size))
     {
         retval = size - 1;
         buf[retval] = '\0';
@@ -871,7 +880,7 @@ size_t sw_vsnprintf(char *buf, size_t size, const char *format, va_list args)
     return retval;
 }
 
-void swoole_ioctl_set_block(int sock, int nonblock)
+int swoole_ioctl_set_block(int sock, int nonblock)
 {
     int ret;
     do
@@ -883,10 +892,15 @@ void swoole_ioctl_set_block(int sock, int nonblock)
     if (ret < 0)
     {
         swSysWarn("ioctl(%d, FIONBIO, %d) failed", sock, nonblock);
+        return SW_ERR;
+    }
+    else
+    {
+        return SW_OK;
     }
 }
 
-void swoole_fcntl_set_option(int sock, int nonblock, int cloexec)
+int swoole_fcntl_set_option(int sock, int nonblock, int cloexec)
 {
     int opts, ret;
 
@@ -921,6 +935,7 @@ void swoole_fcntl_set_option(int sock, int nonblock, int cloexec)
         if (ret < 0)
         {
             swSysWarn("fcntl(%d, SETFL, opts) failed", sock);
+            return SW_ERR;
         }
     }
 
@@ -956,9 +971,11 @@ void swoole_fcntl_set_option(int sock, int nonblock, int cloexec)
         if (ret < 0)
         {
             swSysWarn("fcntl(%d, SETFD, opts) failed", sock);
+            return SW_ERR;
         }
     }
 #endif
+    return SW_OK;
 }
 
 static int *swoole_kmp_borders(char *needle, size_t nlen)
@@ -1081,7 +1098,7 @@ char *swoole_kmp_strnstr(char *haystack, char *needle, uint32_t length)
  * DNS lookup
  */
 #ifdef HAVE_GETHOSTBYNAME2_R
-int swoole_gethostbyname(int flags, char *name, char *addr)
+int swoole_gethostbyname(int flags, const char *name, char *addr)
 {
     int __af = flags & (~SW_DNS_LOOKUP_RANDOM);
     int index = 0;
@@ -1091,6 +1108,10 @@ int swoole_gethostbyname(int flags, char *name, char *addr)
     struct hostent *result;
 
     char *buf = (char*) sw_malloc(buf_len);
+    if (!buf)
+    {
+        return SW_ERR;
+    }
     memset(buf, 0, buf_len);
     while ((rc = gethostbyname2_r(name, __af, &hbuf, buf, buf_len, &result, &err)) == ERANGE)
     {
@@ -1149,9 +1170,9 @@ int swoole_gethostbyname(int flags, char *name, char *addr)
     return SW_OK;
 }
 #else
-int swoole_gethostbyname(int flags, char *name, char *addr)
+int swoole_gethostbyname(int flags, const char *name, char *addr)
 {
-	int __af = flags & (~SW_DNS_LOOKUP_RANDOM);
+    int __af = flags & (~SW_DNS_LOOKUP_RANDOM);
     int index = 0;
 
     struct hostent *host_entry;
@@ -1258,7 +1279,7 @@ SW_API int swoole_add_function(const char *name, void* func)
     return swHashMap_add(SwooleG.functions, (char *) name, strlen(name), func);
 }
 
-SW_API void* swoole_get_function(char *name, uint32_t length)
+SW_API void* swoole_get_function(const char *name, uint32_t length)
 {
     if (!SwooleG.functions)
     {
@@ -1362,19 +1383,21 @@ int swoole_shell_exec(const char *command, pid_t *pid, uint8_t get_error_stream)
 char* swoole_string_format(size_t n, const char *format, ...)
 {
     char *buf = sw_malloc(n);
-    if (buf)
+    if (!buf)
     {
-        int ret;
-        va_list va_list;
-        va_start(va_list, format);
-        ret = vsnprintf(buf, n, format, va_list);
-        va_end(va_list);
-        if (ret >= 0)
-        {
-            return buf;
-        }
-        sw_free(buf);
+        return NULL;
     }
+
+    int ret;
+    va_list va_list;
+    va_start(va_list, format);
+    ret = vsnprintf(buf, n, format, va_list);
+    va_end(va_list);
+    if (ret >= 0)
+    {
+        return buf;
+    }
+    sw_free(buf);
     return NULL;
 }
 
@@ -1422,12 +1445,23 @@ static void swoole_fatal_error(int code, const char *format, ...)
     size_t retval = 0;
     va_list args;
 
-    SwooleGS->lock_2.lock(&SwooleGS->lock_2);
     retval += sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, "(ERROR %d): ", code);
     va_start(args, format);
     retval += sw_vsnprintf(sw_error + retval, SW_ERROR_MSG_SIZE - retval, format, args);
     va_end(args);
     SwooleG.write_log(SW_LOG_ERROR, sw_error, retval);
-    SwooleGS->lock_2.unlock(&SwooleGS->lock_2);
-    exit(255);
+    exit(1);
+}
+
+void swDataHead_dump(const swDataHead *data)
+{
+    printf("swDataHead[%p]\n"
+            "{\n"
+            "    int fd = %d;\n"
+            "    uint32_t len = %d;\n"
+            "    int16_t reactor_id = %d;\n"
+            "    uint8_t type = %d;\n"
+            "    uint8_t flags = %d;\n"
+            "    uint16_t server_fd = %d;\n"
+            "}\n", data, data->fd, data->len, data->reactor_id, data->type, data->flags, data->server_fd);
 }
