@@ -16,74 +16,81 @@
 
 #include "coroutine.h"
 #include "coroutine_c_api.h"
-#include "async.h"
 
 using namespace swoole;
 
-size_t Coroutine::stack_size = SW_DEFAULT_C_STACK_SIZE;
-Coroutine* Coroutine::current = nullptr;
-long Coroutine::last_cid = 0;
-uint64_t Coroutine::peak_num = 0;
-coro_php_yield_t  Coroutine::on_yield = nullptr;
-coro_php_resume_t Coroutine::on_resume = nullptr;
-coro_php_close_t  Coroutine::on_close = nullptr;
+sw_co_thread_local Coroutine* Coroutine::current = nullptr;
+sw_co_thread_local long Coroutine::last_cid = 0;
+sw_co_thread_local std::unordered_map<long, Coroutine*> Coroutine::coroutines;
+sw_co_thread_local uint64_t Coroutine::peak_num = 0;
 
-std::unordered_map<long, Coroutine*> Coroutine::coroutines;
+size_t Coroutine::stack_size = SW_DEFAULT_C_STACK_SIZE;
+sw_coro_on_swap_t Coroutine::on_yield = nullptr;
+sw_coro_on_swap_t Coroutine::on_resume = nullptr;
+sw_coro_on_swap_t Coroutine::on_close = nullptr;
+sw_coro_bailout_t Coroutine::on_bailout = nullptr;
 
 void Coroutine::yield()
 {
+    SW_ASSERT(current == this || on_bailout != nullptr);
     state = SW_CORO_WAITING;
-    if (on_yield)
+    if (sw_likely(on_yield))
     {
         on_yield(task);
     }
     current = origin;
-    ctx.SwapOut();
+    ctx.swap_out();
 }
 
 void Coroutine::resume()
 {
+    SW_ASSERT(current != this);
+    if (sw_unlikely(on_bailout))
+    {
+        return;
+    }
     state = SW_CORO_RUNNING;
-    if (on_resume)
+    if (sw_likely(on_resume))
     {
         on_resume(task);
     }
     origin = current;
     current = this;
-    ctx.SwapIn();
-    if (ctx.end)
-    {
-        close();
-    }
+    ctx.swap_in();
+    check_end();
 }
 
 void Coroutine::yield_naked()
 {
+    SW_ASSERT(current == this);
     state = SW_CORO_WAITING;
     current = origin;
-    ctx.SwapOut();
+    ctx.swap_out();
 }
 
 void Coroutine::resume_naked()
 {
+    SW_ASSERT(current != this);
+    if (sw_unlikely(on_bailout))
+    {
+        return;
+    }
     state = SW_CORO_RUNNING;
     origin = current;
     current = this;
-    ctx.SwapIn();
-    if (ctx.end)
-    {
-        close();
-    }
+    ctx.swap_in();
+    check_end();
 }
 
 void Coroutine::close()
 {
+    SW_ASSERT(current == this);
     state = SW_CORO_END;
     if (on_close)
     {
         on_close(task);
     }
-#ifndef SW_NO_USE_ASM_CONTEXT
+#ifdef SW_USE_ASM_CONTEXT
     swTraceLog(SW_TRACE_CONTEXT, "coroutine#%ld stack memory use less than %ld bytes", get_cid(), ctx.get_stack_usage());
 #endif
     current = origin;
@@ -110,33 +117,68 @@ void Coroutine::print_list()
             state = "[END]";
             break;
         default:
-            assert(0);
+            abort();
             return;
         }
         printf("Coroutine\t%ld\t%s\n", i->first, state);
     }
 }
 
-void Coroutine::set_on_yield(coro_php_yield_t func)
+void Coroutine::set_on_yield(sw_coro_on_swap_t func)
 {
     on_yield = func;
 }
 
-void Coroutine::set_on_resume(coro_php_resume_t func)
+void Coroutine::set_on_resume(sw_coro_on_swap_t func)
 {
     on_resume = func;
 }
 
-void Coroutine::set_on_close(coro_php_close_t func)
+void Coroutine::set_on_close(sw_coro_on_swap_t func)
 {
     on_close = func;
 }
 
+void Coroutine::bailout(sw_coro_bailout_t func)
+{
+    Coroutine *co = current;
+    if (!co)
+    {
+        // marks that it can no longer resume any coroutine
+        on_bailout = (sw_coro_bailout_t) -1;
+        return;
+    }
+    if (!func)
+    {
+        swError("bailout without bailout function");
+    }
+    if (!co->task)
+    {
+        // TODO: decoupling
+        exit(255);
+    }
+    on_bailout = func;
+    // find the coroutine which is closest to the main
+    while (co->origin)
+    {
+        co = co->origin;
+    }
+    // it will jump to main context directly (it also breaks contexts)
+    co->yield();
+    // expect that never here
+    exit(1);
+}
+
 uint8_t swoole_coroutine_is_in()
 {
     return !!Coroutine::get_current();
 }
 
+long swoole_coroutine_get_current_id()
+{
+    return Coroutine::get_current_cid();
+}
+
 /**
  * for gdb
  */
