@@ -14,7 +14,9 @@
   +----------------------------------------------------------------------+
 */
 
-#include "channel.h"
+#include "coroutine_channel.h"
+
+using swoole::coroutine::Channel;
 
 #include <unordered_map>
 
@@ -54,6 +56,7 @@ void Channel::yield(enum opcode type)
 
 void* Channel::pop(double timeout)
 {
+    Coroutine *current_co = Coroutine::get_current_safe();
     if (closed)
     {
         return nullptr;
@@ -68,15 +71,15 @@ void* Channel::pop(double timeout)
             long msec = (long) (timeout * 1000);
             msg.chan = this;
             msg.type = CONSUMER;
-            msg.co = Coroutine::get_current();
-            msg.timer = swTimer_add(&SwooleG.timer, msec, 0, &msg, timer_callback);
+            msg.co = current_co;
+            msg.timer = swoole_timer_add(msec, SW_FALSE, timer_callback, &msg);
         }
 
         yield(CONSUMER);
 
         if (msg.timer)
         {
-            swTimer_del(&SwooleG.timer, msg.timer);
+            swoole_timer_del(msg.timer);
         }
         if (msg.error || closed)
         {
@@ -101,6 +104,7 @@ void* Channel::pop(double timeout)
 
 bool Channel::push(void *data, double timeout)
 {
+    Coroutine *current_co = Coroutine::get_current_safe();
     if (closed)
     {
         return false;
@@ -115,15 +119,15 @@ bool Channel::push(void *data, double timeout)
             long msec = (long) (timeout * 1000);
             msg.chan = this;
             msg.type = PRODUCER;
-            msg.co = Coroutine::get_current();
-            msg.timer = swTimer_add(&SwooleG.timer, msec, 0, &msg, timer_callback);
+            msg.co = current_co;
+            msg.timer = swoole_timer_add(msec, SW_FALSE, timer_callback, &msg);
         }
 
         yield(PRODUCER);
 
         if (msg.timer)
         {
-            swTimer_del(&SwooleG.timer, msg.timer);
+            swoole_timer_del(msg.timer);
         }
         if (msg.error || closed)
         {
