@@ -14,12 +14,8 @@
   +----------------------------------------------------------------------+
 */
 
-#include "socket.h"
-#include "async.h"
-#include "coroutine.h"
-#include "lru_cache.h"
-
-#ifndef _WIN32
+#include "swoole.h"
+#include "coroutine_cxx_api.h"
 
 #include <sys/file.h>
 #include <sys/types.h>
@@ -30,63 +26,34 @@
 #include <string>
 #include <iostream>
 
-using namespace swoole;
-using namespace std;
+using swoole::Coroutine;
+using swoole::coroutine::Socket;
+using swoole::coroutine::System;
 
-static size_t dns_cache_capacity = 1000;
-static time_t dns_cache_expire = 60;
-static LRUCache *dns_cache = nullptr;
-
-void swoole::set_dns_cache_expire(time_t expire)
-{
-    dns_cache_expire = expire;
-}
-
-void swoole::set_dns_cache_capacity(size_t capacity)
-{
-    dns_cache_capacity = capacity;
-    delete dns_cache;
-    dns_cache = nullptr;
-}
-
-void swoole::clear_dns_cache()
-{
-    if (dns_cache)
-    {
-        dns_cache->clear();
-    }
-}
-
-extern "C"
-{
-struct aio_task
-{
-    Coroutine *co;
-    swAio_event *event;
-};
+SW_EXTERN_C_BEGIN
 
 int swoole_coroutine_socket(int domain, int type, int protocol)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return socket(domain, type, protocol);
     }
     Socket *socket = new Socket(domain, type, protocol);
-    if (unlikely(socket->socket == nullptr))
+    int fd = socket->get_fd();
+    if (sw_unlikely(fd < 0))
     {
         delete socket;
-        return -1;
     }
-    return socket->socket->fd;
+    return socket->get_fd();
 }
 
 ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::send(sockfd, buf, len, flags);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, sockfd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, sockfd);
     if (conn == nullptr)
     {
         goto _no_coro;
@@ -97,11 +64,11 @@ ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags
 
 ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::sendmsg(sockfd, msg, flags);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, sockfd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, sockfd);
     if (conn == nullptr)
     {
         goto _no_coro;
@@ -112,11 +79,11 @@ ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags
 
 ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::recvmsg(sockfd, msg, flags);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, sockfd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, sockfd);
     if (conn == nullptr)
     {
         goto _no_coro;
@@ -127,11 +94,11 @@ ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags)
 
 ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::recv(sockfd, buf, len, flags);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, sockfd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, sockfd);
     if (conn == nullptr)
     {
         goto _no_coro;
@@ -149,11 +116,11 @@ ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags)
 
 int swoole_coroutine_close(int fd)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return close(fd);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, fd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, fd);
     if (conn == nullptr)
     {
         goto _no_coro;
@@ -172,11 +139,11 @@ int swoole_coroutine_close(int fd)
 
 int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return connect(sockfd, addr, addrlen);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, sockfd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, sockfd);
     if (conn == nullptr)
     {
         goto _no_coro;
@@ -185,13 +152,14 @@ int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t
     return socket->connect(addr, addrlen) ? 0 : -1;
 }
 
+#if 1
 int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current() || nfds != 1 || timeout == 0))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current() || nfds != 1 || timeout == 0))
     {
         _poll: return poll(fds, nfds, timeout);
     }
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, fds[0].fd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, fds[0].fd);
     if (conn == nullptr)
     {
         goto _poll;
@@ -208,6 +176,40 @@ int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout)
     }
     return 1;
 }
+#else
+int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout)
+{
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current() || nfds != 1 || timeout == 0))
+    {
+        return poll(fds, nfds, timeout);
+    }
+
+    std::unordered_map<int, swoole::socket_poll_fd> _fds;
+    for(int i = 0; i< nfds; i++)
+    {
+        _fds.emplace(std::make_pair(fds[i].fd, swoole::socket_poll_fd(fds[i].events, &fds[i])));
+    }
+
+    if (!System::socket_poll(_fds, (double)timeout / 1000))
+    {
+        return -1;
+    }
+
+    int retval;
+    for (auto &i : _fds)
+    {
+        int revents = i.second.revents;
+        struct pollfd *_fd = (struct pollfd *) i.second.ptr;
+        _fd->revents = revents;
+        if (revents > 0)
+        {
+            retval++;
+        }
+    }
+
+    return retval;
+}
+#endif
 
 static void handler_access(swAio_event *event)
 {
@@ -289,48 +291,9 @@ static void aio_onCompleted(swAio_event *event)
     ((Coroutine *) event->object)->resume();
 }
 
-static void aio_onReadFileCompleted(swAio_event *event)
-{
-    aio_task *task = (aio_task *) event->object;
-    task->event->buf = event->buf;
-    task->event->nbytes = event->ret;
-    task->event->error = event->error;
-    ((Coroutine *) task->co)->resume();
-}
-
-static void aio_onWriteFileCompleted(swAio_event *event)
-{
-    aio_task *task = (aio_task *) event->object;
-    task->event->ret = event->ret;
-    task->event->error = event->error;
-    ((Coroutine *) task->co)->resume();
-}
-
-static void aio_onDNSCompleted(swAio_event *event)
-{
-    if (event->canceled)
-    {
-        return;
-    }
-    aio_task *task = (aio_task *) event->object;
-    task->event->ret = event->ret;
-    task->event->error = event->error;
-    ((Coroutine *) task->co)->resume();
-}
-
-static void aio_onDNSTimeout(swTimer *timer, swTimer_node *tnode)
-{
-    swAio_event *event = (swAio_event *) tnode->data;
-    event->canceled = 1;
-    aio_task *task = (aio_task *) event->object;
-    task->event->ret = -1;
-    task->event->error = SW_ERROR_DNSLOOKUP_RESOLVE_TIMEOUT;
-    ((Coroutine *) task->co)->resume();
-}
-
 int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return open(pathname, flags, mode);
     }
@@ -345,7 +308,7 @@ int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -356,12 +319,12 @@ int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
 
 ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return read(fd, buf, count);
     }
 
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, fd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, fd);
     if (conn && conn->fdtype == SW_FD_CORO_SOCKET)
     {
         Socket *socket = (Socket *) conn->object;
@@ -378,7 +341,7 @@ ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -389,12 +352,12 @@ ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
 
 ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return write(fd, buf, count);
     }
 
-    swConnection *conn = swReactor_get(SwooleG.main_reactor, fd);
+    swSocket *conn = swReactor_get(SwooleTG.reactor, fd);
     if (conn && conn->fdtype == SW_FD_CORO_SOCKET)
     {
         Socket *socket = (Socket *) conn->object;
@@ -411,7 +374,7 @@ ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -422,7 +385,7 @@ ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
 
 off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return lseek(fd, offset, whence);
     }
@@ -437,7 +400,7 @@ off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -448,7 +411,7 @@ off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
 
 int swoole_coroutine_fstat(int fd, struct stat *statbuf)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return fstat(fd, statbuf);
     }
@@ -462,7 +425,7 @@ int swoole_coroutine_fstat(int fd, struct stat *statbuf)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -473,7 +436,7 @@ int swoole_coroutine_fstat(int fd, struct stat *statbuf)
 
 int swoole_coroutine_unlink(const char *pathname)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return unlink(pathname);
     }
@@ -486,7 +449,7 @@ int swoole_coroutine_unlink(const char *pathname)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -497,7 +460,7 @@ int swoole_coroutine_unlink(const char *pathname)
 
 int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return statvfs(path, buf);
     }
@@ -511,7 +474,7 @@ int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -522,7 +485,7 @@ int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
 
 int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return mkdir(pathname, mode);
     }
@@ -536,7 +499,7 @@ int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -547,7 +510,7 @@ int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
 
 int swoole_coroutine_rmdir(const char *pathname)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return rmdir(pathname);
     }
@@ -560,7 +523,7 @@ int swoole_coroutine_rmdir(const char *pathname)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -571,7 +534,7 @@ int swoole_coroutine_rmdir(const char *pathname)
 
 int swoole_coroutine_rename(const char *oldpath, const char *newpath)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return rename(oldpath, newpath);
     }
@@ -585,7 +548,7 @@ int swoole_coroutine_rename(const char *oldpath, const char *newpath)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -596,7 +559,7 @@ int swoole_coroutine_rename(const char *oldpath, const char *newpath)
 
 int swoole_coroutine_access(const char *pathname, int mode)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return access(pathname, mode);
     }
@@ -610,7 +573,7 @@ int swoole_coroutine_access(const char *pathname, int mode)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -621,7 +584,7 @@ int swoole_coroutine_access(const char *pathname, int mode)
 
 int swoole_coroutine_flock(int fd, int operation)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return flock(fd, operation);
     }
@@ -635,7 +598,7 @@ int swoole_coroutine_flock(int fd, int operation)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return -1;
@@ -644,359 +607,6 @@ int swoole_coroutine_flock(int fd, int operation)
     return ev.ret;
 }
 
-static void sleep_timeout(swTimer *timer, swTimer_node *tnode)
-{
-    ((Coroutine *) tnode->data)->resume();
-}
-
-int Coroutine::sleep(double sec)
-{
-    Coroutine* co = Coroutine::get_current_safe();
-    if (swTimer_add(&SwooleG.timer, (long) (sec * 1000), 0, co, sleep_timeout) == NULL)
-    {
-        return -1;
-    }
-    co->yield();
-    return 0;
-}
-
-swString* Coroutine::read_file(const char *file, int lock)
-{
-    aio_task task;
-
-    swAio_event ev;
-    bzero(&ev, sizeof(swAio_event));
-
-    task.co = Coroutine::get_current_safe();
-    task.event = &ev;
-
-    ev.lock = lock ? 1 : 0;
-    ev.type = SW_AIO_READ_FILE;
-    ev.object = (void*) &task;
-    ev.handler = swAio_handler_read_file;
-    ev.callback = aio_onReadFileCompleted;
-    ev.req = (void*) file;
-
-    int ret = swAio_dispatch(&ev);
-    if (ret < 0)
-    {
-        return NULL;
-    }
-    task.co->yield();
-    if (ev.error == 0)
-    {
-        swString *str = (swString *) sw_malloc(sizeof(swString));
-        str->str = (char*) ev.buf;
-        str->length = ev.nbytes;
-        return str;
-    }
-    else
-    {
-        SwooleG.error = ev.error;
-        return NULL;
-    }
-}
-
-ssize_t Coroutine::write_file(const char *file, char *buf, size_t length, int lock, int flags)
-{
-    aio_task task;
-
-    swAio_event ev;
-    bzero(&ev, sizeof(swAio_event));
-
-    task.co = Coroutine::get_current_safe();
-    task.event = &ev;
-
-    ev.lock = lock ? 1 : 0;
-    ev.type = SW_AIO_WRITE_FILE;
-    ev.buf = buf;
-    ev.nbytes = length;
-    ev.object = (void*) &task;
-    ev.handler = swAio_handler_write_file;
-    ev.callback = aio_onWriteFileCompleted;
-    ev.req = (void*) file;
-    ev.flags = flags;
-
-    int ret = swAio_dispatch(&ev);
-    if (ret < 0)
-    {
-        return -1;
-    }
-    task.co->yield();
-    if (ev.error != 0)
-    {
-        SwooleG.error = ev.error;
-    }
-    return ev.ret;
-}
-
-string Coroutine::gethostbyname(const string &hostname, int domain, double timeout)
-{
-    if (dns_cache == nullptr && dns_cache_capacity != 0)
-    {
-        dns_cache = new LRUCache(dns_cache_capacity);
-    }
-
-    string cache_key;
-    if (dns_cache)
-    {
-        cache_key.append(domain == AF_INET ? "4_" : "6_");
-        cache_key.append(hostname);
-        auto cache = dns_cache->get(cache_key);
-
-        if (cache)
-        {
-            return *(string *)cache.get();
-        }
-    }
-
-    swAio_event ev;
-    aio_task task;
-
-    bzero(&ev, sizeof(swAio_event));
-    if (hostname.size() < SW_IP_MAX_LENGTH)
-    {
-        ev.nbytes = SW_IP_MAX_LENGTH + 1;
-    }
-    else
-    {
-        ev.nbytes = hostname.size() + 1;
-    }
-
-    task.co = Coroutine::get_current_safe();
-    task.event = &ev;
-
-    ev.buf = sw_malloc(ev.nbytes);
-    if (!ev.buf)
-    {
-        return "";
-    }
-
-    memcpy(ev.buf, hostname.c_str(), hostname.size());
-    ((char *) ev.buf)[hostname.size()] = 0;
-    ev.flags = domain;
-    ev.type = SW_AIO_GETHOSTBYNAME;
-    ev.object = (void*) &task;
-    ev.handler = swAio_handler_gethostbyname;
-    ev.callback = aio_onDNSCompleted;
-    /* TODO: find a better way */
-    ev.ret = 1;
-
-    swAio_event *event = swAio_dispatch2(&ev);
-    swTimer_node *timer = nullptr;
-    if (timeout > 0)
-    {
-        timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, event, aio_onDNSTimeout);
-    }
-    task.co->yield();
-    if (ev.ret == 1)
-    {
-        /* TODO: find a better way */
-        /* canceled */
-        event->canceled = 1;
-        ev.ret = -1;
-        ev.error = SW_ERROR_DNSLOOKUP_RESOLVE_FAILED;
-    }
-    if (timer)
-    {
-        swTimer_del(&SwooleG.timer, timer);
-    }
-
-    if (ev.ret == -1)
-    {
-        SwooleG.error = ev.error;
-        return "";
-    }
-    else
-    {
-        if (dns_cache)
-        {
-            string *addr = new string((char *) ev.buf);
-            dns_cache->set(cache_key, shared_ptr<string>(addr), dns_cache_expire);
-            sw_free(ev.buf);
-            return *addr;
-        }
-
-        string addr((char *) ev.buf);
-        sw_free(ev.buf);
-        return addr;
-    }
-}
-
-struct coro_poll_task
-{
-    std::unordered_map<int, socket_poll_fd> *fds;
-    Coroutine *co = nullptr;
-    swTimer_node *timer = nullptr;
-    bool success = false;
-    bool wait = true;
-};
-
-static std::unordered_map<int, coro_poll_task *> coro_poll_task_map;
-
-static inline void socket_poll_clean(coro_poll_task *task)
-{
-    swReactor *reactor = SwooleG.main_reactor;
-    for (auto i = task->fds->begin(); i != task->fds->end(); i++)
-    {
-        coro_poll_task_map.erase(i->first);
-        if (reactor->del(reactor, i->first) < 0)
-        {
-            //TODO print error log
-            continue;
-        }
-    }
-}
-
-static void socket_poll_timeout(swTimer *timer, swTimer_node *tnode)
-{
-    coro_poll_task *task = (coro_poll_task *) tnode->data;
-    task->timer = nullptr;
-    task->success = false;
-    task->wait = false;
-    socket_poll_clean(task);
-    task->co->resume();
-}
-
-static void socket_poll_completed(void *data)
-{
-    coro_poll_task *task = (coro_poll_task *) data;
-    socket_poll_clean(task);
-    task->co->resume();
-}
-
-static inline void socket_poll_trigger_event(swReactor *reactor, int fd, enum swEvent_type event)
-{
-    coro_poll_task *task = coro_poll_task_map[fd];
-    auto i = task->fds->find(fd);
-    if (event == SW_EVENT_ERROR && !(i->second.events & SW_EVENT_ERROR))
-    {
-        if (i->second.events & SW_EVENT_READ)
-        {
-            i->second.revents |= SW_EVENT_READ;
-        }
-        if (i->second.events & SW_EVENT_WRITE)
-        {
-            i->second.revents |= SW_EVENT_WRITE;
-        }
-    }
-    else
-    {
-        i->second.revents |= event;
-    }
-    if (task->wait)
-    {
-        task->wait = false;
-        task->success = true;
-        if (task->timer)
-        {
-            swTimer_del(&SwooleG.timer, task->timer);
-            task->timer = nullptr;
-        }
-        reactor->defer(reactor, socket_poll_completed, task);
-    }
-}
-
-static int socket_poll_read_callback(swReactor *reactor, swEvent *event)
-{
-    socket_poll_trigger_event(reactor, event->fd, SW_EVENT_READ);
-    return SW_OK;
-}
-
-static int socket_poll_write_callback(swReactor *reactor, swEvent *event)
-{
-    socket_poll_trigger_event(reactor, event->fd, SW_EVENT_WRITE);
-    return SW_OK;
-}
-
-static int socket_poll_error_callback(swReactor *reactor, swEvent *event)
-{
-    socket_poll_trigger_event(reactor, event->fd, SW_EVENT_ERROR);
-    return SW_OK;
-}
-
-bool Coroutine::socket_poll(std::unordered_map<int, socket_poll_fd> &fds, double timeout)
-{
-    swReactor *reactor = SwooleG.main_reactor;
-    if (unlikely(!swReactor_handle_isset(reactor, SW_FD_CORO_POLL)))
-    {
-        reactor->setHandle(reactor, SW_FD_CORO_POLL | SW_EVENT_READ, socket_poll_read_callback);
-        reactor->setHandle(reactor, SW_FD_CORO_POLL | SW_EVENT_WRITE, socket_poll_write_callback);
-        reactor->setHandle(reactor, SW_FD_CORO_POLL | SW_EVENT_ERROR, socket_poll_error_callback);
-    }
-
-    if (timeout == 0)
-    {
-        struct pollfd *event_list = (struct pollfd *) sw_calloc(fds.size(), sizeof(struct pollfd));
-        int j = 0;
-        for (auto i = fds.begin(); i != fds.end(); i++)
-        {
-            event_list[j].fd = i->first;
-            event_list[j].events = i->second.events;
-            event_list[j].revents = 0;
-            j++;
-        }
-        int retval = ::poll(event_list, fds.size(), 0);
-        if (retval > 0)
-        {
-            for (size_t i = 0; i < fds.size(); i++)
-            {
-                auto _e = fds.find(event_list[i].fd);
-                int16_t revents = event_list[i].revents;
-                int16_t sw_revents = 0;
-                if (revents & POLLIN)
-                {
-                    sw_revents |= SW_EVENT_READ;
-                }
-                if (revents & POLLOUT)
-                {
-                    sw_revents |= SW_EVENT_WRITE;
-                }
-                //ignore ERR and HUP, because event is already processed at IN and OUT handler.
-                if ((((revents & POLLERR) || (revents & POLLHUP)) && !((revents & POLLIN) || (revents & POLLOUT))))
-                {
-                    sw_revents |= SW_EVENT_ERROR;
-                }
-                _e->second.revents = sw_revents;
-            }
-        }
-        sw_free(event_list);
-        return retval > 0;
-    }
-
-    size_t tasked_num = 0;
-    coro_poll_task task;
-    task.fds = &fds;
-    task.co = Coroutine::get_current_safe();
-
-    for (auto i = fds.begin(); i != fds.end(); i++)
-    {
-        if (reactor->add(reactor, i->first, i->second.events | SW_FD_CORO_POLL) < 0)
-        {
-            continue;
-        }
-        else
-        {
-            coro_poll_task_map[i->first] = &task;
-            tasked_num++;
-        }
-    }
-
-    if (unlikely(tasked_num == 0))
-    {
-        return false;
-    }
-
-    if (timeout > 0)
-    {
-        task.timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, &task, socket_poll_timeout);
-    }
-
-    task.co->yield();
-
-    return task.success;
-}
-
 #if 0
 static void handler_opendir(swAio_event *event)
 {
@@ -1014,7 +624,7 @@ static void handler_readdir(swAio_event *event)
 
 DIR *swoole_coroutine_opendir(const char *name)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return opendir(name);
     }
@@ -1027,7 +637,7 @@ DIR *swoole_coroutine_opendir(const char *name)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return nullptr;
@@ -1038,7 +648,7 @@ DIR *swoole_coroutine_opendir(const char *name)
 
 struct dirent *swoole_coroutine_readdir(DIR *dirp)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
     {
         return readdir(dirp);
     }
@@ -1051,7 +661,7 @@ struct dirent *swoole_coroutine_readdir(DIR *dirp)
     ev.object = Coroutine::get_current();
     ev.req = &ev;
 
-    int ret = swAio_dispatch(&ev);
+    ssize_t ret = swAio_dispatch(&ev);
     if (ret < 0)
     {
         return nullptr;
@@ -1060,6 +670,64 @@ struct dirent *swoole_coroutine_readdir(DIR *dirp)
     return (struct dirent *) ev.buf;
 }
 #endif
+
+void swoole_coroutine_sleep(int sec)
+{
+    System::sleep((double) sec);
 }
 
-#endif
+void swoole_coroutine_usleep(int usec)
+{
+    System::sleep((double) usec / 1024 / 1024);
+}
+
+int swoole_coroutine_socket_set_timeout(int fd, int which, double timeout)
+{
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
+    {
+        _error: errno = EINVAL;
+        return SW_ERR;
+    }
+    swSocket *conn = swReactor_get(SwooleTG.reactor, fd);
+    if (conn == nullptr)
+    {
+        goto _error;
+    }
+    Socket *socket = (Socket *) conn->object;
+    if (which == SO_RCVTIMEO)
+    {
+        socket->set_timeout(timeout, swoole::SW_TIMEOUT_READ);
+        return SW_OK;
+    }
+    else if (which == SO_SNDTIMEO)
+    {
+        socket->set_timeout(timeout, swoole::SW_TIMEOUT_WRITE);
+        return SW_OK;
+    }
+    else
+    {
+        goto _error;
+    }
+}
+
+int swoole_coroutine_socket_wait_event(int fd, int event, double timeout)
+{
+    if (sw_unlikely(SwooleTG.reactor == nullptr || !Coroutine::get_current()))
+    {
+        _error: errno = EINVAL;
+        return SW_ERR;
+    }
+    swSocket *conn = swReactor_get(SwooleTG.reactor, fd);
+    if (conn == nullptr)
+    {
+        goto _error;
+    }
+    Socket *socket = (Socket *) conn->object;
+    double ori_timeout = socket->get_timeout(event == SW_EVENT_READ ? swoole::SW_TIMEOUT_READ : swoole::SW_TIMEOUT_WRITE);
+    socket->set_timeout(timeout);
+    bool retval = socket->poll((enum swEvent_type) event);
+    socket->set_timeout(ori_timeout);
+    return retval ? SW_OK : SW_ERR;
+}
+
+SW_EXTERN_C_END
