@@ -21,16 +21,6 @@
 #include <sys/file.h>
 #include <sys/stat.h>
 
-#ifdef __sun
-#include <fcntl.h>
-#define LOCK_SH             F_RDLCK
-#define LOCK_EX             F_WRLCK
-#define LOCK_UN             F_UNLCK
-struct flock lock;
-#define flock(fildes, cmd) \
-    lock.l_type = (cmd), fcntl(fildes, F_SETLK, &lock)
-#endif
-
 #if 0
 swAsyncIO SwooleAIO;
 
@@ -49,7 +39,7 @@ int swAio_init(void)
         swWarn("AIO has already been initialized");
         return SW_ERR;
     }
-    if (!SwooleG.main_reactor)
+    if (!SwooleTG.reactor)
     {
         swWarn("No eventloop, cannot initialized");
         return SW_ERR;
@@ -77,8 +67,8 @@ int swAio_init(void)
     _pipe_read = _aio_pipe.getFd(&_aio_pipe, 0);
     _pipe_write = _aio_pipe.getFd(&_aio_pipe, 1);
 
-    SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_AIO, swAio_onCompleted);
-    SwooleG.main_reactor->add(SwooleG.main_reactor, _pipe_read, SW_FD_AIO);
+    SwooleTG.reactor->setHandle(SwooleTG.reactor, SW_FD_AIO, swAio_onCompleted);
+    swoole_event_add(_pipe_read, SW_FD_AIO);
 
     if (swThreadPool_run(&pool) < 0)
     {
@@ -124,7 +114,8 @@ static int swAio_onTask(swThreadPool *pool, void *task, int task_len)
 
     swTrace("aio_thread ok. ret=%d, error=%d", event->ret, event->error);
 
-    _error: do
+    _error:
+    do
     {
         SwooleAIO.lock.lock(&SwooleAIO.lock);
         int ret = write(_pipe_write, &task, sizeof(task));
@@ -186,9 +177,9 @@ void swAio_free(void)
         return;
     }
     swThreadPool_free(&pool);
-    if (SwooleG.main_reactor)
+    if (SwooleTG.reactor)
     {
-        SwooleG.main_reactor->del(SwooleG.main_reactor, _pipe_read);
+        SwooleTG.reactor->del(SwooleTG.reactor, _pipe_read);
     }
     _aio_pipe.close(&_aio_pipe);
     SwooleAIO.init = 0;
@@ -312,7 +303,8 @@ void swAio_handler_read_file(swAio_event *event)
     if (fstat(fd, &file_stat) < 0)
     {
         swSysWarn("fstat(%s) failed", (char * )event->req);
-        _error: close(fd);
+        _error:
+        close(fd);
         event->ret = ret;
         event->error = errno;
         return;
@@ -443,7 +435,7 @@ void swAio_handler_gethostbyname(swAio_event *event)
     int ret;
 
 #ifndef HAVE_GETHOSTBYNAME2_R
-    SwooleAIO.lock.lock(&SwooleAIO.lock);
+    SwooleG.lock.lock(&SwooleG.lock);
 #endif
     if (event->flags == AF_INET6)
     {
@@ -455,12 +447,12 @@ void swAio_handler_gethostbyname(swAio_event *event)
     }
     bzero(event->buf, event->nbytes);
 #ifndef HAVE_GETHOSTBYNAME2_R
-    SwooleAIO.lock.unlock(&SwooleAIO.lock);
+    SwooleG.lock.unlock(&SwooleG.lock);
 #endif
 
     if (ret < 0)
     {
-        event->error = h_errno;
+        event->error = SW_ERROR_DNSLOOKUP_RESOLVE_FAILED;
     }
     else
     {
