@@ -17,7 +17,10 @@
 #include "server.h"
 #include "http.h"
 #include "connection.h"
+#include <sys/time.h>
+#include <time.h>
 
+static int swServer_destory(swServer *serv);
 static int swServer_start_check(swServer *serv);
 static void swServer_signal_handler(int sig);
 static void swServer_disable_accept(swReactor *reactor);
@@ -26,14 +29,11 @@ static void swServer_master_update_time(swServer *serv);
 static int swServer_tcp_send(swServer *serv, int session_id, void *data, uint32_t length);
 static int swServer_tcp_sendwait(swServer *serv, int session_id, void *data, uint32_t length);
 static int swServer_tcp_close(swServer *serv, int session_id, int reset);
-static int swServer_tcp_sendfile(swServer *serv, int session_id, char *filename, uint32_t filename_length, off_t offset, size_t length);
+static int swServer_tcp_sendfile(swServer *serv, int session_id, const char *file, uint32_t l_file, off_t offset, size_t length);
 static int swServer_tcp_notify(swServer *serv, swConnection *conn, int event);
 static int swServer_tcp_feedback(swServer *serv, int session_id, int event);
 
-static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, int fd, int from_fd, int reactor_id);
-
-int16_t sw_errno;
-char sw_error[SW_ERROR_MSG_SIZE];
+static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, int fd, int server_fd);
 
 static void swServer_disable_accept(swReactor *reactor)
 {
@@ -63,7 +63,7 @@ void swServer_enable_accept(swReactor *reactor)
         {
             continue;
         }
-        reactor->add(reactor, ls->sock, SW_FD_LISTEN);
+        reactor->add(reactor, ls->sock, SW_FD_STREAM_SERVER);
     }
 }
 
@@ -85,16 +85,13 @@ void swServer_close_port(swServer *serv, enum swBool_type only_stream_port)
 int swServer_master_onAccept(swReactor *reactor, swEvent *event)
 {
     swServer *serv = (swServer *) reactor->ptr;
-    swReactor *sub_reactor;
-    swSocketAddress client_addr;
     swListenPort *listen_host = (swListenPort *) serv->connection_list[event->fd].object;
 
-    int new_fd = 0, reactor_id = 0, i;
+    int new_fd = 0, i;
 
-    //SW_ACCEPT_AGAIN
     for (i = 0; i < SW_ACCEPT_MAX_COUNT; i++)
     {
-        new_fd = swSocket_accept(event->fd, &client_addr);
+        new_fd = swSocket_accept(event->fd, &event->socket->info);
         if (new_fd < 0)
         {
             switch (errno)
@@ -124,54 +121,50 @@ int swServer_master_onAccept(swReactor *reactor, swEvent *event)
             return SW_OK;
         }
 
-        if (serv->single_thread)
-        {
-            reactor_id = 0;
-            sub_reactor = reactor;
-        }
-        else
-        {
-            reactor_id = new_fd % serv->reactor_num;
-            sub_reactor = &serv->reactor_threads[reactor_id].reactor;
-        }
-
         //add to connection_list
-        swConnection *conn = swServer_connection_new(serv, listen_host, new_fd, event->fd, reactor_id);
-        memcpy(&conn->info.addr, &client_addr, sizeof(client_addr));
-       
+        swConnection *conn = swServer_connection_new(serv, listen_host, new_fd, event->fd);
+        memcpy(&conn->info.addr, &event->socket->info, sizeof(event->socket->info));
         conn->socket_type = listen_host->type;
 
 #ifdef SW_USE_OPENSSL
+        swSocket *_socket = swReactor_get(reactor, conn->fd);
         if (listen_host->ssl)
         {
-            if (swSSL_create(conn, listen_host->ssl_context, 0) < 0)
+            if (swSSL_create(_socket, listen_host->ssl_context, 0) < 0)
             {
-                bzero(conn, sizeof(swConnection));
-                close(new_fd);
+                reactor->close(reactor, new_fd);
+                return SW_OK;
+            }
+            else
+            {
+                conn->ssl = 1;
+            }
+        }
+        else
+        {
+            _socket->ssl = NULL;
+        }
+#endif
+        if (serv->single_thread)
+        {
+            if (swServer_connection_incoming(serv, reactor, conn) < 0)
+            {
+                reactor->close(reactor, new_fd);
                 return SW_OK;
             }
         }
         else
         {
-            conn->ssl = NULL;
+            swDataHead ev = {0};
+            ev.type = SW_SERVER_EVENT_INCOMING;
+            ev.fd = new_fd;
+            int _pipe_fd = swServer_get_send_pipe(serv, conn->session_id, conn->reactor_id);
+            if (reactor->write(reactor, _pipe_fd, &ev, sizeof(ev)) < 0)
+            {
+                reactor->close(reactor, new_fd);
+                return SW_OK;
+            }
         }
-#endif
-        /*
-         * [!!!] new_connection function must before reactor->add
-         */
-        conn->connect_notify = 1;
-        if (sub_reactor->add(sub_reactor, new_fd, SW_FD_TCP | SW_EVENT_WRITE) < 0)
-        {
-            bzero(conn, sizeof(swConnection));
-            close(new_fd);
-            return SW_OK;
-        }
-
-#ifdef SW_ACCEPT_AGAIN
-        continue;
-#else
-        break;
-#endif
     }
     return SW_OK;
 }
@@ -193,14 +186,33 @@ static int swServer_start_check(swServer *serv)
     //disable notice when use SW_DISPATCH_ROUND and SW_DISPATCH_QUEUE
     if (serv->factory_mode == SW_MODE_PROCESS)
     {
-        if (serv->dispatch_mode == SW_DISPATCH_ROUND || serv->dispatch_mode == SW_DISPATCH_QUEUE)
+        if (!swServer_support_unsafe_events(serv))
         {
-            if (!serv->enable_unsafe_event)
+            if (serv->onConnect)
             {
-                serv->onConnect = NULL;
-                serv->onClose = NULL;
-                serv->disable_notify = 1;
+                swWarn("cannot set 'onConnect' event when using dispatch_mode=1/3/7");
+                serv->onConnect = nullptr;
             }
+            if (serv->onClose)
+            {
+                swWarn("cannot set 'onClose' event when using dispatch_mode=1/3/7");
+                serv->onClose = nullptr;
+            }
+            if (serv->onBufferFull)
+            {
+                swWarn("cannot set 'onBufferFull' event when using dispatch_mode=1/3/7");
+                serv->onBufferFull = nullptr;
+            }
+            if (serv->onBufferEmpty)
+            {
+                swWarn("cannot set 'onBufferEmpty' event when using dispatch_mode=1/3/7");
+                serv->onBufferEmpty = nullptr;
+            }
+            serv->disable_notify = 1;
+        }
+        if (!swServer_support_send_yield(serv))
+        {
+            serv->send_yield = 0;
         }
     }
     //AsyncTask
@@ -380,12 +392,13 @@ int swServer_create_task_worker(swServer *serv)
     }
 
     swProcessPool *pool = &serv->gs->task_workers;
-    if (swProcessPool_create(pool, serv->task_worker_num, serv->task_max_request, key, ipc_mode) < 0)
+    if (swProcessPool_create(pool, serv->task_worker_num, key, ipc_mode) < 0)
     {
         swWarn("[Master] create task_workers failed");
         return SW_ERR;
     }
 
+    swProcessPool_set_max_request(pool, serv->task_max_request, serv->task_max_request_grace);
     swProcessPool_set_start_id(pool, serv->worker_num);
     swProcessPool_set_type(pool, SW_PROCESS_TASKWORKER);
 
@@ -406,18 +419,7 @@ int swServer_create_task_worker(swServer *serv)
  */
 int swServer_worker_create(swServer *serv, swWorker *worker)
 {
-    /**
-     * Create shared memory storage
-     */
-    worker->send_shm = sw_shm_malloc(serv->buffer_output_size);
-    if (worker->send_shm == NULL)
-    {
-        swWarn("malloc for worker->store failed");
-        return SW_ERR;
-    }
-    swMutex_create(&worker->lock, 1);
-
-    return SW_OK;
+    return swMutex_create(&worker->lock, 1);
 }
 
 /**
@@ -466,18 +468,13 @@ int swServer_worker_init(swServer *serv, swWorker *worker)
     else
     {
         SwooleWG.max_request = serv->max_request;
-        if (SwooleWG.max_request > 10)
+        if (serv->max_request_grace > 0)
         {
-            int n = swoole_system_random(1, SwooleWG.max_request / 2);
-            if (n > 0)
-            {
-                SwooleWG.max_request += n;
-            }
+            SwooleWG.max_request += swoole_system_random(1, serv->max_request_grace);
         }
     }
 
     worker->start_time = serv->gs->now;
-    worker->request_time = 0;
     worker->request_count = 0;
 
     return SW_OK;
@@ -537,13 +534,13 @@ int swServer_start(swServer *serv)
     {
         swoole_call_hook(SW_GLOBAL_HOOK_BEFORE_SERVER_START, serv);
     }
-    //cann't start 2 servers at the same time, please use process->exec.
+    //cannot start 2 servers at the same time, please use process->exec.
     if (!sw_atomic_cmp_set(&serv->gs->start, 0, 1))
     {
         swoole_error_log(SW_LOG_ERROR, SW_ERROR_SERVER_ONLY_START_ONE, "must only start one server");
         return SW_ERR;
     }
-    //init loggger
+    //init logger
     if (SwooleG.log_file)
     {
         swLog_init(SwooleG.log_file);
@@ -601,6 +598,11 @@ int swServer_start(swServer *serv)
         return SW_ERR;
     }
 
+    if (swMutex_create(&serv->lock, 0) < 0)
+    {
+        return SW_ERR;
+    }
+
     /**
      * store to swProcessPool object
      */
@@ -609,7 +611,7 @@ int swServer_start(swServer *serv)
     serv->gs->event_workers.worker_num = serv->worker_num;
     serv->gs->event_workers.use_msgqueue = 0;
 
-    int i;
+    uint32_t i;
     for (i = 0; i < serv->worker_num; i++)
     {
         serv->gs->event_workers.workers[i].pool = &serv->gs->event_workers;
@@ -623,11 +625,24 @@ int swServer_start(swServer *serv)
     if (serv->task_worker_num > 0 && serv->worker_num > 0)
     {
         serv->task_result = (swEventData *) sw_shm_calloc(serv->worker_num, sizeof(swEventData));
+        if (!serv->task_result)
+        {
+            swWarn("malloc[serv->task_result] failed");
+            return SW_ERR;
+        }
         serv->task_notify = (swPipe *) sw_calloc(serv->worker_num, sizeof(swPipe));
+        if (!serv->task_notify)
+        {
+            swWarn("malloc[serv->task_notify] failed");
+            sw_shm_free(serv->task_result);
+            return SW_ERR;
+        }
         for (i = 0; i < serv->worker_num; i++)
         {
             if (swPipeNotify_auto(&serv->task_notify[i], 1, 0))
             {
+                sw_shm_free(serv->task_result);
+                sw_free(serv->task_notify);
                 return SW_ERR;
             }
         }
@@ -646,7 +661,7 @@ int swServer_start(swServer *serv)
             i++;
         }
     }
-
+    serv->running = 1;
     //factory start
     if (factory->start(factory) < 0)
     {
@@ -674,8 +689,7 @@ int swServer_start(swServer *serv)
     {
         return SW_ERR;
     }
-    swServer_free(serv);
-    serv->gs->start = 0;
+    swServer_destory(serv);
     //remove PID file
     if (serv->pid_file)
     {
@@ -706,7 +720,9 @@ void swServer_init(swServer *serv)
     //http server
     serv->http_parse_cookie = 1;
     serv->http_parse_post = 1;
+#ifdef SW_HAVE_ZLIB
     serv->http_compression = 1;
+#endif
     serv->http_compression_level = 1; // Z_BEST_SPEED
     serv->upload_tmp_dir = sw_strdup("/tmp");
 
@@ -716,6 +732,17 @@ void swServer_init(swServer *serv)
     serv->task_ipc_mode = SW_TASK_IPC_UNIXSOCK;
 
     serv->enable_coroutine = 1;
+    serv->reload_async = 1;
+    serv->send_yield = 1;
+
+#ifdef __linux__
+    serv->timezone = timezone;
+#else
+    struct timezone tz;
+    struct timeval tv;
+    gettimeofday(&tv, &tz);
+    serv->timezone = tz.tz_minuteswest * 60;
+#endif
 
     /**
      * alloc shared memory
@@ -749,6 +776,11 @@ int swServer_create(swServer *serv)
         return SW_ERR;
     }
 
+    if (SwooleG.socket_array->item_num < serv->max_connection)
+    {
+        swArray_alloc(SwooleG.socket_array, serv->max_connection);
+    }
+
     if (serv->factory_mode == SW_MODE_BASE)
     {
         return swReactorProcess_create(serv);
@@ -761,15 +793,37 @@ int swServer_create(swServer *serv)
 
 int swServer_shutdown(swServer *serv)
 {
+    serv->running = 0;
     //stop all thread
-    SwooleG.main_reactor->running = 0;
+    if (SwooleTG.reactor)
+    {
+        swReactor *reactor = SwooleTG.reactor;
+        reactor->wait_exit = 1;
+        swListenPort *port;
+        LL_FOREACH(serv->listen_list, port)
+        {
+            if (swSocket_is_stream(port->type))
+            {
+                reactor->del(reactor, port->sock);
+            }
+        }
+        if (serv->master_timer)
+        {
+            swoole_timer_del(serv->master_timer);
+            serv->master_timer = NULL;
+        }
+    }
+    else
+    {
+        SwooleG.running = 0;
+    }
+    swInfo("Server is shutdown now");
     return SW_OK;
 }
 
-int swServer_free(swServer *serv)
+static int swServer_destory(swServer *serv)
 {
     swTraceLog(SW_TRACE_SERVER, "release service");
-
     /**
      * shutdown workers
      */
@@ -777,22 +831,6 @@ int swServer_free(swServer *serv)
     {
         serv->factory.shutdown(&(serv->factory));
     }
-    /**
-     * Shutdown heartbeat thread
-     */
-    if (serv->heartbeat_pidt)
-    {
-        swTraceLog(SW_TRACE_SERVER, "terminate heartbeat thread");
-        if (pthread_cancel(serv->heartbeat_pidt) < 0)
-        {
-            swSysWarn("pthread_cancel(%ld) failed", (ulong_t )serv->heartbeat_pidt);
-        }
-        //wait thread
-        if (pthread_join(serv->heartbeat_pidt, NULL) < 0)
-        {
-            swSysWarn("pthread_join(%ld) failed", (ulong_t )serv->heartbeat_pidt);
-        }
-    }
     if (serv->factory_mode == SW_MODE_BASE)
     {
         swTraceLog(SW_TRACE_SERVER, "terminate task workers");
@@ -801,13 +839,13 @@ int swServer_free(swServer *serv)
             swProcessPool_shutdown(&serv->gs->task_workers);
         }
     }
-    else if (!serv->single_thread)
+    else
     {
         swTraceLog(SW_TRACE_SERVER, "terminate reactor threads");
         /**
          * Wait until all the end of the thread
          */
-        swReactorThread_free(serv);
+        swReactorThread_join(serv);
     }
 
     swListenPort *port;
@@ -824,35 +862,30 @@ int swServer_free(swServer *serv)
     {
         close(SwooleG.null_fd);
     }
+    swSignal_clear();
     /**
-     * free worker pipe
+     * shutdown status
      */
-    if (serv->factory.free)
-    {
-        serv->factory.free(&serv->factory);
-    }
-    if (serv->gs->start > 0 && serv->onShutdown != NULL)
+    serv->gs->start = 0;
+    serv->gs->shutdown = 1;
+    /**
+     * callback
+     */
+    if (serv->onShutdown)
     {
         serv->onShutdown(serv);
     }
-    return SW_OK;
-}
-
-int swServer_udp_send(swServer *serv, swSendData *resp)
-{
-    struct sockaddr_in addr_in;
-    int sock = resp->info.from_fd;
-
-    addr_in.sin_family = AF_INET;
-    addr_in.sin_port = htons((uint16_t) resp->info.from_id); //from_id is remote port
-    addr_in.sin_addr.s_addr = (uint32_t) resp->info.fd; //fd is remote ip address
-
-    int ret = swSocket_sendto_blocking(sock, resp->data, resp->info.len, 0, (struct sockaddr*) &addr_in, sizeof(addr_in));
-    if (ret < 0)
+    if (serv->factory_mode == SW_MODE_BASE)
     {
-        swSysWarn("sendto to client[%s:%d] failed", inet_ntoa(addr_in.sin_addr), resp->info.from_id);
+        swReactorProcess_free(serv);
     }
-    return ret;
+    else
+    {
+        swReactorThread_free(serv);
+    }
+    serv->lock.free(&serv->lock);
+    SwooleG.serv = nullptr;
+    return SW_OK;
 }
 
 /**
@@ -866,7 +899,7 @@ static int swServer_tcp_feedback(swServer *serv, int session_id, int event)
         return SW_ERR;
     }
 
-    if (event == SW_EVENT_CONFIRM && !conn->listen_wait)
+    if (event == SW_SERVER_EVENT_CONFIRM && !conn->socket->listen_wait)
     {
         return SW_ERR;
     }
@@ -875,7 +908,7 @@ static int swServer_tcp_feedback(swServer *serv, int session_id, int event)
     bzero(&_send, sizeof(_send));
     _send.info.type = event;
     _send.info.fd = session_id;
-    _send.info.from_id = conn->from_id;
+    _send.info.reactor_id = conn->reactor_id;
 
     if (serv->factory_mode == SW_MODE_PROCESS)
     {
@@ -890,9 +923,12 @@ static int swServer_tcp_feedback(swServer *serv, int session_id, int event)
 void swServer_store_pipe_fd(swServer *serv, swPipe *p)
 {
     int master_fd = p->getFd(p, SW_PIPE_MASTER);
+    int worker_fd = p->getFd(p, SW_PIPE_WORKER);
 
-    serv->connection_list[p->getFd(p, SW_PIPE_WORKER)].object = p;
+    serv->connection_list[worker_fd].object = p;
+    serv->connection_list[worker_fd].socket = (swSocket *) swArray_alloc(SwooleG.socket_array, worker_fd);
     serv->connection_list[master_fd].object = p;
+    serv->connection_list[master_fd].socket = (swSocket *) swArray_alloc(SwooleG.socket_array, master_fd);
 
     if (master_fd > swServer_get_minfd(serv))
     {
@@ -906,7 +942,8 @@ swPipe * swServer_get_pipe_object(swServer *serv, int pipe_fd)
 }
 
 /**
- * [Worker]
+ * @process Worker
+ * @return SW_OK or SW_ERR
  */
 static int swServer_tcp_send(swServer *serv, int session_id, void *data, uint32_t length)
 {
@@ -914,14 +951,14 @@ static int swServer_tcp_send(swServer *serv, int session_id, void *data, uint32_
     bzero(&_send.info, sizeof(_send.info));
     swFactory *factory = &(serv->factory);
 
-    if (unlikely(swIsMaster()))
+    if (sw_unlikely(swIsMaster()))
     {
         swoole_error_log(SW_LOG_ERROR, SW_ERROR_SERVER_SEND_IN_MASTER, "can't send data to the connections in master process");
         return SW_ERR;
     }
 
     _send.info.fd = session_id;
-    _send.info.type = SW_EVENT_TCP;
+    _send.info.type = SW_SERVER_EVENT_SEND_DATA;
     _send.data = (char*) data;
     _send.info.len = length;
     return factory->finish(factory, &_send) < 0 ? SW_ERR : SW_OK;
@@ -937,7 +974,7 @@ int swServer_master_send(swServer *serv, swSendData *_send)
     uint32_t _send_length = _send->info.len;
 
     swConnection *conn;
-    if (_send->info.type != SW_EVENT_CLOSE)
+    if (_send->info.type != SW_SERVER_EVENT_CLOSE)
     {
         conn = swServer_connection_verify(serv, session_id);
     }
@@ -947,7 +984,7 @@ int swServer_master_send(swServer *serv, swSendData *_send)
     }
     if (!conn)
     {
-        if (_send->info.type == SW_EVENT_TCP)
+        if (_send->info.type == SW_SERVER_EVENT_SEND_DATA)
         {
             swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_NOT_EXIST, "send %d byte failed, session#%d does not exist", _send_length, session_id);
         }
@@ -959,15 +996,10 @@ int swServer_master_send(swServer *serv, swSendData *_send)
     }
 
     int fd = conn->fd;
-    swReactor *reactor;
+    swReactor *reactor = SwooleTG.reactor;
 
-    if (serv->single_thread)
+    if (!serv->single_thread)
     {
-        reactor = SwooleG.main_reactor;
-    }
-    else
-    {
-        reactor = &(serv->reactor_threads[conn->from_id].reactor);
         assert(fd % serv->reactor_num == reactor->id);
         assert(fd % serv->reactor_num == SwooleTG.id);
     }
@@ -985,27 +1017,29 @@ int swServer_master_send(swServer *serv, swSendData *_send)
         return SW_ERR;
     }
 
+    swSocket *_socket = conn->socket;
+
     /**
      * Reset send buffer, Immediately close the connection.
      */
-    if (_send->info.type == SW_EVENT_CLOSE && (conn->close_reset || conn->removed))
+    if (_send->info.type == SW_SERVER_EVENT_CLOSE && (conn->close_reset || conn->peer_closed))
     {
-        goto close_fd;
+        goto _close_fd;
     }
-    else if (_send->info.type == SW_EVENT_CONFIRM)
+    else if (_send->info.type == SW_SERVER_EVENT_CONFIRM)
     {
-        reactor->add(reactor, conn->fd, conn->fdtype | SW_EVENT_READ);
-        conn->listen_wait = 0;
+        reactor->add(reactor, conn->fd, conn->socket->fdtype | SW_EVENT_READ);
+        conn->socket->listen_wait = 0;
         return SW_OK;
     }
     /**
      * pause recv data
      */
-    else if (_send->info.type == SW_EVENT_PAUSE_RECV)
+    else if (_send->info.type == SW_SERVER_EVENT_PAUSE_RECV)
     {
-        if (conn->events & SW_EVENT_WRITE)
+        if (_socket->events & SW_EVENT_WRITE)
         {
-            return reactor->set(reactor, conn->fd, conn->fdtype | SW_EVENT_WRITE);
+            return reactor->set(reactor, conn->fd, conn->socket->fdtype | SW_EVENT_WRITE);
         }
         else
         {
@@ -1015,41 +1049,41 @@ int swServer_master_send(swServer *serv, swSendData *_send)
     /**
      * resume recv data
      */
-    else if (_send->info.type == SW_EVENT_RESUME_RECV)
+    else if (_send->info.type == SW_SERVER_EVENT_RESUME_RECV)
     {
-        if (conn->events & SW_EVENT_WRITE)
+        if (_socket->events & SW_EVENT_WRITE)
         {
-            return reactor->set(reactor, conn->fd, conn->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
+            return reactor->set(reactor, conn->fd, _socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
         }
         else
         {
-            return reactor->add(reactor, conn->fd, conn->fdtype | SW_EVENT_READ);
+            return reactor->add(reactor, conn->fd, _socket->fdtype | SW_EVENT_READ);
         }
     }
 
-    if (swBuffer_empty(conn->out_buffer))
+    if (swBuffer_empty(_socket->out_buffer))
     {
         /**
          * close connection.
          */
-        if (_send->info.type == SW_EVENT_CLOSE)
+        if (_send->info.type == SW_SERVER_EVENT_CLOSE)
         {
-            close_fd:
+            _close_fd:
             reactor->close(reactor, fd);
             return SW_OK;
         }
-#ifdef SW_REACTOR_SYNC_SEND
         //Direct send
-        if (_send->info.type != SW_EVENT_SENDFILE)
+        if (_send->info.type != SW_SERVER_EVENT_SEND_FILE)
         {
-            if (!conn->direct_send)
+            if (!_socket->direct_send)
             {
-                goto buffer_send;
+                goto _buffer_send;
             }
 
             ssize_t n;
 
-            direct_send: n = swConnection_send(conn, _send_data, _send_length, 0);
+            _direct_send:
+            n = swConnection_send(_socket, _send_data, _send_length, 0);
             if (n == _send_length)
             {
                 return SW_OK;
@@ -1058,28 +1092,25 @@ int swServer_master_send(swServer *serv, swSendData *_send)
             {
                 _send_data += n;
                 _send_length -= n;
-                goto buffer_send;
+                goto _buffer_send;
             }
             else if (errno == EINTR)
             {
-                goto direct_send;
+                goto _direct_send;
             }
             else
             {
-                goto buffer_send;
+                goto _buffer_send;
             }
         }
-#endif
         //buffer send
         else
         {
-#ifdef SW_REACTOR_SYNC_SEND
-            buffer_send:
-#endif
-            if (!conn->out_buffer)
+            _buffer_send:
+            if (!_socket->out_buffer)
             {
-                conn->out_buffer = swBuffer_new(SW_SEND_BUFFER_SIZE);
-                if (conn->out_buffer == NULL)
+                _socket->out_buffer = swBuffer_new(SW_SEND_BUFFER_SIZE);
+                if (_socket->out_buffer == NULL)
                 {
                     return SW_ERR;
                 }
@@ -1089,29 +1120,29 @@ int swServer_master_send(swServer *serv, swSendData *_send)
 
     swBuffer_chunk *chunk;
     //close connection
-    if (_send->info.type == SW_EVENT_CLOSE)
+    if (_send->info.type == SW_SERVER_EVENT_CLOSE)
     {
-        chunk = swBuffer_new_chunk(conn->out_buffer, SW_CHUNK_CLOSE, 0);
+        chunk = swBuffer_new_chunk(_socket->out_buffer, SW_CHUNK_CLOSE, 0);
         chunk->store.data.val1 = _send->info.type;
         conn->close_queued = 1;
     }
     //sendfile to client
-    else if (_send->info.type == SW_EVENT_SENDFILE)
+    else if (_send->info.type == SW_SERVER_EVENT_SEND_FILE)
     {
         swSendFile_request *req = (swSendFile_request *) _send_data;
-        swConnection_sendfile(conn, req->filename, req->offset, req->length);
+        swConnection_sendfile(conn->socket, req->filename, req->offset, req->length);
     }
     //send data
     else
     {
         //connection is closed
-        if (conn->removed)
+        if (conn->peer_closed)
         {
             swWarn("connection#%d is closed by client", fd);
             return SW_ERR;
         }
         //connection output buffer overflow
-        if (conn->out_buffer->length >= conn->buffer_size)
+        if (_socket->out_buffer->length >= _socket->buffer_size)
         {
             if (serv->send_yield)
             {
@@ -1128,25 +1159,25 @@ int swServer_master_send(swServer *serv, swSendData *_send)
             }
         }
 
-        if (swBuffer_append(conn->out_buffer, _send_data, _send_length) < 0)
+        if (swBuffer_append(_socket->out_buffer, _send_data, _send_length) < 0)
         {
             swWarn("append to pipe_buffer failed");
             return SW_ERR;
         }
 
         swListenPort *port = swServer_get_port(serv, fd);
-        if (serv->onBufferFull && conn->high_watermark == 0 && conn->out_buffer->length >= port->buffer_high_watermark)
+        if (serv->onBufferFull && conn->high_watermark == 0 && _socket->out_buffer->length >= port->buffer_high_watermark)
         {
-            serv->notify(serv, conn, SW_EVENT_BUFFER_FULL);
+            serv->notify(serv, conn, SW_SERVER_EVENT_BUFFER_FULL);
             conn->high_watermark = 1;
         }
     }
 
     //listen EPOLLOUT event
-    if (reactor->set(reactor, fd, SW_EVENT_TCP | SW_EVENT_WRITE | SW_EVENT_READ) < 0
+    if (reactor->set(reactor, fd, SW_FD_SESSION | SW_EVENT_WRITE | SW_EVENT_READ) < 0
             && (errno == EBADF || errno == ENOENT))
     {
-        goto close_fd;
+        goto _close_fd;
     }
 
     return SW_OK;
@@ -1157,28 +1188,27 @@ int swServer_master_send(swServer *serv, swSendData *_send)
  */
 static int swServer_tcp_notify(swServer *serv, swConnection *conn, int event)
 {
-    swDataHead notify_event;
+    swDataHead notify_event = {0};
     notify_event.type = event;
-    notify_event.from_id = conn->from_id;
+    notify_event.reactor_id = conn->reactor_id;
     notify_event.fd = conn->fd;
-    notify_event.from_fd = conn->from_fd;
-    notify_event.len = 0;
-    notify_event.flags = 0;
+    notify_event.server_fd = conn->server_fd;
     return serv->factory.notify(&serv->factory, &notify_event);
 }
 
 /**
- * [Worker]
+ * @process Worker
+ * @return SW_OK or SW_ERR
  */
-static int swServer_tcp_sendfile(swServer *serv, int session_id, char *filename, uint32_t filename_length, off_t offset, size_t length)
+static int swServer_tcp_sendfile(swServer *serv, int session_id, const char *file, uint32_t l_file, off_t offset, size_t length)
 {
-    if (unlikely(session_id <= 0 || session_id > SW_MAX_SESSION_ID))
+    if (sw_unlikely(session_id <= 0 || session_id > SW_MAX_SESSION_ID))
     {
         swoole_error_log(SW_LOG_WARNING, SW_ERROR_SESSION_INVALID_ID, "invalid fd[%d]", session_id);
         return SW_ERR;
     }
 
-    if (unlikely(swIsMaster()))
+    if (sw_unlikely(swIsMaster()))
     {
         swoole_error_log(SW_LOG_ERROR, SW_ERROR_SERVER_SEND_IN_MASTER, "can't send data to the connections in master process");
         return SW_ERR;
@@ -1188,22 +1218,23 @@ static int swServer_tcp_sendfile(swServer *serv, int session_id, char *filename,
     swSendFile_request *req = (swSendFile_request*) _buffer;
 
     // file name size
-    if (unlikely(filename_length > SW_IPC_BUFFER_SIZE - sizeof(swSendFile_request) - 1))
+    if (sw_unlikely(l_file > SW_IPC_BUFFER_SIZE - sizeof(swSendFile_request) - 1))
     {
         swoole_error_log(
             SW_LOG_WARNING, SW_ERROR_NAME_TOO_LONG, "sendfile name[%.8s...] length %u is exceed the max name len %u",
-            filename, filename_length, (uint32_t) (SW_IPC_BUFFER_SIZE - sizeof(swSendFile_request) - 1)
+            file, l_file, (uint32_t) (SW_IPC_BUFFER_SIZE - sizeof(swSendFile_request) - 1)
         );
         return SW_ERR;
     }
     // string must be zero termination (for `state` system call)
-    (filename = strncpy((char *) req->filename, filename, filename_length))[filename_length] = '\0';
+    char *_file = strncpy((char *) req->filename, file, l_file);
+    _file[l_file] = '\0';
 
     // check state
     struct stat file_stat;
-    if (stat(filename, &file_stat) < 0)
+    if (stat(_file, &file_stat) < 0)
     {
-        swoole_error_log(SW_LOG_WARNING, SW_ERROR_SYSTEM_CALL_FAIL, "stat(%s) failed", filename);
+        swoole_error_log(SW_LOG_WARNING, SW_ERROR_SYSTEM_CALL_FAIL, "stat(%s) failed", _file);
         return SW_ERR;
     }
     if (file_stat.st_size <= offset)
@@ -1217,15 +1248,15 @@ static int swServer_tcp_sendfile(swServer *serv, int session_id, char *filename,
     // construct send data
     swSendData send_data = {{0}};
     send_data.info.fd = session_id;
-    send_data.info.type = SW_EVENT_SENDFILE;
-    send_data.info.len = sizeof(swSendFile_request) + filename_length + 1;
+    send_data.info.type = SW_SERVER_EVENT_SEND_FILE;
+    send_data.info.len = sizeof(swSendFile_request) + l_file + 1;
     send_data.data = _buffer;
 
-    return serv->factory.finish(&serv->factory, &send_data);
+    return serv->factory.finish(&serv->factory, &send_data) < 0 ? SW_ERR : SW_OK;
 }
 
 /**
- * [Worker]
+ * [Worker] Returns the number of bytes sent
  */
 static int swServer_tcp_sendwait(swServer *serv, int session_id, void *data, uint32_t length)
 {
@@ -1257,7 +1288,7 @@ SW_API void swServer_call_hook(swServer *serv, enum swServer_hook_type type, voi
  */
 static int swServer_tcp_close(swServer *serv, int session_id, int reset)
 {
-    if (unlikely(swIsMaster()))
+    if (sw_unlikely(swIsMaster()))
     {
         swoole_error_log(SW_LOG_ERROR, SW_ERROR_SERVER_SEND_IN_MASTER, "can't close the connections in master process");
         return SW_ERR;
@@ -1281,9 +1312,9 @@ static int swServer_tcp_close(swServer *serv, int session_id, int reset)
     {
         swWorker *worker = swServer_get_worker(serv, conn->fd % serv->worker_num);
         swDataHead ev = {0};
-        ev.type = SW_EVENT_CLOSE;
+        ev.type = SW_SERVER_EVENT_CLOSE;
         ev.fd = session_id;
-        ev.from_id = conn->from_id;
+        ev.reactor_id = conn->reactor_id;
         ret = swWorker_send2worker(worker, &ev, sizeof(ev), SW_PIPE_MASTER);
     }
     else
@@ -1542,7 +1573,7 @@ int swServer_add_systemd_socket(swServer *serv)
     return count;
 }
 
-swListenPort* swServer_add_port(swServer *serv, int type, const char *host, int port)
+swListenPort* swServer_add_port(swServer *serv, enum swSocket_type type, const char *host, int port)
 {
     if (serv->listen_port_num >= SW_MAX_LISTEN_PORT)
     {
@@ -1575,7 +1606,7 @@ swListenPort* swServer_add_port(swServer *serv, int type, const char *host, int
 
     if (type & SW_SOCK_SSL)
     {
-        type = type & (~SW_SOCK_SSL);
+        type = (enum swSocket_type) (type & (~SW_SOCK_SSL));
         if (swSocket_is_stream(type))
         {
             ls->type = type;
@@ -1651,7 +1682,7 @@ int swServer_get_socket(swServer *serv, int port)
 
 static void swServer_signal_handler(int sig)
 {
-    swTraceLog(SW_TRACE_SERVER, "signal[%d] triggered", sig);
+    swTraceLog(SW_TRACE_SERVER, "signal[%d] %s triggered in %d", sig, swSignal_str(sig), getpid());
 
     swServer *serv = SwooleG.serv;
     int status;
@@ -1659,21 +1690,13 @@ static void swServer_signal_handler(int sig)
     switch (sig)
     {
     case SIGTERM:
-        if (SwooleG.main_reactor)
-        {
-            SwooleG.main_reactor->running = 0;
-        }
-        else
-        {
-            SwooleG.running = 0;
-        }
-        swInfo("Server is shutdown now");
+        swServer_shutdown(serv);
         break;
     case SIGALRM:
         swSystemTimer_signal_handler(SIGALRM);
         break;
     case SIGCHLD:
-        if (!SwooleG.running)
+        if (!serv->running)
         {
             break;
         }
@@ -1709,35 +1732,53 @@ static void swServer_signal_handler(int sig)
         }
         else
         {
-            swKill(serv->gs->manager_pid, sig);
+            swoole_kill(serv->gs->manager_pid, sig);
         }
         break;
     default:
 #ifdef SIGRTMIN
         if (sig == SIGRTMIN)
         {
-            int i;
+            uint32_t i;
             swWorker *worker;
             for (i = 0; i < SwooleG.serv->worker_num + serv->task_worker_num + SwooleG.serv->user_worker_num; i++)
             {
                 worker = swServer_get_worker(SwooleG.serv, i);
-                swKill(worker->pid, SIGRTMIN);
+                swoole_kill(worker->pid, SIGRTMIN);
             }
             if (SwooleG.serv->factory_mode == SW_MODE_PROCESS)
             {
-                swKill(serv->gs->manager_pid, SIGRTMIN);
+                swoole_kill(serv->gs->manager_pid, SIGRTMIN);
             }
-            swServer_reopen_log_file(SwooleG.serv);
+            swLog_reopen(SwooleG.serv->daemonize ? SW_TRUE : SW_FALSE);
         }
 #endif
         break;
     }
 }
 
+void swServer_connection_each(swServer *serv, void (*callback)(swConnection *conn))
+{
+    swConnection *conn;
+
+    int fd;
+    int serv_max_fd = swServer_get_maxfd(serv);
+    int serv_min_fd = swServer_get_minfd(serv);
+
+    for (fd = serv_min_fd; fd <= serv_max_fd; fd++)
+    {
+        conn = swServer_connection_get(serv, fd);
+        if (conn && conn->socket && conn->active == 1 && conn->closed == 0 && conn->socket->fdtype == SW_FD_SESSION)
+        {
+            callback(conn);
+        }
+    }
+}
+
 /**
  * new connection
  */
-static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, int fd, int from_fd, int reactor_id)
+static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, int fd, int server_fd)
 {
     swConnection* connection = NULL;
 
@@ -1753,6 +1794,12 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     connection = &(serv->connection_list[fd]);
     bzero(connection, sizeof(swConnection));
 
+    swSocket *_socket = swReactor_get(SwooleTG.reactor, fd);
+    _socket->object = connection;
+    _socket->buffer_size = ls->socket_buffer_size;
+    _socket->fd = fd;
+    _socket->fdtype = SW_FD_SESSION;
+
     //TCP Nodelay
     if (ls->open_tcp_nodelay && ls->type != SW_SOCK_UNIX_STREAM)
     {
@@ -1761,7 +1808,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
         {
             swSysWarn("setsockopt(TCP_NODELAY) failed");
         }
-        connection->tcp_nodelay = 1;
+        _socket->tcp_nodelay = 1;
     }
 
     //socket recv buffer size
@@ -1783,19 +1830,17 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     }
 
     connection->fd = fd;
-    connection->from_id = serv->factory_mode == SW_MODE_BASE ? SwooleWG.id : reactor_id;
-    connection->from_fd = (sw_atomic_t) from_fd;
+    connection->reactor_id = serv->factory_mode == SW_MODE_BASE ? SwooleWG.id : fd % serv->reactor_num;
+    connection->server_fd = (sw_atomic_t) server_fd;
     connection->connect_time = serv->gs->now;
     connection->last_time = serv->gs->now;
     connection->active = 1;
-    connection->buffer_size = ls->socket_buffer_size;
+    connection->socket = _socket;
 
-#ifdef SW_REACTOR_SYNC_SEND
     if (!ls->ssl)
     {
-        connection->direct_send = 1;
+        _socket->direct_send = 1;
     }
-#endif
 
     swSession *session;
     sw_spinlock(&serv->gs->spinlock);
@@ -1806,7 +1851,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     {
         session_id++;
         //SwooleGS->session_round just has 24 bits size;
-        if (unlikely(session_id == 1 << 24))
+        if (sw_unlikely(session_id == 1 << 24))
         {
             session_id = 1;
         }
@@ -1816,7 +1861,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
         {
             session->fd = fd;
             session->id = session_id;
-            session->reactor_id = connection->from_id;
+            session->reactor_id = connection->reactor_id;
             break;
         }
     }
