@@ -14,7 +14,6 @@
   +----------------------------------------------------------------------+
 */
 
-#include "swoole.h"
 #include "server.h"
 #include "client.h"
 #include "async.h"
@@ -25,16 +24,9 @@
 static int swWorker_onPipeReceive(swReactor *reactor, swEvent *event);
 static int swWorker_onStreamAccept(swReactor *reactor, swEvent *event);
 static int swWorker_onStreamRead(swReactor *reactor, swEvent *event);
-static int swWorker_onStreamPackage(swConnection *conn, char *data, uint32_t length);
+static int swWorker_onStreamPackage(swProtocol *proto, swSocket *conn, char *data, uint32_t length);
 static int swWorker_onStreamClose(swReactor *reactor, swEvent *event);
-
-void swWorker_free(swWorker *worker)
-{
-    if (worker->send_shm)
-    {
-        sw_shm_free(worker->send_shm);
-    }
-}
+static int swWorker_reactor_is_empty(swReactor *reactor);
 
 void swWorker_signal_init(void)
 {
@@ -65,7 +57,7 @@ void swWorker_signal_handler(int signo)
         /**
          * Event worker
          */
-        if (SwooleG.main_reactor)
+        if (SwooleTG.reactor)
         {
             swWorker_stop(SwooleWG.worker);
         }
@@ -94,7 +86,7 @@ void swWorker_signal_handler(int signo)
 #ifdef SIGRTMIN
         if (signo == SIGRTMIN)
         {
-            swServer_reopen_log_file(SwooleG.serv);
+            swLog_reopen(SwooleG.serv->daemonize ? SW_TRUE : SW_FALSE);
         }
 #endif
         break;
@@ -112,20 +104,20 @@ static sw_inline int swWorker_discard_data(swServer *serv, swEventData *task)
         {
             return SW_FALSE;
         }
-        goto discard_data;
+        goto _discard_data;
     }
     else
     {
         if (conn->closed)
         {
-            goto discard_data;
+            goto _discard_data;
         }
         else
         {
             return SW_FALSE;
         }
     }
-    discard_data:
+    _discard_data:
     {
         swoole_error_log(SW_LOG_WARNING, SW_ERROR_SESSION_DISCARD_TIMEOUT_DATA, "[1]received the wrong data[%d bytes] from socket#%d", task->info.len, session_id);
     }
@@ -136,7 +128,6 @@ static int swWorker_onStreamAccept(swReactor *reactor, swEvent *event)
 {
     swSocketAddress client_addr;
     int fd =  swSocket_accept(event->fd, &client_addr);
-
     if (fd < 0)
     {
         switch (errno)
@@ -150,25 +141,18 @@ static int swWorker_onStreamAccept(swReactor *reactor, swEvent *event)
         }
     }
 
-    swConnection *conn = swReactor_get(reactor, fd);
-    bzero(conn, sizeof(swConnection));
+    swSocket *conn = swReactor_get(reactor, fd);
+    bzero(conn, sizeof(swSocket));
     conn->fd = fd;
-    conn->active = 1;
     conn->socket_type = SW_SOCK_UNIX_STREAM;
     conn->nonblock = 1;
-    memcpy(&conn->info.addr, &client_addr, sizeof(client_addr));
 
-    if (reactor->add(reactor, fd, SW_FD_STREAM | SW_EVENT_READ) < 0)
-    {
-        return SW_ERR;
-    }
-
-    return SW_OK;
+    return reactor->add(reactor, fd, SW_FD_STREAM | SW_EVENT_READ);
 }
 
 static int swWorker_onStreamRead(swReactor *reactor, swEvent *event)
 {
-    swConnection *conn = event->socket;
+    swSocket *conn = event->socket;
     swServer *serv = (swServer *) reactor->ptr;
     swProtocol *protocol = &serv->stream_protocol;
     swString *buffer;
@@ -202,27 +186,27 @@ static int swWorker_onStreamRead(swReactor *reactor, swEvent *event)
 
 static int swWorker_onStreamClose(swReactor *reactor, swEvent *event)
 {
-    swConnection *conn = event->socket;
+    swSocket *conn = event->socket;
     swServer *serv = (swServer *) reactor->ptr;
 
     swString_clear(conn->recv_buffer);
     swLinkedList_append(serv->buffer_pool, conn->recv_buffer);
-    conn->recv_buffer = NULL;
+    conn->recv_buffer = nullptr;
 
-    reactor->del(reactor, conn->fd);
-    reactor->close(reactor, conn->fd);
+    reactor->del(reactor, event->fd);
+    reactor->close(reactor, event->fd);
 
     return SW_OK;
 }
 
-static int swWorker_onStreamPackage(swConnection *conn, char *data, uint32_t length)
+static int swWorker_onStreamPackage(swProtocol *proto, swSocket *conn, char *data, uint32_t length)
 {
-    swServer *serv = SwooleG.serv;
+    swServer *serv = (swServer *) proto->private_data_2;
 
     /**
      * passing memory pointer
      */
-    swPackagePtr task;
+    swPacket_ptr task;
     memcpy(&task.info, data + 4, sizeof(task.info));
     task.info.flags = SW_EVENT_DATA_PTR;
 
@@ -241,7 +225,7 @@ static int swWorker_onStreamPackage(swConnection *conn, char *data, uint32_t len
      * stream end
      */
     int _end = 0;
-    SwooleG.main_reactor->write(SwooleG.main_reactor, conn->fd, (void *) &_end, sizeof(_end));
+    SwooleTG.reactor->write(SwooleTG.reactor, conn->fd, (void *) &_end, sizeof(_end));
 
     return SW_OK;
 }
@@ -250,16 +234,13 @@ typedef int (*task_callback)(swServer *, swEventData *);
 
 static sw_inline void swWorker_do_task(swServer *serv, swWorker *worker, swEventData *task, task_callback callback)
 {
-    worker->request_time = serv->gs->now;
 #ifdef SW_BUFFER_RECV_TIME
     serv->last_receive_usec = task->info.time;
 #endif
     callback(serv, task);
-    worker->request_time = 0;
 #ifdef SW_BUFFER_RECV_TIME
     serv->last_receive_usec = 0;
 #endif
-    worker->traced = 0;
     worker->request_count++;
     sw_atomic_fetch_add(&serv->stats->request_count, 1);
 }
@@ -273,14 +254,13 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
     swConnection *conn;
 #endif
 
-    factory->last_from_id = task->info.from_id;
     swWorker *worker = SwooleWG.worker;
     //worker busy
     worker->status = SW_WORKER_BUSY;
     //packet chunk
     if (task->info.flags & SW_EVENT_DATA_CHUNK)
     {
-        package = swWorker_get_buffer(serv, task->info.from_id);
+        package = swWorker_get_buffer(serv, task->info.reactor_id);
         //merge data to package buffer
         swString_append_ptr(package, task->data, task->info.len);
         //wait more data
@@ -292,9 +272,7 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
 
     switch (task->info.type)
     {
-    case SW_EVENT_TCP6:
-    case SW_EVENT_TCP:
-    case SW_EVENT_UNIX_STREAM:
+    case SW_SERVER_EVENT_SEND_DATA:
         //discard data
         if (swWorker_discard_data(serv, task) == SW_TRUE)
         {
@@ -303,32 +281,32 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
         swWorker_do_task(serv, worker, task, serv->onReceive);
         break;
 
-    case SW_EVENT_UDP:
-    case SW_EVENT_UDP6:
-    case SW_EVENT_UNIX_DGRAM:
+    case SW_SERVER_EVENT_SNED_DGRAM:
         swWorker_do_task(serv, worker, task, serv->onPacket);
         break;
 
-    case SW_EVENT_CLOSE:
+    case SW_SERVER_EVENT_CLOSE:
 #ifdef SW_USE_OPENSSL
         conn = swServer_connection_verify_no_ssl(serv, task->info.fd);
-        if (conn && conn->ssl_client_cert.length > 0)
+        if (conn && conn->ssl_client_cert && conn->ssl_client_cert_pid == SwooleG.pid)
         {
-            sw_free(conn->ssl_client_cert.str);
-            bzero(&conn->ssl_client_cert, sizeof(conn->ssl_client_cert.str));
+            sw_free(conn->ssl_client_cert);
+            conn->ssl_client_cert = nullptr;
         }
 #endif
         factory->end(factory, task->info.fd);
         break;
 
-    case SW_EVENT_CONNECT:
+    case SW_SERVER_EVENT_CONNECT:
 #ifdef SW_USE_OPENSSL
         //SSL client certificate
         if (task->info.len > 0)
         {
             conn = swServer_connection_verify_no_ssl(serv, task->info.fd);
-            conn->ssl_client_cert.str = sw_strndup(task->data, task->info.len);
-            conn->ssl_client_cert.size = conn->ssl_client_cert.length = task->info.len;
+            char *cert_data = NULL;
+            size_t length = swWorker_get_data(serv, task, &cert_data);
+            conn->ssl_client_cert = swString_dup(cert_data, length);
+            conn->ssl_client_cert_pid = SwooleG.pid;
         }
 #endif
         if (serv->onConnect)
@@ -337,25 +315,25 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
         }
         break;
 
-    case SW_EVENT_BUFFER_FULL:
+    case SW_SERVER_EVENT_BUFFER_FULL:
         if (serv->onBufferFull)
         {
             serv->onBufferFull(serv, &task->info);
         }
         break;
 
-    case SW_EVENT_BUFFER_EMPTY:
+    case SW_SERVER_EVENT_BUFFER_EMPTY:
         if (serv->onBufferEmpty)
         {
             serv->onBufferEmpty(serv, &task->info);
         }
         break;
 
-    case SW_EVENT_FINISH:
+    case SW_SERVER_EVENT_FINISH:
         serv->onFinish(serv, task);
         break;
 
-    case SW_EVENT_PIPE_MESSAGE:
+    case SW_SERVER_EVENT_PIPE_MESSAGE:
         serv->onPipeMessage(serv, task);
         break;
 
@@ -454,13 +432,9 @@ void swWorker_onStart(swServer *serv)
         {
             continue;
         }
-        else
-        {
-            swWorker_free(worker);
-        }
         if (swIsWorker())
         {
-            swSetNonBlock(worker->pipe_master);
+            swSocket_set_nonblock(worker->pipe_master);
         }
     }
 
@@ -480,9 +454,9 @@ void swWorker_onStart(swServer *serv)
     }
 
 #ifdef HAVE_SIGNALFD
-    if (SwooleG.use_signalfd && SwooleG.main_reactor && SwooleG.signal_fd == 0)
+    if (SwooleG.use_signalfd && SwooleTG.reactor && SwooleG.signal_fd == 0)
     {
-        swSignalfd_setup(SwooleG.main_reactor);
+        swSignalfd_setup(SwooleTG.reactor);
     }
 #endif
 
@@ -502,39 +476,34 @@ void swWorker_stop(swWorker *worker)
     swServer *serv = (swServer *) worker->pool->ptr;
     worker->status = SW_WORKER_BUSY;
 
+    swReactor *reactor = SwooleTG.reactor;
+
     /**
      * force to end
      */
     if (serv->reload_async == 0)
     {
         SwooleG.running = 0;
-        SwooleG.main_reactor->running = 0;
+        reactor->running = 0;
         return;
     }
 
     //The worker process is shutting down now.
-    if (SwooleWG.wait_exit)
+    if (reactor->wait_exit)
     {
         return;
     }
 
-    //remove read event
-    if (worker->pipe_worker)
-    {
-        swReactor_remove_read_event(SwooleG.main_reactor, worker->pipe_worker);
-    }
-
     if (serv->stream_fd > 0)
     {
-        SwooleG.main_reactor->del(SwooleG.main_reactor, serv->stream_fd);
+        reactor->del(reactor, serv->stream_fd);
         close(serv->stream_fd);
         serv->stream_fd = 0;
     }
 
-    if (serv->onWorkerStop)
+    if (worker->pipe_worker)
     {
-        serv->onWorkerStop(serv, SwooleWG.id);
-        serv->onWorkerStop = NULL;
+        swReactor_remove_read_event(reactor, worker->pipe_worker);
     }
 
     if (serv->factory_mode == SW_MODE_BASE)
@@ -542,19 +511,38 @@ void swWorker_stop(swWorker *worker)
         swListenPort *port;
         LL_FOREACH(serv->listen_list, port)
         {
-            SwooleG.main_reactor->del(SwooleG.main_reactor, port->sock);
-            swPort_free(port);
+            reactor->del(reactor, port->sock);
         }
-
-        if (worker->pipe_worker)
+        if (worker->pipe_master)
         {
-            SwooleG.main_reactor->del(SwooleG.main_reactor, worker->pipe_worker);
-            SwooleG.main_reactor->del(SwooleG.main_reactor, worker->pipe_master);
+            swReactor_remove_read_event(reactor, worker->pipe_master);
         }
+        int fd;
+        int serv_max_fd = swServer_get_maxfd(serv);
+        int serv_min_fd = swServer_get_minfd(serv);
 
-        goto try_to_exit;
+        for (fd = serv_min_fd; fd <= serv_max_fd; fd++)
+        {
+            swConnection *conn = swServer_connection_get(serv, fd);
+            if (conn && conn->socket && conn->active && !conn->peer_closed && conn->socket->fdtype == SW_FD_SESSION)
+            {
+                swReactor_remove_read_event(reactor, fd);
+            }
+        }
+        //clear timer
+        if (serv->master_timer)
+        {
+            swoole_timer_del(serv->master_timer);
+            serv->master_timer = NULL;
+        }
+        if (serv->heartbeat_timer)
+        {
+            swoole_timer_del(serv->heartbeat_timer);
+            serv->heartbeat_timer = NULL;
+        }
+        goto _try_to_exit;
     }
-
+    
     swWorkerStopMessage msg;
     msg.pid = SwooleG.pid;
     msg.worker_id = SwooleWG.id;
@@ -566,45 +554,39 @@ void swWorker_stop(swWorker *worker)
     }
     else
     {
-        swKill(serv->gs->manager_pid, SIGIO);
+        swoole_kill(serv->gs->manager_pid, SIGIO);
     }
 
-    try_to_exit:
-    SwooleWG.wait_exit = 1;
+    _try_to_exit: reactor->wait_exit = 1;
+    reactor->is_empty = swWorker_reactor_is_empty;
     SwooleWG.exit_time = serv->gs->now;
 
-    swWorker_try_to_exit();
+    if (swWorker_reactor_is_empty(reactor))
+    {
+        reactor->running = 0;
+        SwooleG.running = 0;
+    }
 }
 
-void swWorker_try_to_exit()
+static int swWorker_reactor_is_empty(swReactor *reactor)
 {
-    swWorker *worker = SwooleWG.worker;
-    swServer *serv = (swServer *) worker->pool->ptr;
-
-    //close all client connections
-    if (serv->factory_mode == SW_MODE_BASE)
+    swServer *serv;
+    if (SwooleG.process_type == SW_PROCESS_TASKWORKER)
     {
-        int find_fd = swServer_get_minfd(serv);
-        int max_fd = swServer_get_maxfd(serv);
-        swConnection *conn;
-        for (; find_fd <= max_fd; find_fd++)
-        {
-            conn = &serv->connection_list[find_fd];
-            if (conn->active == 1 && swSocket_is_stream(conn->socket_type) && !(conn->events & SW_EVENT_WRITE))
-            {
-                serv->close(serv, conn->session_id, 0);
-            }
-        }
+        swProcessPool *pool = (swProcessPool *) reactor->ptr;
+        serv = (swServer *) pool->ptr;
+    }
+    else
+    {
+        serv = (swServer *) reactor->ptr;
     }
-
     uint8_t call_worker_exit_func = 0;
 
     while (1)
     {
-        if (swReactor_empty(SwooleG.main_reactor))
+        if (swReactor_empty(reactor))
         {
-            SwooleG.main_reactor->running = 0;
-            SwooleG.running = 0;
+            return SW_TRUE;
         }
         else
         {
@@ -617,41 +599,39 @@ void swWorker_try_to_exit()
             int remaining_time = serv->max_wait_time - (time(NULL) - SwooleWG.exit_time);
             if (remaining_time <= 0)
             {
-                SwooleG.running = 0;
-                SwooleG.main_reactor->running = 0;
-                swoole_error_log(SW_LOG_WARNING, SW_ERROR_SERVER_WORKER_EXIT_TIMEOUT,
-                        "worker exit timeout, forced to terminate");
+                swoole_error_log(SW_LOG_WARNING, SW_ERROR_SERVER_WORKER_EXIT_TIMEOUT, "worker exit timeout, forced to terminate");
+                return SW_TRUE;
             }
             else
             {
                 int timeout_msec = remaining_time * 1000;
-                if (SwooleG.main_reactor->timeout_msec < 0 || SwooleG.main_reactor->timeout_msec > timeout_msec)
+                if (reactor->timeout_msec < 0 || reactor->timeout_msec > timeout_msec)
                 {
-                    SwooleG.main_reactor->timeout_msec = timeout_msec;
+                    reactor->timeout_msec = timeout_msec;
                 }
             }
         }
         break;
     }
+
+    return SW_FALSE;
 }
 
-void swWorker_clean(void)
+void swWorker_clean_pipe_buffer(swServer *serv)
 {
-    int i;
-    swServer *serv = (swServer *) SwooleWG.worker->pool->ptr;
-
+    uint32_t i;
     for (i = 0; i < serv->worker_num + serv->task_worker_num; i++)
     {
         swWorker *worker = swServer_get_worker(serv, i);
-        if (SwooleG.main_reactor)
+        if (SwooleTG.reactor)
         {
             if (worker->pipe_worker)
             {
-                swReactor_wait_write_buffer(SwooleG.main_reactor, worker->pipe_worker);
+                swReactor_wait_write_buffer(SwooleTG.reactor, worker->pipe_worker);
             }
             if (worker->pipe_master)
             {
-                swReactor_wait_write_buffer(SwooleG.main_reactor, worker->pipe_master);
+                swReactor_wait_write_buffer(SwooleTG.reactor, worker->pipe_master);
             }
         }
     }
@@ -668,51 +648,41 @@ int swWorker_loop(swServer *serv, int worker_id)
     swWorker *worker = swServer_get_worker(serv, worker_id);
     swServer_worker_init(serv, worker);
 
-    SwooleG.main_reactor = (swReactor *) sw_malloc(sizeof(swReactor));
-    if (SwooleG.main_reactor == NULL)
+    if (swoole_event_init() < 0)
     {
-        swError("[Worker] malloc for reactor failed");
         return SW_ERR;
     }
 
-    if (swReactor_create(SwooleG.main_reactor, SW_REACTOR_MAXEVENTS) < 0)
-    {
-        swError("[Worker] create worker_reactor failed");
-        return SW_ERR;
-    }
-
-    worker->status = SW_WORKER_IDLE;
-
-    int pipe_worker = worker->pipe_worker;
-
-    swSetNonBlock(pipe_worker);
-    SwooleG.main_reactor->ptr = serv;
-    SwooleG.main_reactor->add(SwooleG.main_reactor, pipe_worker, SW_FD_PIPE | SW_EVENT_READ);
-    SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_PIPE, swWorker_onPipeReceive);
-    SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_WRITE, swReactor_onWrite);
-
+    swReactor * reactor = SwooleTG.reactor;
     /**
      * set pipe buffer size
      */
-    int i;
-    swConnection *pipe_socket;
-    for (i = 0; i < serv->worker_num + serv->task_worker_num; i++)
+    for (uint32_t i = 0; i < serv->worker_num + serv->task_worker_num; i++)
     {
-        worker = swServer_get_worker(serv, i);
-        pipe_socket = swReactor_get(SwooleG.main_reactor, worker->pipe_master);
+        swWorker *_worker = swServer_get_worker(serv, i);
+        swSocket *pipe_socket;
+        pipe_socket = swReactor_get(reactor, _worker->pipe_master);
         pipe_socket->buffer_size = INT_MAX;
         pipe_socket->fdtype = SW_FD_PIPE;
-        pipe_socket = swReactor_get(SwooleG.main_reactor, worker->pipe_worker);
+        pipe_socket = swReactor_get(reactor, _worker->pipe_worker);
         pipe_socket->buffer_size = INT_MAX;
         pipe_socket->fdtype = SW_FD_PIPE;
     }
 
+    int pipe_worker = worker->pipe_worker;
+
+    swSocket_set_nonblock(pipe_worker);
+    reactor->ptr = serv;
+    reactor->add(reactor, pipe_worker, SW_FD_PIPE | SW_EVENT_READ);
+    swReactor_set_handler(reactor, SW_FD_PIPE, swWorker_onPipeReceive);
+
     if (serv->dispatch_mode == SW_DISPATCH_STREAM)
     {
-        SwooleG.main_reactor->add(SwooleG.main_reactor, serv->stream_fd, SW_FD_LISTEN | SW_EVENT_READ);
-        SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_LISTEN, swWorker_onStreamAccept);
-        SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM, swWorker_onStreamRead);
+        reactor->add(reactor, serv->stream_fd, SW_FD_STREAM_SERVER | SW_EVENT_READ);
+        swReactor_set_handler(reactor, SW_FD_STREAM_SERVER, swWorker_onStreamAccept);
+        swReactor_set_handler(reactor, SW_FD_STREAM, swWorker_onStreamRead);
         swStream_set_protocol(&serv->stream_protocol);
+        serv->stream_protocol.private_data_2 = serv;
         serv->stream_protocol.package_max_length = INT_MAX;
         serv->stream_protocol.onPackage = swWorker_onStreamPackage;
         serv->buffer_pool = swLinkedList_new(0, NULL);
@@ -722,12 +692,15 @@ int swWorker_loop(swServer *serv, int worker_id)
         }
     }
 
+    worker->status = SW_WORKER_IDLE;
     swWorker_onStart(serv);
 
     //main loop
-    SwooleG.main_reactor->wait(SwooleG.main_reactor, NULL);
+    reactor->wait(reactor, NULL);
     //clear pipe buffer
-    swWorker_clean();
+    swWorker_clean_pipe_buffer(serv);
+    //reactor free
+    swoole_event_free();
     //worker shutdown
     swWorker_onStop(serv);
     return SW_OK;
@@ -739,11 +712,11 @@ int swWorker_loop(swServer *serv, int worker_id)
 int swWorker_send2reactor(swServer *serv, swEventData *ev_data, size_t sendn, int session_id)
 {
     int ret;
-    int _pipe_fd = swWorker_get_send_pipe(serv, session_id, ev_data->info.from_id);
+    int _pipe_fd = swServer_get_send_pipe(serv, session_id, ev_data->info.reactor_id);
 
-    if (SwooleG.main_reactor)
+    if (SwooleTG.reactor)
     {
-        ret = SwooleG.main_reactor->write(SwooleG.main_reactor, _pipe_fd, ev_data, sendn);
+        ret = SwooleTG.reactor->write(SwooleTG.reactor, _pipe_fd, ev_data, sendn);
     }
     else
     {
@@ -762,7 +735,7 @@ static int swWorker_onPipeReceive(swReactor *reactor, swEvent *event)
     swPipeBuffer *buffer = serv->pipe_buffers[0];
     int ret;
 
-    read_from_pipe:
+    _read_from_pipe:
 
     if (read(event->fd, buffer, serv->ipc_max_size) > 0)
     {
@@ -781,7 +754,7 @@ static int swWorker_onPipeReceive(swReactor *reactor, swEvent *event)
             }
             else if (ret > 0)
             {
-                goto read_from_pipe;
+                goto _read_from_pipe;
             }
         }
         return ret;
@@ -789,7 +762,7 @@ static int swWorker_onPipeReceive(swReactor *reactor, swEvent *event)
     return SW_ERR;
 }
 
-int swWorker_send2worker(swWorker *dst_worker, void *buf, int n, int flag)
+int swWorker_send2worker(swWorker *dst_worker, const void *buf, int n, int flag)
 {
     int pipefd, ret;
 
@@ -817,9 +790,9 @@ int swWorker_send2worker(swWorker *dst_worker, void *buf, int n, int flag)
         return swMsgQueue_push(dst_worker->pool->queue, (swQueue_data *) &msg, n);
     }
 
-    if ((flag & SW_PIPE_NONBLOCK) && SwooleG.main_reactor)
+    if ((flag & SW_PIPE_NONBLOCK) && SwooleTG.reactor)
     {
-        return SwooleG.main_reactor->write(SwooleG.main_reactor, pipefd, buf, n);
+        return SwooleTG.reactor->write(SwooleTG.reactor, pipefd, buf, n);
     }
     else
     {
