@@ -13,8 +13,8 @@
   | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
   +----------------------------------------------------------------------+
 */
-#include "php_swoole.h"
-#include "zend_variables.h"
+#include "php_swoole_cxx.h"
+#include "php_swoole_library.h"
 
 #if (HAVE_PCRE || HAVE_BUNDLED_PCRE) && !defined(COMPILE_DL_PCRE)
 #include "ext/pcre/php_pcre.h"
@@ -28,71 +28,17 @@
 #include <ifaddrs.h>
 #include <sys/ioctl.h>
 
+#include "zend_exceptions.h"
+
 ZEND_DECLARE_MODULE_GLOBALS(swoole)
 
 extern sapi_module_struct sapi_module;
 
+static swoole::CallbackManager rshutdown_callbacks;
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
-//arginfo event
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_add, 0, 0, 2)
-    ZEND_ARG_INFO(0, fd)
-    ZEND_ARG_CALLABLE_INFO(0, read_callback, 1)
-    ZEND_ARG_CALLABLE_INFO(0, write_callback, 1)
-    ZEND_ARG_INFO(0, events)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_set, 0, 0, 1)
-    ZEND_ARG_INFO(0, fd)
-    ZEND_ARG_CALLABLE_INFO(0, read_callback, 1)
-    ZEND_ARG_CALLABLE_INFO(0, write_callback, 1)
-    ZEND_ARG_INFO(0, events)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_write, 0, 0, 2)
-    ZEND_ARG_INFO(0, fd)
-    ZEND_ARG_INFO(0, data)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_defer, 0, 0, 1)
-    ZEND_ARG_CALLABLE_INFO(0, callback, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_cycle, 0, 0, 1)
-    ZEND_ARG_CALLABLE_INFO(0, callback, 1)
-    ZEND_ARG_INFO(0, before)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_del, 0, 0, 1)
-    ZEND_ARG_INFO(0, fd)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_isset, 0, 0, 1)
-    ZEND_ARG_INFO(0, fd)
-    ZEND_ARG_INFO(0, events)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_timer_tick, 0, 0, 2)
-    ZEND_ARG_INFO(0, ms)
-    ZEND_ARG_CALLABLE_INFO(0, callback, 0)
-    ZEND_ARG_VARIADIC_INFO(0, params)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_timer_after, 0, 0, 2)
-    ZEND_ARG_INFO(0, ms)
-    ZEND_ARG_CALLABLE_INFO(0, callback, 0)
-    ZEND_ARG_VARIADIC_INFO(0, params)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_timer_exists, 0, 0, 1)
-    ZEND_ARG_INFO(0, timer_id)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_timer_clear, 0, 0, 1)
-    ZEND_ARG_INFO(0, timer_id)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_async_set, 0, 0, 1)
     ZEND_ARG_ARRAY_INFO(0, settings, 0)
 ZEND_END_ARG_INFO()
@@ -111,10 +57,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_defer, 0, 0, 1)
     ZEND_ARG_CALLABLE_INFO(0, callback, 0)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_exec, 0, 0, 1)
-    ZEND_ARG_INFO(0, command)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_select, 0, 0, 3)
     ZEND_ARG_INFO(1, read_array)
     ZEND_ARG_INFO(1, write_array)
@@ -140,10 +82,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_get_mime_type, 0, 0, 1)
     ZEND_ARG_INFO(0, filename)
 ZEND_END_ARG_INFO()
 
-//arginfo end
-
-#include "zend_exceptions.h"
-
 static PHP_FUNCTION(swoole_get_mime_type);
 static PHP_FUNCTION(swoole_hashcode);
 
@@ -152,28 +90,11 @@ const zend_function_entry swoole_functions[] =
     PHP_FE(swoole_version, arginfo_swoole_void)
     PHP_FE(swoole_cpu_num, arginfo_swoole_void)
     PHP_FE(swoole_last_error, arginfo_swoole_void)
-    /*------swoole_event-----*/
-    PHP_FE(swoole_event_add, arginfo_swoole_event_add)
-    PHP_FE(swoole_event_set, arginfo_swoole_event_set)
-    PHP_FE(swoole_event_del, arginfo_swoole_event_del)
-    PHP_FE(swoole_event_exit, arginfo_swoole_void)
-    PHP_FE(swoole_event_wait, arginfo_swoole_void)
-    PHP_FE(swoole_event_write, arginfo_swoole_event_write)
-    PHP_FE(swoole_event_defer, arginfo_swoole_event_defer)
-    PHP_FE(swoole_event_cycle, arginfo_swoole_event_cycle)
-    PHP_FE(swoole_event_dispatch, arginfo_swoole_void)
-    PHP_FE(swoole_event_isset, arginfo_swoole_event_isset)
-    /*------swoole_timer-----*/
-    PHP_FE(swoole_timer_after, arginfo_swoole_timer_after)
-    PHP_FE(swoole_timer_tick, arginfo_swoole_timer_tick)
-    PHP_FE(swoole_timer_exists, arginfo_swoole_timer_exists)
-    PHP_FE(swoole_timer_clear, arginfo_swoole_timer_clear)
     /*------swoole_async_io------*/
     PHP_FE(swoole_async_dns_lookup_coro, arginfo_swoole_async_dns_lookup_coro)
     PHP_FE(swoole_async_set, arginfo_swoole_async_set)
     /*------swoole_coroutine------*/
     PHP_FE(swoole_coroutine_create, arginfo_swoole_coroutine_create)
-    PHP_FE(swoole_coroutine_exec, arginfo_swoole_coroutine_exec)
     PHP_FE(swoole_coroutine_defer, arginfo_swoole_coroutine_defer)
     PHP_FALIAS(go, swoole_coroutine_create, arginfo_swoole_coroutine_create)
     PHP_FALIAS(defer, swoole_coroutine_defer, arginfo_swoole_coroutine_defer)
@@ -192,52 +113,21 @@ const zend_function_entry swoole_functions[] =
     PHP_FE_END /* Must be the last line in swoole_functions[] */
 };
 
-static const zend_function_entry swoole_timer_methods[] =
-{
-    ZEND_FENTRY(tick, ZEND_FN(swoole_timer_tick), arginfo_swoole_timer_tick, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(after, ZEND_FN(swoole_timer_after), arginfo_swoole_timer_after, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(exists, ZEND_FN(swoole_timer_exists), arginfo_swoole_timer_exists, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(clear, ZEND_FN(swoole_timer_clear), arginfo_swoole_timer_clear, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_FE_END
-};
-
-static const zend_function_entry swoole_event_methods[] =
-{
-    ZEND_FENTRY(add, ZEND_FN(swoole_event_add), arginfo_swoole_event_add, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(del, ZEND_FN(swoole_event_del), arginfo_swoole_event_del, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(set, ZEND_FN(swoole_event_set), arginfo_swoole_event_set, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(exit, ZEND_FN(swoole_event_exit), arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(write, ZEND_FN(swoole_event_write), arginfo_swoole_event_write, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(wait, ZEND_FN(swoole_event_wait), arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(defer, ZEND_FN(swoole_event_defer), arginfo_swoole_event_defer, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(cycle, ZEND_FN(swoole_event_cycle), arginfo_swoole_event_cycle, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(dispatch, ZEND_FN(swoole_event_dispatch), arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    ZEND_FENTRY(isset, ZEND_FN(swoole_event_isset), arginfo_swoole_event_isset, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_FE_END
-};
-
 #if PHP_MEMORY_DEBUG
 php_vmstat_t php_vmstat;
 #endif
 
-static zend_class_entry *swoole_timer_ce;
-static zend_object_handlers swoole_timer_handlers;
-
-static zend_class_entry *swoole_event_ce;
-static zend_object_handlers swoole_event_handlers;
-
 zend_class_entry *swoole_exception_ce;
 zend_object_handlers swoole_exception_handlers;
 
+zend_class_entry *swoole_error_ce;
+zend_object_handlers swoole_error_handlers;
+
 zend_module_entry swoole_module_entry =
 {
-#if ZEND_MODULE_API_NO >= 20050922
     STANDARD_MODULE_HEADER_EX,
     NULL,
     NULL,
-#else
-    STANDARD_MODULE_HEADER,
-#endif
     "swoole",
     swoole_functions,
     PHP_MINIT(swoole),
@@ -261,6 +151,11 @@ PHP_INI_BEGIN()
  * enable swoole coroutine
  */
 STD_ZEND_INI_BOOLEAN("swoole.enable_coroutine", "On", PHP_INI_ALL, OnUpdateBool, enable_coroutine, zend_swoole_globals, swoole_globals)
+STD_ZEND_INI_BOOLEAN("swoole.enable_library", "On", PHP_INI_ALL, OnUpdateBool, enable_library, zend_swoole_globals, swoole_globals)
+/**
+ * enable swoole coroutine epreemptive scheduler
+ */
+STD_ZEND_INI_BOOLEAN("swoole.enable_preemptive_scheduler", "Off", PHP_INI_ALL, OnUpdateBool, enable_preemptive_scheduler, zend_swoole_globals, swoole_globals)
 /**
  * display error
  */
@@ -278,114 +173,20 @@ PHP_INI_END()
 static void php_swoole_init_globals(zend_swoole_globals *swoole_globals)
 {
     swoole_globals->enable_coroutine = 1;
+    swoole_globals->enable_library = 1;
+    swoole_globals->enable_preemptive_scheduler = 0;
     swoole_globals->socket_buffer_size = SW_SOCKET_BUFFER_SIZE;
     swoole_globals->display_errors = 1;
     swoole_globals->use_shortname = 1;
     swoole_globals->rshutdown_functions = NULL;
 }
 
-#if PHP_VERSION_ID < 80000
-void php_swoole_class_unset_property_deny(zval *zobject, zval *zmember, void **cache_slot)
-{
-    if (EXPECTED(zend_hash_find(&(Z_OBJCE_P(zobject)->properties_info), Z_STR_P(zmember))))
-    {
-        zend_throw_error(NULL, "Property %s of class %s cannot be unset", Z_STRVAL_P(zmember), ZSTR_VAL(Z_OBJCE_P(zobject)->name));
-        return;
-    }
-    std_object_handlers.unset_property(zobject, zmember, cache_slot);
-}
-#else
-void php_swoole_class_unset_property_deny(zend_object *object, zend_string *member, void **cache_slot)
-{
-    if (EXPECTED(zend_hash_find(object->properties, member)))
-    {
-        zend_throw_error(NULL, "Property %s of class %s cannot be unset", ZSTR_VAL(member), ZSTR_VAL(object->ce->name));
-        return;
-    }
-    std_object_handlers.unset_property(object, member, cache_slot);
-}
-#endif
-
-ssize_t php_swoole_length_func(swProtocol *protocol, swConnection *conn, char *data, uint32_t length)
-{
-    SwooleG.lock.lock(&SwooleG.lock);
-
-    zval *retval = NULL;
-
-    zval args[1];
-    ZVAL_STRINGL(&args[0], data, length);
-
-    zval *callback = (zval *) protocol->private_data;
-
-    if (sw_call_user_function_ex(EG(function_table), NULL, callback, &retval, 1, args, 0, NULL) == FAILURE)
-    {
-        swoole_php_fatal_error(E_WARNING, "length function handler error");
-        goto error;
-    }
-    zval_ptr_dtor(&args[0]);
-    if (UNEXPECTED(EG(exception)))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-        goto error;
-    }
-    if (retval)
-    {
-        ssize_t length = zval_get_long(retval);
-        zval_ptr_dtor(retval);
-        SwooleG.lock.unlock(&SwooleG.lock);
-        return length;
-    }
-    error:
-    SwooleG.lock.unlock(&SwooleG.lock);
-    return -1;
-}
-
-int php_swoole_dispatch_func(swServer *serv, swConnection *conn, swSendData *data)
-{
-    zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache*) serv->private_data_3;
-    zval args[4];
-    zval *zserver = &args[0], *zfd = &args[1], *ztype = &args[2], *zdata = NULL;
-    zval _retval, *retval = &_retval;
-    int worker_id = -1;
-
-    SwooleG.lock.lock(&SwooleG.lock);
-    *zserver = *((zval *) serv->ptr2);
-    ZVAL_LONG(zfd, (zend_long) (conn ? conn->session_id : data->info.fd));
-    ZVAL_LONG(ztype, (zend_long) data->info.type);
-    if (sw_zend_function_max_num_args(fci_cache->function_handler) > 3)
-    {
-        // optimization: reduce memory copy
-        zdata = &args[3];
-        ZVAL_STRINGL(zdata, data->data, data->info.len > SW_IPC_BUFFER_SIZE ? SW_IPC_BUFFER_SIZE : data->info.len);
-    }
-    if (sw_call_user_function_fast_ex(NULL, fci_cache, retval, zdata ? 4 : 3, args) == FAILURE)
-    {
-        swoole_php_fatal_error(E_WARNING, "dispatch function handler error");
-    }
-    else if (!ZVAL_IS_NULL(retval))
-    {
-        worker_id = (int) zval_get_long(retval);
-        if (worker_id >= serv->worker_num)
-        {
-            swoole_php_fatal_error(E_WARNING, "invalid target worker-id[%d]", worker_id);
-            worker_id = -1;
-        }
-        zval_ptr_dtor(retval);
-    }
-    if (zdata)
-    {
-        zval_ptr_dtor(zdata);
-    }
-    SwooleG.lock.unlock(&SwooleG.lock);
-    return worker_id;
-}
-
 static sw_inline uint32_t swoole_get_new_size(uint32_t old_size, int handle)
 {
     uint32_t new_size = old_size * 2;
     if (handle > SWOOLE_OBJECT_MAX)
     {
-        swoole_php_fatal_error(E_ERROR, "handle %d exceed %d", handle, SWOOLE_OBJECT_MAX);
+        php_swoole_fatal_error(E_ERROR, "handle %d exceed %d", handle, SWOOLE_OBJECT_MAX);
         return 0;
     }
     while (new_size <= (uint32_t) handle)
@@ -403,7 +204,7 @@ void swoole_set_object_by_handle(uint32_t handle, void *ptr)
 {
     assert(handle < SWOOLE_OBJECT_MAX);
 
-    if (unlikely(handle >= swoole_objects.size))
+    if (sw_unlikely(handle >= swoole_objects.size))
     {
         uint32_t old_size = swoole_objects.size;
         uint32_t new_size = swoole_get_new_size(old_size, handle);
@@ -414,7 +215,7 @@ void swoole_set_object_by_handle(uint32_t handle, void *ptr)
         new_ptr = sw_realloc(old_ptr, sizeof(void*) * new_size);
         if (!new_ptr)
         {
-            swoole_php_fatal_error(E_ERROR, "malloc(%d) failed", (int )(new_size * sizeof(void *)));
+            php_swoole_fatal_error(E_ERROR, "malloc(%d) failed", (int )(new_size * sizeof(void *)));
             return;
         }
         bzero((char*) new_ptr + (old_size * sizeof(void*)), (new_size - old_size) * sizeof(void*));
@@ -434,7 +235,7 @@ void swoole_set_property_by_handle(uint32_t handle, int property_id, void *ptr)
 {
     assert(handle < SWOOLE_OBJECT_MAX);
 
-    if (unlikely(handle >= swoole_objects.property_size[property_id]))
+    if (sw_unlikely(handle >= swoole_objects.property_size[property_id]))
     {
         uint32_t old_size = swoole_objects.property_size[property_id];
         uint32_t new_size = 0;
@@ -455,7 +256,7 @@ void swoole_set_property_by_handle(uint32_t handle, int property_id, void *ptr)
         }
         if (new_ptr == NULL)
         {
-            swoole_php_fatal_error(E_ERROR, "malloc(%d) failed", (int )(new_size * sizeof(void *)));
+            php_swoole_fatal_error(E_ERROR, "malloc(%d) failed", (int )(new_size * sizeof(void *)));
             return;
         }
         if (old_size > 0)
@@ -474,107 +275,30 @@ void swoole_set_property_by_handle(uint32_t handle, int property_id, void *ptr)
     swoole_objects.property[property_id][handle] = ptr;
 }
 
-int swoole_register_rshutdown_function(swCallback func, int push_back)
-{
-    if (SWOOLE_G(rshutdown_functions) == NULL)
-    {
-        SWOOLE_G(rshutdown_functions) = swLinkedList_new(0, NULL);
-        if (SWOOLE_G(rshutdown_functions) == NULL)
-        {
-            return SW_ERR;
-        }
-    }
-    if (push_back)
-    {
-        return swLinkedList_append(SWOOLE_G(rshutdown_functions), (void*) func);
-    }
-    else
-    {
-        return swLinkedList_prepend(SWOOLE_G(rshutdown_functions), (void*) func);
-    }
-}
-
 void php_swoole_register_shutdown_function(const char *function)
 {
     php_shutdown_function_entry shutdown_function_entry;
     shutdown_function_entry.arg_count = 1;
     shutdown_function_entry.arguments = (zval *) safe_emalloc(sizeof(zval), 1, 0);
     ZVAL_STRING(&shutdown_function_entry.arguments[0], function);
-    register_user_shutdown_function((char *) function, ZSTR_LEN(Z_STR(shutdown_function_entry.arguments[0])),
-            &shutdown_function_entry);
+    register_user_shutdown_function((char *) function, ZSTR_LEN(Z_STR(shutdown_function_entry.arguments[0])), &shutdown_function_entry);
 }
 
-static void php_swoole_old_shutdown_function_move(zval *zv)
+void php_swoole_register_rshutdown_callback(swCallback cb, void *private_data)
 {
-    php_shutdown_function_entry *old_shutdown_function_entry = (php_shutdown_function_entry *) Z_PTR_P(zv);
-    zend_hash_next_index_insert_mem(BG(user_shutdown_function_names), old_shutdown_function_entry, sizeof(php_shutdown_function_entry));
-    efree(old_shutdown_function_entry);
+    rshutdown_callbacks.append(cb, private_data);
 }
 
-void php_swoole_register_shutdown_function_prepend(const char *function)
-{
-    HashTable *old_user_shutdown_function_names = BG(user_shutdown_function_names);
-    if (!old_user_shutdown_function_names)
-    {
-        php_swoole_register_shutdown_function(function);
-        return;
-    }
-    BG(user_shutdown_function_names) = NULL;
-    php_swoole_register_shutdown_function(function);
-
-#ifdef SW_CORO_ZEND_TRY
-    zend_try
-#endif
-    {
-        old_user_shutdown_function_names->pDestructor = php_swoole_old_shutdown_function_move;
-        zend_hash_destroy(old_user_shutdown_function_names);
-        FREE_HASHTABLE(old_user_shutdown_function_names);
-    }
-#ifdef SW_CORO_ZEND_TRY
-    zend_end_try();
-#endif
-}
-
-static void php_swoole_fatal_error(int code, const char *format, ...)
+static void fatal_error(int code, const char *format, ...)
 {
     va_list args;
-    swString *buffer = SwooleTG.buffer_stack;
-    zend_string *backtrace;
-    const char *space, *class_name = get_active_class_name(&space);
-
-    SwooleGS->lock_2.lock(&SwooleGS->lock_2);
-    swString_clear(buffer);
-    buffer->length += sw_snprintf(buffer->str, buffer->size, "(PHP Fatal Error: %d):\n%s%s%s: ", code, class_name, space, get_active_function_name());
+    zend_object *exception;
     va_start(args, format);
-    buffer->length += sw_vsnprintf(buffer->str + buffer->length, buffer->size - buffer->length, format, args);
+    exception = zend_throw_error_exception(swoole_error_ce, swoole::cpp_string::vformat(format, args).c_str(), code, E_ERROR);
     va_end(args);
-    swString_append_ptr(buffer, SW_STRL("\nStack trace:\n"));
-    backtrace = sw_get_debug_print_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 0);
-    if (likely(backtrace))
-    {
-        swString_append_ptr(buffer, ZSTR_VAL(backtrace), ZSTR_LEN(backtrace));
-        zend_string_release(backtrace);
-    }
-    SwooleG.write_log(SW_LOG_ERROR, buffer->str, buffer->length);
-    SwooleGS->lock_2.unlock(&SwooleGS->lock_2);
-    exit(255);
-}
-
-void swoole_call_rshutdown_function(void *arg)
-{
-    if (SWOOLE_G(rshutdown_functions))
-    {
-        swLinkedList *rshutdown_functions = SWOOLE_G(rshutdown_functions);
-        swLinkedList_node *node = rshutdown_functions->head;
-        swCallback func = NULL;
-
-        while (node)
-        {
-            func = (swCallback) node->data;
-            func(arg);
-            node = node->next;
-        }
-    }
+    zend_exception_error(exception, E_ERROR);
+    // should never here
+    exit(1);
 }
 
 swoole_object_array swoole_objects;
@@ -634,8 +358,8 @@ PHP_MINIT_FUNCTION(swoole)
     /**
      * simple api
      */
-    SW_REGISTER_LONG_CONSTANT("SWOOLE_SOCK_SYNC", SW_SOCK_SYNC);
-    SW_REGISTER_LONG_CONSTANT("SWOOLE_SOCK_ASYNC", SW_SOCK_ASYNC);
+    SW_REGISTER_BOOL_CONSTANT("SWOOLE_SOCK_SYNC", SW_SOCK_SYNC);
+    SW_REGISTER_BOOL_CONSTANT("SWOOLE_SOCK_ASYNC", SW_SOCK_ASYNC);
 
     SW_REGISTER_LONG_CONSTANT("SWOOLE_SYNC", SW_FLAG_SYNC);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ASYNC", SW_FLAG_ASYNC);
@@ -708,21 +432,26 @@ PHP_MINIT_FUNCTION(swoole)
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SESSION_INVALID_ID", SW_ERROR_SESSION_INVALID_ID);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SESSION_DISCARD_TIMEOUT_DATA", SW_ERROR_SESSION_DISCARD_TIMEOUT_DATA);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_OUTPUT_BUFFER_OVERFLOW", SW_ERROR_OUTPUT_BUFFER_OVERFLOW);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_OUTPUT_SEND_YIELD", SW_ERROR_OUTPUT_SEND_YIELD);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_NOT_READY", SW_ERROR_SSL_NOT_READY);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_CANNOT_USE_SENFILE", SW_ERROR_SSL_CANNOT_USE_SENFILE);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_EMPTY_PEER_CERTIFICATE", SW_ERROR_SSL_EMPTY_PEER_CERTIFICATE);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_VEFIRY_FAILED", SW_ERROR_SSL_VEFIRY_FAILED);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_BAD_CLIENT", SW_ERROR_SSL_BAD_CLIENT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_BAD_PROTOCOL", SW_ERROR_SSL_BAD_PROTOCOL);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SSL_RESET", SW_ERROR_SSL_RESET);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_PACKAGE_LENGTH_TOO_LARGE", SW_ERROR_PACKAGE_LENGTH_TOO_LARGE);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_PACKAGE_LENGTH_NOT_FOUND", SW_ERROR_PACKAGE_LENGTH_NOT_FOUND);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_DATA_LENGTH_TOO_LARGE", SW_ERROR_DATA_LENGTH_TOO_LARGE);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_TASK_PACKAGE_TOO_BIG", SW_ERROR_TASK_PACKAGE_TOO_BIG);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_TASK_DISPATCH_FAIL", SW_ERROR_TASK_DISPATCH_FAIL);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_TASK_TIMEOUT", SW_ERROR_TASK_TIMEOUT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_HTTP2_STREAM_ID_TOO_BIG", SW_ERROR_HTTP2_STREAM_ID_TOO_BIG);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_HTTP2_STREAM_NO_HEADER", SW_ERROR_HTTP2_STREAM_NO_HEADER);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_HTTP2_STREAM_NOT_FOUND", SW_ERROR_HTTP2_STREAM_NOT_FOUND);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_AIO_BAD_REQUEST", SW_ERROR_AIO_BAD_REQUEST);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_AIO_CANCELED", SW_ERROR_AIO_CANCELED);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_AIO_TIMEOUT", SW_ERROR_AIO_TIMEOUT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CLIENT_NO_CONNECTION", SW_ERROR_CLIENT_NO_CONNECTION);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SOCKET_CLOSED", SW_ERROR_SOCKET_CLOSED);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SOCKS5_UNSUPPORT_VERSION", SW_ERROR_SOCKS5_UNSUPPORT_VERSION);
@@ -749,6 +478,7 @@ PHP_MINIT_FUNCTION(swoole)
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_SERVER_WORKER_EXIT_TIMEOUT", SW_ERROR_SERVER_WORKER_EXIT_TIMEOUT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CO_OUT_OF_COROUTINE", SW_ERROR_CO_OUT_OF_COROUTINE);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CO_HAS_BEEN_BOUND", SW_ERROR_CO_HAS_BEEN_BOUND);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CO_HAS_BEEN_DISCARDED", SW_ERROR_CO_HAS_BEEN_DISCARDED);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CO_MUTEX_DOUBLE_UNLOCK", SW_ERROR_CO_MUTEX_DOUBLE_UNLOCK);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CO_BLOCK_OBJECT_LOCKED", SW_ERROR_CO_BLOCK_OBJECT_LOCKED);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_ERROR_CO_BLOCK_OBJECT_WAITING", SW_ERROR_CO_BLOCK_OBJECT_WAITING);
@@ -772,13 +502,14 @@ PHP_MINIT_FUNCTION(swoole)
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_WORKER", SW_TRACE_WORKER);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_REACTOR", SW_TRACE_REACTOR);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_PHP", SW_TRACE_PHP);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_HTTP", SW_TRACE_HTTP);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_HTTP2", SW_TRACE_HTTP2);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_EOF_PROTOCOL", SW_TRACE_EOF_PROTOCOL);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_LENGTH_PROTOCOL", SW_TRACE_LENGTH_PROTOCOL);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_CLOSE", SW_TRACE_CLOSE);
-    SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_HTTP_CLIENT", SW_TRACE_HTTP_CLIENT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_REDIS_CLIENT", SW_TRACE_REDIS_CLIENT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_MYSQL_CLIENT", SW_TRACE_MYSQL_CLIENT);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_HTTP_CLIENT", SW_TRACE_HTTP_CLIENT);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_AIO", SW_TRACE_AIO);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_SSL", SW_TRACE_SSL);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_NORMAL", SW_TRACE_NORMAL);
@@ -787,6 +518,7 @@ PHP_MINIT_FUNCTION(swoole)
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_SOCKET", SW_TRACE_SOCKET);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_COROUTINE", SW_TRACE_COROUTINE);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_CONTEXT", SW_TRACE_CONTEXT);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_CO_HTTP_SERVER", SW_TRACE_CO_HTTP_SERVER);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_TRACE_ALL", SW_TRACE_ALL);
 
     /**
@@ -810,69 +542,71 @@ PHP_MINIT_FUNCTION(swoole)
         zend_hash_str_del(CG(function_table), ZEND_STRL("defer"));
     }
 
-    SW_INIT_CLASS_ENTRY(swoole_timer, "Swoole\\Timer", "swoole_timer", NULL, swoole_timer_methods);
-    SW_SET_CLASS_SERIALIZABLE(swoole_timer, zend_class_serialize_deny, zend_class_unserialize_deny);
-    SW_SET_CLASS_CLONEABLE(swoole_timer, zend_class_clone_deny);
-    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_timer, zend_class_unset_property_deny);
-
-    SW_INIT_CLASS_ENTRY(swoole_event, "Swoole\\Event", "swoole_event", NULL, swoole_event_methods);
-    SW_SET_CLASS_SERIALIZABLE(swoole_event, zend_class_serialize_deny, zend_class_unserialize_deny);
-    SW_SET_CLASS_CLONEABLE(swoole_event, zend_class_clone_deny);
-    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_event, zend_class_unset_property_deny);
-
-    SW_INIT_EXCEPTION_CLASS_ENTRY(swoole_exception, "Swoole\\Exception", "swoole_exception", NULL, NULL);
-
-    //swoole init
     swoole_init();
     if (!SWOOLE_G(enable_coroutine))
     {
         SwooleG.enable_coroutine = 0;
     }
-    if (strcasecmp("cli", sapi_module.name) == 0)
+    if (strcmp("cli", sapi_module.name) == 0 || strcmp("phpdbg", sapi_module.name) == 0)
     {
         SWOOLE_G(cli) = 1;
     }
 
-    swoole_server_init(module_number);
-    swoole_server_port_init(module_number);
-    swoole_client_init(module_number);
-    swoole_socket_coro_init(module_number);
-    swoole_client_coro_init(module_number);
-    swoole_redis_coro_init(module_number);
-#ifdef SW_USE_POSTGRESQL
-    swoole_postgresql_coro_init(module_number);
-#endif
-    swoole_mysql_coro_init(module_number);
-    swoole_http_client_coro_init(module_number);
-	swoole_coroutine_util_init(module_number);
-    swoole_async_coro_init(module_number);
-    swoole_process_init(module_number);
-    swoole_process_pool_init(module_number);
-    swoole_table_init(module_number);
-    swoole_runtime_init(module_number);
-    swoole_lock_init(module_number);
-    swoole_atomic_init(module_number);
-    swoole_http_server_init(module_number);
-    swoole_buffer_init(module_number);
-    swoole_websocket_init(module_number);
-    swoole_channel_coro_init(module_number);
-#ifdef SW_USE_HTTP2
-    swoole_http2_client_coro_init(module_number);
-#endif
-#ifdef SW_USE_FAST_SERIALIZE
-    swoole_serialize_init(module_number);
-#endif
-    swoole_redis_server_init(module_number);
+    SW_INIT_CLASS_ENTRY_EX2(swoole_exception, "Swoole\\Exception", "swoole_exception", NULL, NULL, zend_ce_exception, zend_get_std_object_handlers());
 
-    SwooleG.fatal_error = php_swoole_fatal_error;
+    SW_INIT_CLASS_ENTRY_EX2(swoole_error, "Swoole\\Error", "swoole_error", NULL, NULL, zend_ce_error, zend_get_std_object_handlers());
+
+    /** <Sort by dependency> **/
+    php_swoole_event_minit(module_number);
+    // base
+    php_swoole_atomic_minit(module_number);
+    php_swoole_buffer_minit(module_number);
+    php_swoole_lock_minit(module_number);
+    php_swoole_process_minit(module_number);
+    php_swoole_process_pool_minit(module_number);
+    php_swoole_table_minit(module_number);
+    php_swoole_timer_minit(module_number);
+    // coroutine
+    php_swoole_async_coro_minit(module_number);
+    php_swoole_coroutine_minit(module_number);
+    php_swoole_coroutine_system_minit(module_number);
+    php_swoole_coroutine_scheduler_minit(module_number);
+    php_swoole_channel_coro_minit(module_number);
+    php_swoole_runtime_minit(module_number);
+    // client
+    php_swoole_socket_coro_minit(module_number);
+    php_swoole_client_minit(module_number);
+    php_swoole_client_coro_minit(module_number);
+    php_swoole_http_client_coro_minit(module_number);
+    php_swoole_mysql_coro_minit(module_number);
+    php_swoole_redis_coro_minit(module_number);
+#ifdef SW_USE_HTTP2
+    php_swoole_http2_client_coro_minit(module_number);
+#endif
+    // server
+    php_swoole_server_minit(module_number);
+    php_swoole_server_port_minit(module_number);
+    php_swoole_http_request_minit(module_number);
+    php_swoole_http_response_minit(module_number);
+    php_swoole_http_server_minit(module_number);
+    php_swoole_http_server_coro_minit(module_number);
+    php_swoole_websocket_server_minit(module_number);
+    php_swoole_redis_server_minit(module_number);
+
+    SwooleG.fatal_error = fatal_error;
     SwooleG.socket_buffer_size = SWOOLE_G(socket_buffer_size);
     SwooleG.dns_cache_refresh_time = 60;
 
     swoole_objects.size = SWOOLE_OBJECT_DEFAULT;
     swoole_objects.array = (void**) sw_calloc(swoole_objects.size, sizeof(void*));
+    if (!swoole_objects.array)
+    {
+        php_swoole_fatal_error(E_ERROR, "malloc([swoole_objects]) failed");
+        exit(253);
+    }
 
     // enable pcre.jit and use swoole extension on MacOS will lead to coredump, disable it temporarily
-#if defined(PHP_PCRE_VERSION) && PHP_VERSION_ID >= 70300 && __MACH__ && !defined(SW_DEBUG)
+#if defined(PHP_PCRE_VERSION) && defined(HAVE_PCRE_JIT_SUPPORT) && PHP_VERSION_ID >= 70300 && __MACH__ && !defined(SW_DEBUG)
     PCRE_G(jit) = 0;
 #endif
 
@@ -880,7 +614,7 @@ PHP_MINIT_FUNCTION(swoole)
 }
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION
+/* {{{ PHP_MSHUTDOWN_FUNCTION
  */
 PHP_MSHUTDOWN_FUNCTION(swoole)
 {
@@ -902,9 +636,7 @@ PHP_MINFO_FUNCTION(swoole)
     php_info_print_table_row(2, "Version", SWOOLE_VERSION);
     snprintf(buf, sizeof(buf), "%s %s", __DATE__, __TIME__);
     php_info_print_table_row(2, "Built", buf);
-#ifdef SW_COROUTINE
     php_info_print_table_row(2, "coroutine", "enabled");
-#endif
 #ifdef SW_DEBUG
     php_info_print_table_row(2, "debug", "enabled");
 #endif
@@ -912,12 +644,8 @@ PHP_MINFO_FUNCTION(swoole)
     php_info_print_table_row(2, "trace_log", "enabled");
 #endif
 #ifdef SW_NO_USE_ASM_CONTEXT
-#ifdef HAVE_BOOST_CONTEXT
-    php_info_print_table_row(2, "boost.context", "enabled");
-#else
     php_info_print_table_row(2, "ucontext", "enabled");
 #endif
-#endif
 #ifdef HAVE_EPOLL
     php_info_print_table_row(2, "epoll", "enabled");
 #endif
@@ -1002,8 +730,12 @@ PHP_MINFO_FUNCTION(swoole)
 PHP_RINIT_FUNCTION(swoole)
 {
     SWOOLE_G(req_status) = PHP_SWOOLE_RINIT_BEGIN;
-    php_swoole_register_shutdown_function("swoole_internal_call_user_shutdown_begin");
     SwooleG.running = 1;
+    php_swoole_register_shutdown_function("swoole_internal_call_user_shutdown_begin");
+    if (SWOOLE_G(enable_library))
+    {
+        php_swoole_load_library();
+    }
     SWOOLE_G(req_status) = PHP_SWOOLE_RINIT_END;
     return SUCCESS;
 }
@@ -1011,47 +743,17 @@ PHP_RINIT_FUNCTION(swoole)
 PHP_RSHUTDOWN_FUNCTION(swoole)
 {
     SWOOLE_G(req_status) = PHP_SWOOLE_RSHUTDOWN_BEGIN;
-    swoole_call_rshutdown_function(NULL);
-    //clear pipe buffer
-    if (SwooleG.serv && swIsWorker())
-    {
-        swWorker_clean();
-    }
 
-    if (SwooleG.serv && SwooleG.serv->gs->start > 0 && SwooleG.running > 0)
-    {
-        if (PG(last_error_message))
-        {
-            switch(PG(last_error_type))
-            {
-            case E_ERROR:
-            case E_CORE_ERROR:
-            case E_USER_ERROR:
-            case E_COMPILE_ERROR:
-                swoole_error_log(
-                    SW_LOG_ERROR, SW_ERROR_PHP_FATAL_ERROR, "Fatal error: %s in %s on line %d",
-                    PG(last_error_message), PG(last_error_file)?PG(last_error_file):"-", PG(last_error_lineno)
-                );
-                break;
-            default:
-                break;
-            }
-        }
-        else
-        {
-            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SERVER_WORKER_TERMINATED, "worker process is terminated by exit()/die()");
-        }
-    }
+    rshutdown_callbacks.execute();
 
-    if (SwooleAIO.init)
-    {
-        swAio_free();
-    }
+    php_swoole_server_rshutdown();
+    php_swoole_async_coro_rshutdown();
+    php_swoole_redis_server_rshutdown();
+    php_swoole_coroutine_rshutdown();
+    php_swoole_runtime_rshutdown();
+    php_swoole_process_clean();
 
-    swoole_async_coro_shutdown();
-    swoole_redis_server_shutdown();
-
-    SwooleWG.reactor_wait_onexit = 0;
+    SwooleG.running = 0;
     SWOOLE_G(req_status) = PHP_SWOOLE_RSHUTDOWN_END;
 
     return SUCCESS;
@@ -1109,51 +811,47 @@ PHP_FUNCTION(swoole_last_error)
 
 PHP_FUNCTION(swoole_cpu_num)
 {
-    static long cpu_num = 0;
-    if (cpu_num == 0)
-    {
-        cpu_num = MAX(1, sysconf(_SC_NPROCESSORS_CONF));
-    }
-    RETURN_LONG(cpu_num);
+    RETURN_LONG(SW_CPU_NUM);
 }
 
 PHP_FUNCTION(swoole_strerror)
 {
-    zend_long swoole_errno = 0;
+    zend_long swoole_errno;
     zend_long error_type = SW_STRERROR_SYSTEM;
-    char error_msg[256] = {0};
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|l", &swoole_errno, &error_type) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START(1, 2)
+        Z_PARAM_LONG(swoole_errno)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_LONG(error_type)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     if (error_type == SW_STRERROR_GAI)
     {
-        snprintf(error_msg, sizeof(error_msg) - 1, "%s", gai_strerror(swoole_errno));
+        RETURN_STRING(gai_strerror(swoole_errno));
     }
     else if (error_type == SW_STRERROR_DNS)
     {
-        snprintf(error_msg, sizeof(error_msg) - 1, "%s", hstrerror(swoole_errno));
+        RETURN_STRING(hstrerror(swoole_errno));
     }
     else if (error_type == SW_STRERROR_SWOOLE || (swoole_errno > SW_ERROR_START && swoole_errno < SW_ERROR_END))
     {
-        snprintf(error_msg, sizeof(error_msg) - 1, "%s", swoole_strerror(swoole_errno));
+        RETURN_STRING(swoole_strerror(swoole_errno));
     }
     else
     {
-        snprintf(error_msg, sizeof(error_msg) - 1, "%s", strerror(swoole_errno));
+        RETURN_STRING(strerror(swoole_errno));
     }
-    RETURN_STRING(error_msg);
 }
 
 PHP_FUNCTION(swoole_get_mime_type)
 {
     char *filename;
-    zend_long filename_len;
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &filename, &filename_len) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    size_t filename_len;
+
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_STRING(filename, filename_len)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     RETURN_STRING(swoole_mime_type_get(filename));
 }
 
@@ -1164,20 +862,14 @@ PHP_FUNCTION(swoole_errno)
 
 PHP_FUNCTION(swoole_set_process_name)
 {
-#ifdef __MACH__
-    // MacOS doesn't support 'cli_set_process_title'
-    swoole_php_fatal_error(E_WARNING, "swoole_set_process_name is not supported on MacOS");
-    RETURN_FALSE;
-#else
     zend_function *cli_set_process_title = (zend_function *) zend_hash_str_find_ptr(EG(function_table),
             ZEND_STRL("cli_set_process_title"));
     if (!cli_set_process_title)
     {
-        swoole_php_fatal_error(E_WARNING, "swoole_set_process_name only support in CLI mode");
+        php_swoole_fatal_error(E_WARNING, "swoole_set_process_name only support in CLI mode");
         RETURN_FALSE;
     }
     cli_set_process_title->internal_function.handler(INTERNAL_FUNCTION_PARAM_PASSTHRU);
-#endif
 }
 
 PHP_FUNCTION(swoole_get_local_ip)
@@ -1189,7 +881,7 @@ PHP_FUNCTION(swoole_get_local_ip)
 
     if (getifaddrs(&ipaddrs) != 0)
     {
-        swoole_php_sys_error(E_WARNING, "getifaddrs() failed");
+        php_swoole_sys_error(E_WARNING, "getifaddrs() failed");
         RETURN_FALSE;
     }
     array_init(return_value);
@@ -1242,7 +934,7 @@ PHP_FUNCTION(swoole_get_local_mac)
 
     if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
     {
-        swoole_php_sys_error(E_WARNING, "new socket failed");
+        php_swoole_sys_error(E_WARNING, "new socket failed");
         RETURN_FALSE;
     }
     array_init(return_value);
@@ -1279,7 +971,6 @@ PHP_FUNCTION(swoole_internal_call_user_shutdown_begin)
 {
     if (SWOOLE_G(req_status) == PHP_SWOOLE_RINIT_END)
     {
-
         SWOOLE_G(req_status) = PHP_SWOOLE_CALL_USER_SHUTDOWNFUNC_BEGIN;
         RETURN_TRUE;
     }
