@@ -76,12 +76,13 @@ static const zend_function_entry swoole_buffer_methods[] =
 zend_class_entry *swoole_buffer_ce;
 static zend_object_handlers swoole_buffer_handlers;
 
-void swoole_buffer_init(int module_number)
+void php_swoole_buffer_minit(int module_number)
 {
     SW_INIT_CLASS_ENTRY(swoole_buffer, "Swoole\\Buffer", "swoole_buffer", NULL, swoole_buffer_methods);
     SW_SET_CLASS_SERIALIZABLE(swoole_buffer, zend_class_serialize_deny, zend_class_unserialize_deny);
-    SW_SET_CLASS_CLONEABLE(swoole_buffer, zend_class_clone_deny);
-    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_buffer, zend_class_unset_property_deny);
+    SW_SET_CLASS_CLONEABLE(swoole_buffer, sw_zend_class_clone_deny);
+    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_buffer, sw_zend_class_unset_property_deny);
+    // SW_SET_CLASS_CREATE_WITH_ITS_OWN_HANDLERS(swoole_buffer);
 
     zend_declare_property_long(swoole_buffer_ce, ZEND_STRL("capacity"), SW_STRING_BUFFER_DEFAULT, ZEND_ACC_PUBLIC);
     zend_declare_property_long(swoole_buffer_ce, ZEND_STRL("length"), 0, ZEND_ACC_PUBLIC);
@@ -98,6 +99,11 @@ static void swoole_buffer_recycle(swString *buffer)
 
 static PHP_METHOD(swoole_buffer, __construct)
 {
+    php_swoole_fatal_error(
+        E_DEPRECATED, "Class %s is deprecated, it will be removed in v4.5.0",
+        ZSTR_VAL(swoole_buffer_ce->name)
+    );
+
     zend_long size = SW_STRING_BUFFER_DEFAULT;
 
     ZEND_PARSE_PARAMETERS_START_EX(ZEND_PARSE_PARAMS_THROW, 0, 1)
@@ -123,21 +129,21 @@ static PHP_METHOD(swoole_buffer, __construct)
         RETURN_FALSE;
     }
 
-    swoole_set_object(getThis(), buffer);
-    zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("capacity"), size);
-    zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("length"), 0);
+    swoole_set_object(ZEND_THIS, buffer);
+    zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("capacity"), size);
+    zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("length"), 0);
 }
 
 static PHP_METHOD(swoole_buffer, __destruct)
 {
     SW_PREVENT_USER_DESTRUCT();
 
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
     if (buffer)
     {
         swString_free(buffer);
     }
-    swoole_set_object(getThis(), NULL);
+    swoole_set_object(ZEND_THIS, NULL);
 }
 
 static PHP_METHOD(swoole_buffer, append)
@@ -154,7 +160,7 @@ static PHP_METHOD(swoole_buffer, append)
         php_error_docref(NULL, E_WARNING, "string empty");
         RETURN_FALSE;
     }
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
 
     if ((str.length + buffer->length) > buffer->size && (str.length + buffer->length) > SW_STRING_BUFFER_MAXLEN)
     {
@@ -167,9 +173,9 @@ static PHP_METHOD(swoole_buffer, append)
     {
         if (buffer->size > size_old)
         {
-            zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("capacity"), buffer->size);
+            zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("capacity"), buffer->size);
         }
-        zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("length"),
+        zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("length"),
                 buffer->length - buffer->offset);
         RETURN_LONG(buffer->length - buffer->offset);
     }
@@ -189,7 +195,7 @@ static PHP_METHOD(swoole_buffer, substr)
     {
         RETURN_FALSE;
     }
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
 
     if (remove && !(offset == 0 && length <= buffer->length))
     {
@@ -206,13 +212,13 @@ static PHP_METHOD(swoole_buffer, substr)
     }
     if (length + offset > buffer->length)
     {
-        swoole_php_error(E_WARNING, "offset(" ZEND_LONG_FMT ", " ZEND_LONG_FMT ") is out of bounds", offset, length);
+        php_swoole_error(E_WARNING, "offset(" ZEND_LONG_FMT ", " ZEND_LONG_FMT ") is out of bounds", offset, length);
         RETURN_FALSE;
     }
     if (remove)
     {
         buffer->offset += length;
-        zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("length"), buffer->length - buffer->offset);
+        zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("length"), buffer->length - buffer->offset);
 
         if (buffer->offset > SW_STRING_BUFFER_GARBAGE_MIN && buffer->offset * SW_STRING_BUFFER_GARBAGE_RATIO > buffer->size)
         {
@@ -224,7 +230,7 @@ static PHP_METHOD(swoole_buffer, substr)
 
 static PHP_METHOD(swoole_buffer, __toString)
 {
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
     RETURN_STRINGL(buffer->str + buffer->offset, buffer->length - buffer->offset);
 }
 
@@ -246,7 +252,7 @@ static PHP_METHOD(swoole_buffer, write)
         RETURN_FALSE;
     }
 
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
 
     if (offset < 0)
     {
@@ -271,9 +277,9 @@ static PHP_METHOD(swoole_buffer, write)
     {
         if (buffer->size > size_old)
         {
-            zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("capacity"), buffer->size);
+            zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("capacity"), buffer->size);
         }
-        zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("length"),
+        zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("length"),
                 buffer->length - buffer->offset);
         RETURN_LONG(buffer->length - buffer->offset);
     }
@@ -293,7 +299,7 @@ static PHP_METHOD(swoole_buffer, read)
         RETURN_FALSE;
     }
 
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
 
     if (offset < 0)
     {
@@ -324,7 +330,7 @@ static PHP_METHOD(swoole_buffer, expand)
         RETURN_FALSE;
     }
 
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
 
     if (size <= buffer->size)
     {
@@ -334,7 +340,7 @@ static PHP_METHOD(swoole_buffer, expand)
 
     if (swString_extend(buffer, size) == SW_OK)
     {
-        zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("capacity"), size);
+        zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("capacity"), size);
         RETURN_TRUE;
     }
     else
@@ -345,17 +351,17 @@ static PHP_METHOD(swoole_buffer, expand)
 
 static PHP_METHOD(swoole_buffer, recycle)
 {
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
 
     swoole_buffer_recycle(buffer);
 
-    zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("length"), buffer->length);
+    zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("length"), buffer->length);
 }
 
 static PHP_METHOD(swoole_buffer, clear)
 {
-    swString *buffer = swoole_get_object(getThis());
+    swString *buffer = swoole_get_object(ZEND_THIS);
     buffer->length = 0;
     buffer->offset = 0;
-    zend_update_property_long(swoole_buffer_ce, getThis(), ZEND_STRL("length"), 0);
+    zend_update_property_long(swoole_buffer_ce, ZEND_THIS, ZEND_STRL("length"), 0);
 }
