@@ -18,27 +18,8 @@
 #include "php_streams.h"
 #include "php_network.h"
 
-#include "swoole_coroutine.h"
-
 using namespace swoole;
 
-namespace php
-{
-struct process
-{
-    zend_fcall_info fci;
-    zend_fcall_info_cache fci_cache;
-    zend_object *zsocket;
-    int pipe_type;
-};
-
-enum process_pipe_type
-{
-    PIPE_TYPE_STREAM = 1,
-    PIPE_TYPE_DGRAM = 2,
-};
-}
-
 static PHP_METHOD(swoole_process, __construct);
 static PHP_METHOD(swoole_process, __destruct);
 static PHP_METHOD(swoole_process, useQueue);
@@ -54,6 +35,7 @@ static PHP_METHOD(swoole_process, daemon);
 #ifdef HAVE_CPU_AFFINITY
 static PHP_METHOD(swoole_process, setaffinity);
 #endif
+static PHP_METHOD(swoole_process, set);
 static PHP_METHOD(swoole_process, setTimeout);
 static PHP_METHOD(swoole_process, setBlocking);
 static PHP_METHOD(swoole_process, start);
@@ -65,10 +47,10 @@ static PHP_METHOD(swoole_process, exec);
 static PHP_METHOD(swoole_process, exportSocket);
 
 static void php_swoole_onSignal(int signo);
-static void free_signal_callback(void* data);
 
 static uint32_t php_swoole_worker_round_id = 0;
-static zval *signal_callback[SW_SIGNO_MAX];
+static zend_fcall_info_cache *signal_fci_caches[SW_SIGNO_MAX] = {0};
+
 zend_class_entry *swoole_process_ce;
 static zend_object_handlers swoole_process_handlers;
 
@@ -104,6 +86,7 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_process_daemon, 0, 0, 0)
     ZEND_ARG_INFO(0, nochdir)
     ZEND_ARG_INFO(0, noclose)
+    ZEND_ARG_INFO(0, pipes)
 ZEND_END_ARG_INFO()
 
 #ifdef HAVE_CPU_AFFINITY
@@ -112,6 +95,10 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_process_setaffinity, 0, 0, 1)
 ZEND_END_ARG_INFO()
 #endif
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_process_set, 0, 0, 1)
+    ZEND_ARG_ARRAY_INFO(0, settings, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_process_setTimeout, 0, 0, 1)
     ZEND_ARG_INFO(0, seconds)
 ZEND_END_ARG_INFO()
@@ -169,6 +156,7 @@ static const zend_function_entry swoole_process_methods[] =
 #ifdef HAVE_CPU_AFFINITY
     PHP_ME(swoole_process, setaffinity, arginfo_swoole_process_setaffinity, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
 #endif
+    PHP_ME(swoole_process, set, arginfo_swoole_process_set, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_process, setTimeout, arginfo_swoole_process_setTimeout, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_process, setBlocking, arginfo_swoole_process_setBlocking, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_process, useQueue, arginfo_swoole_process_useQueue, ZEND_ACC_PUBLIC)
@@ -187,19 +175,19 @@ static const zend_function_entry swoole_process_methods[] =
     PHP_FE_END
 };
 
-void swoole_process_init(int module_number)
+void php_swoole_process_minit(int module_number)
 {
     SW_INIT_CLASS_ENTRY(swoole_process, "Swoole\\Process", "swoole_process", NULL, swoole_process_methods);
     SW_SET_CLASS_SERIALIZABLE(swoole_process, zend_class_serialize_deny, zend_class_unserialize_deny);
-    SW_SET_CLASS_CLONEABLE(swoole_process, zend_class_clone_deny);
-    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_process, zend_class_unset_property_deny);
+    SW_SET_CLASS_CLONEABLE(swoole_process, sw_zend_class_clone_deny);
+    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_process, sw_zend_class_unset_property_deny);
+    SW_SET_CLASS_CREATE_WITH_ITS_OWN_HANDLERS(swoole_process);
 
     zend_declare_class_constant_long(swoole_process_ce, ZEND_STRL("IPC_NOWAIT"), MSGQUEUE_NOWAIT);
     zend_declare_class_constant_long(swoole_process_ce, ZEND_STRL("PIPE_MASTER"), SW_PIPE_CLOSE_MASTER);
     zend_declare_class_constant_long(swoole_process_ce, ZEND_STRL("PIPE_WORKER"), SW_PIPE_CLOSE_WORKER);
     zend_declare_class_constant_long(swoole_process_ce, ZEND_STRL("PIPE_READ"), SW_PIPE_CLOSE_READ);
     zend_declare_class_constant_long(swoole_process_ce, ZEND_STRL("PIPE_WRITE"), SW_PIPE_CLOSE_WRITE);
-    bzero(signal_callback, sizeof(signal_callback));
 
     zend_declare_property_null(swoole_process_ce, ZEND_STRL("pipe"), ZEND_ACC_PUBLIC);
     zend_declare_property_null(swoole_process_ce, ZEND_STRL("callback"), ZEND_ACC_PUBLIC);
@@ -263,37 +251,36 @@ static PHP_METHOD(swoole_process, __construct)
     //only cli env
     if (!SWOOLE_G(cli))
     {
-        swoole_php_fatal_error(E_ERROR, "swoole_process only can be used in PHP CLI mode");
+        php_swoole_fatal_error(E_ERROR, "%s can only be used in PHP CLI mode", SW_Z_OBJCE_NAME_VAL_P(ZEND_THIS));
         RETURN_FALSE;
     }
 
     if (SwooleG.serv && SwooleG.serv->gs->start == 1 && swIsMaster())
     {
-        swoole_php_fatal_error(E_ERROR, "swoole_process can't be used in master process");
+        php_swoole_fatal_error(E_ERROR, "%s can't be used in master process", SW_Z_OBJCE_NAME_VAL_P(ZEND_THIS));
         RETURN_FALSE;
     }
 
-    if (SwooleAIO.init)
+    if (SwooleTG.aio_init)
     {
-        swoole_php_fatal_error(E_ERROR, "unable to create process with async-io threads");
+        php_swoole_fatal_error(E_ERROR, "unable to create %s with async-io threads", SW_Z_OBJCE_NAME_VAL_P(ZEND_THIS));
         RETURN_FALSE;
     }
 
-    php::process *proc = (php::process *) emalloc(sizeof(php::process));
-    bzero(proc, sizeof(php::process));
+
+    php_swoole_fci *func = (php_swoole_fci*) emalloc(sizeof(php_swoole_fci));
 
     ZEND_PARSE_PARAMETERS_START_EX(ZEND_PARSE_PARAMS_THROW, 1, 4)
-        Z_PARAM_FUNC(proc->fci, proc->fci_cache);
+        Z_PARAM_FUNC(func->fci, func->fci_cache);
         Z_PARAM_OPTIONAL
         Z_PARAM_BOOL(redirect_stdin_and_stdout)
         Z_PARAM_LONG(pipe_type)
         Z_PARAM_BOOL(enable_coroutine)
-    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+    ZEND_PARSE_PARAMETERS_END_EX(efree(func); RETURN_FALSE);
 
-    swWorker *process = (swWorker *) emalloc(sizeof(swWorker));
-    bzero(process, sizeof(swWorker));
+    swWorker *process = (swWorker *) ecalloc(1, sizeof(swWorker));
 
-    int base = 1;
+    uint32_t base = 1;
     if (SwooleG.serv && SwooleG.serv->gs->start)
     {
         base = SwooleG.serv->worker_num + SwooleG.serv->task_worker_num + SwooleG.serv->user_worker_num;
@@ -318,9 +305,11 @@ static PHP_METHOD(swoole_process, __construct)
     if (pipe_type > 0)
     {
         swPipe *_pipe = (swPipe *) emalloc(sizeof(swPipe));
-        int socket_type = pipe_type == php::PIPE_TYPE_STREAM ? SOCK_STREAM : SOCK_DGRAM;
+        int socket_type = pipe_type == zend::PIPE_TYPE_STREAM ? SOCK_STREAM : SOCK_DGRAM;
         if (swPipeUnsock_create(_pipe, 1, socket_type) < 0)
         {
+            efree(func);
+            efree(process);
             RETURN_FALSE;
         }
 
@@ -329,24 +318,25 @@ static PHP_METHOD(swoole_process, __construct)
         process->pipe_worker = _pipe->getFd(_pipe, SW_PIPE_WORKER);
         process->pipe = process->pipe_master;
 
-        zend_update_property_long(swoole_process_ce, getThis(), ZEND_STRL("pipe"), process->pipe_master);
+        zend_update_property_long(swoole_process_ce, ZEND_THIS, ZEND_STRL("pipe"), process->pipe_master);
     }
 
-    proc->pipe_type = pipe_type;
-
-    process->enable_coroutine = enable_coroutine ? 1 : 0;
+    zend::process *proc = new zend::process;
+    sw_zend_fci_cache_persist(&func->fci_cache);
+    proc->func = func;
+    proc->pipe_type = (enum zend::process_pipe_type) pipe_type;
+    proc->enable_coroutine = enable_coroutine;
     process->ptr2 = proc;
 
-    sw_fci_cache_persist(&proc->fci_cache);
-    swoole_set_object(getThis(), process);
+    swoole_set_object(ZEND_THIS, process);
 }
 
 static PHP_METHOD(swoole_process, __destruct)
 {
     SW_PREVENT_USER_DESTRUCT();
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
-    swoole_set_object(getThis(), NULL);
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
+    swoole_set_object(ZEND_THIS, NULL);
     swPipe *_pipe = process->pipe_object;
     if (_pipe)
     {
@@ -357,17 +347,12 @@ static PHP_METHOD(swoole_process, __destruct)
     {
         efree(process->queue);
     }
-    php::process *proc = (php::process *) process->ptr2;
+    zend::process *proc = (zend::process *) process->ptr2;
     if (proc)
     {
-        sw_fci_cache_discard(&proc->fci_cache);
-        if (proc->zsocket)
-        {
-            OBJ_RELEASE(proc->zsocket);
-        }
-        efree(proc);
-        efree(process);
+        delete proc;
     }
+    efree(process);
 }
 
 static PHP_METHOD(swoole_process, wait)
@@ -386,7 +371,7 @@ static PHP_METHOD(swoole_process, wait)
         options |= WNOHANG;
     }
 
-    pid_t pid = swWaitpid(-1, &status, options);
+    pid_t pid = swoole_waitpid(-1, &status, options);
     if (pid > 0)
     {
         array_init(return_value);
@@ -411,7 +396,7 @@ static PHP_METHOD(swoole_process, useQueue)
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
 
     if (msgkey <= 0)
     {
@@ -434,17 +419,17 @@ static PHP_METHOD(swoole_process, useQueue)
     }
     process->queue = queue;
     process->ipc_mode = mode;
-    zend_update_property_long(swoole_process_ce, getThis(), ZEND_STRL("msgQueueId"), queue->msg_id);
-    zend_update_property_long(swoole_process_ce, getThis(), ZEND_STRL("msgQueueKey"), msgkey);
+    zend_update_property_long(swoole_process_ce, ZEND_THIS, ZEND_STRL("msgQueueId"), queue->msg_id);
+    zend_update_property_long(swoole_process_ce, ZEND_THIS, ZEND_STRL("msgQueueKey"), msgkey);
     RETURN_TRUE;
 }
 
 static PHP_METHOD(swoole_process, statQueue)
 {
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (!process->queue)
     {
-        swoole_php_fatal_error(E_WARNING, "no queue, can't get stats of the queue");
+        php_swoole_fatal_error(E_WARNING, "no queue, can't get stats of the queue");
         RETURN_FALSE;
     }
 
@@ -464,7 +449,7 @@ static PHP_METHOD(swoole_process, statQueue)
 
 static PHP_METHOD(swoole_process, freeQueue)
 {
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (process->queue && swMsgQueue_free(process->queue) == SW_OK)
     {
         efree(process->queue);
@@ -487,12 +472,12 @@ static PHP_METHOD(swoole_process, kill)
         RETURN_FALSE;
     }
 
-    int ret = swKill((int) pid, (int) sig);
+    int ret = swoole_kill((int) pid, (int) sig);
     if (ret < 0)
     {
         if (!(sig == 0 && errno == ESRCH))
         {
-            swoole_php_sys_error(E_WARNING, "swKill(%d, %d) failed", (int) pid, (int) sig);
+            php_swoole_sys_error(E_WARNING, "swKill(%d, %d) failed", (int) pid, (int) sig);
         }
         RETURN_FALSE;
     }
@@ -501,92 +486,90 @@ static PHP_METHOD(swoole_process, kill)
 
 static PHP_METHOD(swoole_process, signal)
 {
-    zval *callback = NULL;
-    long signo = 0;
+    zend_long signo = 0;
+    zval *zcallback = NULL;
+    zend_fcall_info_cache *fci_cache = NULL;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "lz", &signo, &callback) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START(1, 2)
+        Z_PARAM_LONG(signo)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_ZVAL_EX(zcallback, 1, 0)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
     if (!SWOOLE_G(cli))
     {
-        swoole_php_fatal_error(E_ERROR, "cannot use swoole_process::signal here");
+        php_swoole_fatal_error(E_ERROR, "%s::signal can only be used in CLI mode", SW_Z_OBJCE_NAME_VAL_P(ZEND_THIS));
         RETURN_FALSE;
     }
 
     if (signo < 0 || signo >= SW_SIGNO_MAX)
     {
-        swoole_php_fatal_error(E_WARNING, "invalid signal number [%ld]", signo);
+        php_swoole_fatal_error(E_WARNING, "invalid signal number [" ZEND_LONG_FMT "]", signo);
         RETURN_FALSE;
     }
 
     php_swoole_check_reactor();
+
     swSignalHandler handler = swSignal_get_handler(signo);
 
     if (handler && handler != php_swoole_onSignal)
     {
-        swoole_php_fatal_error(E_WARNING, "This signal [%ld] processor has been registered by the system", signo);
-        RETURN_FALSE
+        php_swoole_fatal_error(E_WARNING, "signal [" ZEND_LONG_FMT "] processor has been registered by the system", signo);
+        RETURN_FALSE;
     }
 
-    if (callback == NULL || ZVAL_IS_NULL(callback))
+    if (zcallback == NULL)
     {
-        callback = signal_callback[signo];
-        if (callback)
+        fci_cache = signal_fci_caches[signo];
+        if (fci_cache)
         {
             swSignal_add(signo, NULL);
-            SwooleG.main_reactor->defer(SwooleG.main_reactor, free_signal_callback, callback);
-            signal_callback[signo] = NULL;
-            SwooleG.main_reactor->signal_listener_num--;
+            signal_fci_caches[signo] = NULL;
+            swoole_event_defer(sw_zend_fci_cache_free, fci_cache);
+            SwooleTG.reactor->signal_listener_num--;
             RETURN_TRUE;
         }
         else
         {
-            swoole_php_error(E_WARNING, "no callback");
+            php_swoole_error(E_WARNING, "unable to find the callback of signal [" ZEND_LONG_FMT "]", signo);
             RETURN_FALSE;
         }
     }
-    else if (Z_TYPE_P(callback) == IS_LONG && Z_LVAL_P(callback) == (long) SIG_IGN)
+    else if (Z_TYPE_P(zcallback) == IS_LONG && Z_LVAL_P(zcallback) == (zend_long) SIG_IGN)
     {
         handler = NULL;
     }
     else
     {
         char *func_name;
-        if (!sw_zend_is_callable(callback, 0, &func_name))
+        fci_cache = (zend_fcall_info_cache *) ecalloc(1, sizeof(zend_fcall_info_cache));
+        if (!sw_zend_is_callable_ex(zcallback, NULL, 0, &func_name, 0, fci_cache, NULL))
         {
-            swoole_php_error(E_WARNING, "function '%s' is not callable", func_name);
+            php_swoole_error(E_WARNING, "function '%s' is not callable", func_name);
             efree(func_name);
+            efree(fci_cache);
             RETURN_FALSE;
         }
         efree(func_name);
-
-        callback = sw_zval_dup(callback);
-        Z_TRY_ADDREF_P(callback);
-
+        sw_zend_fci_cache_persist(fci_cache);
         handler = php_swoole_onSignal;
     }
 
-    /**
-     * for swSignalfd_setup
-     */
-    SwooleG.main_reactor->check_signalfd = 1;
+    // for swSignalfd_setup
+    SwooleTG.reactor->check_signalfd = 1;
 
-    //free the old callback
-    if (signal_callback[signo])
+    if (signal_fci_caches[signo])
     {
-        SwooleG.main_reactor->defer(SwooleG.main_reactor, free_signal_callback, signal_callback[signo]);
+        // free the old fci_cache
+        swoole_event_defer(sw_zend_fci_cache_free, signal_fci_caches[signo]);
     }
     else
     {
-        SwooleG.main_reactor->signal_listener_num++;
+        SwooleTG.reactor->signal_listener_num++;
     }
-    signal_callback[signo] = callback;
+    signal_fci_caches[signo] = fci_cache;
 
-    /**
-     * use user settings
-     */
+    // use user settings
     SwooleG.use_signalfd = SwooleG.enable_signalfd;
 
     swSignal_add(signo, handler);
@@ -606,18 +589,17 @@ static PHP_METHOD(swoole_process, alarm)
 
     if (!SWOOLE_G(cli))
     {
-        swoole_php_fatal_error(E_ERROR, "cannot use swoole_process::alarm here");
+        php_swoole_fatal_error(E_ERROR, "cannot use %s::alarm here", SW_Z_OBJCE_NAME_VAL_P(ZEND_THIS));
         RETURN_FALSE;
     }
 
-    if (SwooleG.timer.initialized != 0)
+    if (SwooleTG.timer)
     {
-        swoole_php_fatal_error(E_WARNING, "cannot use both 'timer' and 'alarm' at the same time");
+        php_swoole_fatal_error(E_WARNING, "cannot use both 'timer' and 'alarm' at the same time");
         RETURN_FALSE;
     }
 
-    struct itimerval timer_set;
-    bzero(&timer_set, sizeof(timer_set));
+    struct itimerval timer_set = {{0}};
 
     if (usec > 0)
     {
@@ -639,41 +621,30 @@ static PHP_METHOD(swoole_process, alarm)
 
     if (setitimer(type, &timer_set, NULL) < 0)
     {
-        swoole_php_sys_error(E_WARNING, "setitimer() failed");
+        php_swoole_sys_error(E_WARNING, "setitimer() failed");
         RETURN_FALSE;
     }
 
     RETURN_TRUE;
 }
 
-static void free_signal_callback(void* data)
-{
-    zval *callback = (zval*) data;
-    sw_zval_free(callback);
-}
-
 /**
  * safe signal
  */
 static void php_swoole_onSignal(int signo)
 {
-    zval *retval = NULL;
-    zval args[1];
-    zval *callback = signal_callback[signo];
+    zend_fcall_info_cache *fci_cache = signal_fci_caches[signo];
 
-    ZVAL_LONG(& args[0], signo);
+    if (fci_cache)
+    {
+        zval zsigno;
 
-    if (sw_call_user_function_ex(EG(function_table), NULL, callback, &retval, 1, args, 0, NULL) == FAILURE)
-    {
-        swoole_php_fatal_error(E_WARNING, "user_signal handler error");
-    }
-    if (UNEXPECTED(EG(exception)))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-    if (retval)
-    {
-        zval_ptr_dtor(retval);
+        ZVAL_LONG(&zsigno, signo);
+
+        if (UNEXPECTED(sw_zend_call_function_ex2(NULL, fci_cache, 1, &zsigno, NULL) != SUCCESS))
+        {
+            php_swoole_fatal_error(E_WARNING, "%s: signal [%d] handler error", ZSTR_VAL(swoole_process_ce->name), signo);
+        }
     }
 }
 
@@ -681,21 +652,22 @@ zend_bool php_swoole_signal_isset_handler(int signo)
 {
     if (signo < 0 || signo >= SW_SIGNO_MAX)
     {
-        swoole_php_fatal_error(E_WARNING, "invalid signal number [%d]", signo);
-        return SW_FALSE;
+        php_swoole_fatal_error(E_WARNING, "invalid signal number [%d]", signo);
+        return 0;
     }
-    return signal_callback[signo] != NULL;
+    return signal_fci_caches[signo] != NULL;
 }
 
 void php_swoole_process_clean()
 {
-    int i;
-    for (i = 0; i < SW_SIGNO_MAX; i++)
+    for (int i = 0; i < SW_SIGNO_MAX; i++)
     {
-        if (signal_callback[i])
+        zend_fcall_info_cache *fci_cache = signal_fci_caches[i];
+        if (fci_cache)
         {
-            sw_zval_free(signal_callback[i]);
-            signal_callback[i] = NULL;
+            sw_zend_fci_cache_discard(fci_cache);
+            efree(fci_cache);
+            signal_fci_caches[i] = NULL;
         }
     }
 
@@ -714,7 +686,7 @@ int php_swoole_process_start(swWorker *process, zval *zobject)
     {
         if (dup2(process->pipe, STDIN_FILENO) < 0)
         {
-            swoole_php_sys_error(E_WARNING, "dup2() failed");
+            php_swoole_sys_error(E_WARNING, "dup2() failed");
         }
     }
 
@@ -722,7 +694,7 @@ int php_swoole_process_start(swWorker *process, zval *zobject)
     {
         if (dup2(process->pipe, STDOUT_FILENO) < 0)
         {
-            swoole_php_sys_error(E_WARNING, "dup2() failed");
+            php_swoole_sys_error(E_WARNING, "dup2() failed");
         }
     }
 
@@ -730,7 +702,7 @@ int php_swoole_process_start(swWorker *process, zval *zobject)
     {
         if (dup2(process->pipe, STDERR_FILENO) < 0)
         {
-            swoole_php_sys_error(E_WARNING, "dup2() failed");
+            php_swoole_sys_error(E_WARNING, "dup2() failed");
         }
     }
 
@@ -741,77 +713,56 @@ int php_swoole_process_start(swWorker *process, zval *zobject)
     zend_update_property_long(swoole_process_ce, zobject, ZEND_STRL("pid"), process->pid);
     zend_update_property_long(swoole_process_ce, zobject, ZEND_STRL("pipe"), process->pipe_worker);
 
-    zval args[1];
-    zval *retval = NULL;
-    args[0] = *zobject;
-    Z_TRY_ADDREF_P(zobject);
+    zend::process *proc = (zend::process *) process->ptr2;
 
-    php::process *proc = (php::process *) process->ptr2;
-
-    if (process->enable_coroutine)
+    //eventloop create
+    if (proc->enable_coroutine && php_swoole_reactor_init() < 0)
     {
-        if (PHPCoroutine::create(&proc->fci_cache, 1, args) < 0)
-        {
-            swoole_php_error(E_WARNING, "create process coroutine error");
-            return SW_ERR;
-        }
+        return SW_ERR;
     }
-    else
+    //main function
+    if (UNEXPECTED(!zend::function::call(&proc->func->fci_cache, 1, zobject, NULL, proc->enable_coroutine)))
     {
-        zval _retval, *retval = &_retval;
-        if (sw_call_user_function_fast_ex(NULL, &proc->fci_cache, retval, 1, args) == FAILURE)
-        {
-            swoole_php_error(E_WARNING, "callback function error");
-        }
-        zval_ptr_dtor(retval);
+        php_swoole_error(E_WARNING, "%s->onStart handler error", SW_Z_OBJCE_NAME_VAL_P(zobject));
     }
-
-    if (UNEXPECTED(EG(exception)))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-    if (retval)
-    {
-        zval_ptr_dtor(retval);
-    }
-
-    if (SwooleG.main_reactor)
+    //eventloop start
+    if (proc->enable_coroutine)
     {
         php_swoole_event_wait();
     }
-    SwooleG.running = 0;
-
+    // equivalent to exit
     sw_zend_bailout();
+
     return SW_OK;
 }
 
 static PHP_METHOD(swoole_process, start)
 {
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
 
-    if (swKill(process->pid, 0) == 0)
+    if (process->pid && swoole_kill(process->pid, 0) == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "process has already been started");
+        php_swoole_fatal_error(E_WARNING, "process has already been started");
         RETURN_FALSE;
     }
 
-    pid_t pid = swoole_fork();
+    pid_t pid = swoole_fork(0);
     if (pid < 0)
     {
-        swoole_php_sys_error(E_WARNING, "fork() failed");
+        php_swoole_sys_error(E_WARNING, "fork() failed");
         RETURN_FALSE;
     }
     else if (pid > 0)
     {
         process->pid = pid;
         process->child_process = 0;
-        zend_update_property_long(swoole_server_ce, getThis(), ZEND_STRL("pid"), process->pid);
+        zend_update_property_long(swoole_server_ce, ZEND_THIS, ZEND_STRL("pid"), process->pid);
         RETURN_LONG(pid);
     }
     else
     {
         process->child_process = 1;
-        SW_CHECK_RETURN(php_swoole_process_start(process, getThis()));
+        SW_CHECK_RETURN(php_swoole_process_start(process, ZEND_THIS));
     }
     RETURN_TRUE;
 }
@@ -830,11 +781,11 @@ static PHP_METHOD(swoole_process, read)
         buf_size = 65536;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
 
     if (process->pipe == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "no pipe, cannot read from pipe");
+        php_swoole_fatal_error(E_WARNING, "no pipe, cannot read from pipe");
         RETURN_FALSE;
     }
 
@@ -845,13 +796,13 @@ static PHP_METHOD(swoole_process, read)
         efree(buf);
         if (errno != EINTR)
         {
-            swoole_php_sys_error(E_WARNING, "read() failed");
+            php_swoole_sys_error(E_WARNING, "read() failed");
         }
         RETURN_FALSE;
     }
     buf->val[ret] = 0;
     buf->len = ret;
-    ZVAL_STR(return_value, buf);
+    RETURN_STR(buf);
 }
 
 static PHP_METHOD(swoole_process, write)
@@ -866,26 +817,26 @@ static PHP_METHOD(swoole_process, write)
 
     if (data_len < 1)
     {
-        swoole_php_fatal_error(E_WARNING, "the data to send is empty");
+        php_swoole_fatal_error(E_WARNING, "the data to send is empty");
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (process->pipe == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "no pipe, cannot write into pipe");
+        php_swoole_fatal_error(E_WARNING, "no pipe, cannot write into pipe");
         RETURN_FALSE;
     }
 
     int ret;
 
     //async write
-    if (SwooleG.main_reactor)
+    if (SwooleTG.reactor)
     {
-        swConnection *_socket = swReactor_get(SwooleG.main_reactor, process->pipe);
+        swSocket *_socket = swReactor_get(SwooleTG.reactor, process->pipe);
         if (_socket && _socket->nonblock)
         {
-            ret = SwooleG.main_reactor->write(SwooleG.main_reactor, process->pipe, data, (size_t) data_len);
+            ret = swoole_event_write(process->pipe, data, (size_t) data_len);
         }
         else
         {
@@ -894,12 +845,13 @@ static PHP_METHOD(swoole_process, write)
     }
     else
     {
-        _blocking_read: ret = swSocket_write_blocking(process->pipe, data, data_len);
+        _blocking_read:
+        ret = swSocket_write_blocking(process->pipe, data, data_len);
     }
 
     if (ret < 0)
     {
-        swoole_php_sys_error(E_WARNING, "write() failed");
+        php_swoole_sys_error(E_WARNING, "write() failed");
         RETURN_FALSE;
     }
     ZVAL_LONG(return_value, ret);
@@ -910,16 +862,16 @@ static PHP_METHOD(swoole_process, write)
  */
 static PHP_METHOD(swoole_process, exportSocket)
 {
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (process->pipe == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "no pipe, cannot export stream");
+        php_swoole_fatal_error(E_WARNING, "no pipe, cannot export stream");
         RETURN_FALSE;
     }
-    php::process *proc = (php::process *) process->ptr2;
+    zend::process *proc = (zend::process *) process->ptr2;
     if (!proc->zsocket)
     {
-        proc->zsocket = php_swoole_export_socket_ex(process->pipe, proc->pipe_type == php::PIPE_TYPE_STREAM ? SW_SOCK_UNIX_STREAM : SW_SOCK_UNIX_DGRAM);
+        proc->zsocket = php_swoole_dup_socket(process->pipe, proc->pipe_type == zend::PIPE_TYPE_STREAM ? SW_SOCK_UNIX_STREAM : SW_SOCK_UNIX_DGRAM);
         if (!proc->zsocket)
         {
             RETURN_FALSE;
@@ -947,24 +899,24 @@ static PHP_METHOD(swoole_process, push)
 
     if (length <= 0)
     {
-        swoole_php_fatal_error(E_WARNING, "the data to push is empty");
+        php_swoole_fatal_error(E_WARNING, "the data to push is empty");
         RETURN_FALSE;
     }
     else if (length >= sizeof(message.data))
     {
-        swoole_php_fatal_error(E_WARNING, "the data to push is too big");
+        php_swoole_fatal_error(E_WARNING, "the data to push is too big");
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
 
     if (!process->queue)
     {
-        swoole_php_fatal_error(E_WARNING, "no msgqueue, cannot use push()");
+        php_swoole_fatal_error(E_WARNING, "no msgqueue, cannot use push()");
         RETURN_FALSE;
     }
 
-    message.type = process->id;
+    message.type = process->id + 1;
     memcpy(message.data, data, length);
 
     if (swMsgQueue_push(process->queue, (swQueue_data *)&message, length) < 0)
@@ -988,10 +940,10 @@ static PHP_METHOD(swoole_process, pop)
         maxsize = SW_MSGMAX;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (!process->queue)
     {
-        swoole_php_fatal_error(E_WARNING, "no msgqueue, cannot use pop()");
+        php_swoole_fatal_error(E_WARNING, "no msgqueue, cannot use pop()");
         RETURN_FALSE;
     }
 
@@ -1007,7 +959,7 @@ static PHP_METHOD(swoole_process, pop)
     }
     else
     {
-        message.type = process->id;
+        message.type = process->id + 1;
     }
 
     int n = swMsgQueue_pop(process->queue, (swQueue_data *) &message, maxsize);
@@ -1031,7 +983,7 @@ static PHP_METHOD(swoole_process, exec)
 
     if (execfile_len < 1)
     {
-        swoole_php_fatal_error(E_WARNING, "exec file name is empty");
+        php_swoole_fatal_error(E_WARNING, "exec file name is empty");
         RETURN_FALSE;
     }
 
@@ -1052,7 +1004,7 @@ static PHP_METHOD(swoole_process, exec)
 
     if (execv(execfile, exec_args) < 0)
     {
-        swoole_php_sys_error(E_WARNING, "execv(%s) failed", execfile);
+        php_swoole_sys_error(E_WARNING, "execv(%s) failed", execfile);
         RETURN_FALSE;
     }
     else
@@ -1065,11 +1017,41 @@ static PHP_METHOD(swoole_process, daemon)
 {
     zend_bool nochdir = 1;
     zend_bool noclose = 1;
+    zval *zpipes = NULL;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "|bb", &nochdir, &noclose) == FAILURE)
+    ZEND_PARSE_PARAMETERS_START(0, 3)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_BOOL(nochdir)
+        Z_PARAM_BOOL(noclose)
+        Z_PARAM_ARRAY(zpipes)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    zval *elem;
+    int fd = 0;
+
+    if (zpipes)
     {
-        RETURN_FALSE;
+        ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(zpipes), elem)
+        {
+            if (!ZVAL_IS_NULL(elem))
+            {
+                int new_fd = swoole_convert_to_fd(elem);
+                if (new_fd >= 0)
+                {
+                    if (dup2(new_fd, fd) < 0)
+                    {
+                        swSysWarn("dup2(%d, %d) failed", new_fd, fd);
+                    }
+                }
+            }
+            if (fd++ == 2)
+            {
+                break;
+            }
+        }
+        ZEND_HASH_FOREACH_END();
     }
+
     RETURN_BOOL(daemon(nochdir, noclose) == 0);
 }
 
@@ -1087,7 +1069,7 @@ static PHP_METHOD(swoole_process, setaffinity)
     }
     if (php_swoole_array_length(array) > SW_CPU_NUM)
     {
-        swoole_php_fatal_error(E_WARNING, "More than the number of CPU");
+        php_swoole_fatal_error(E_WARNING, "More than the number of CPU");
         RETURN_FALSE;
     }
 
@@ -1098,7 +1080,7 @@ static PHP_METHOD(swoole_process, setaffinity)
     SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(array), value)
         if (zval_get_long(value) >= SW_CPU_NUM)
         {
-            swoole_php_fatal_error(E_WARNING, "invalid cpu id [%d]", (int) Z_LVAL_P(value));
+            php_swoole_fatal_error(E_WARNING, "invalid cpu id [%d]", (int) Z_LVAL_P(value));
             RETURN_FALSE;
         }
         CPU_SET(Z_LVAL_P(value), &cpu_set);
@@ -1111,7 +1093,7 @@ static PHP_METHOD(swoole_process, setaffinity)
     if (sched_setaffinity(getpid(), sizeof(cpu_set), &cpu_set) < 0)
 #endif
     {
-        swoole_php_sys_error(E_WARNING, "sched_setaffinity() failed");
+        php_swoole_sys_error(E_WARNING, "sched_setaffinity() failed");
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -1127,17 +1109,17 @@ static PHP_METHOD(swoole_process, exit)
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
 
     if (getpid() != process->pid)
     {
-        swoole_php_fatal_error(E_WARNING, "not current process");
+        php_swoole_fatal_error(E_WARNING, "not current process");
         RETURN_FALSE;
     }
 
     if (ret_code < 0 || ret_code > 255)
     {
-        swoole_php_fatal_error(E_WARNING, "exit ret_code range is [>0 and <255] ");
+        php_swoole_fatal_error(E_WARNING, "exit ret_code range is [>0 and <255] ");
         ret_code = 1;
     }
 
@@ -1163,10 +1145,16 @@ static PHP_METHOD(swoole_process, close)
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (process->pipe == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "no pipe, cannot close the pipe");
+        php_swoole_fatal_error(E_WARNING, "no pipe, cannot close the pipe");
+        RETURN_FALSE;
+    }
+
+    if (process->pipe_object == nullptr)
+    {
+        php_swoole_fatal_error(E_WARNING, "cannot close the pipe");
         RETURN_FALSE;
     }
 
@@ -1185,7 +1173,7 @@ static PHP_METHOD(swoole_process, close)
     }
     if (ret < 0)
     {
-        swoole_php_sys_error(E_WARNING, "close() failed");
+        php_swoole_sys_error(E_WARNING, "close() failed");
         RETURN_FALSE;
     }
     if (which == 0)
@@ -1197,6 +1185,27 @@ static PHP_METHOD(swoole_process, close)
     RETURN_TRUE;
 }
 
+static PHP_METHOD(swoole_process, set)
+{
+    zval *zset = NULL;
+    HashTable *vht = NULL;
+    zval *ztmp;
+
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_ARRAY(zset)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    vht = Z_ARRVAL_P(zset);
+
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
+    zend::process *proc = (zend::process *) process->ptr2;
+
+    if (php_swoole_array_get_value(vht, "enable_coroutine", ztmp))
+    {
+        proc->enable_coroutine = zval_is_true(ztmp);
+    }
+}
+
 static PHP_METHOD(swoole_process, setTimeout)
 {
     double seconds;
@@ -1205,10 +1214,10 @@ static PHP_METHOD(swoole_process, setTimeout)
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (process->pipe == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "no pipe, cannot setTimeout the pipe");
+        php_swoole_fatal_error(E_WARNING, "no pipe, cannot setTimeout the pipe");
         RETURN_FALSE;
     }
     SW_CHECK_RETURN(swSocket_set_timeout(process->pipe, seconds));
@@ -1222,23 +1231,23 @@ static PHP_METHOD(swoole_process, setBlocking)
         RETURN_FALSE;
     }
 
-    swWorker *process = (swWorker *) swoole_get_object(getThis());
+    swWorker *process = (swWorker *) swoole_get_object(ZEND_THIS);
     if (process->pipe == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "no pipe, cannot setBlocking the pipe");
+        php_swoole_fatal_error(E_WARNING, "no pipe, cannot setBlocking the pipe");
         RETURN_FALSE;
     }
     if (blocking)
     {
-        swSetBlock(process->pipe);
+        swSocket_set_blocking(process->pipe);
     }
     else
     {
-        swSetNonBlock(process->pipe);
+        swSocket_set_nonblock(process->pipe);
     }
-    if (SwooleG.main_reactor)
+    if (SwooleTG.reactor)
     {
-        swConnection *_socket = swReactor_get(SwooleG.main_reactor, process->pipe);
+        swSocket *_socket = swReactor_get(SwooleTG.reactor, process->pipe);
         if (_socket)
         {
             _socket->nonblock = blocking ? 0 : 1;
