@@ -16,10 +16,6 @@
 
 #include "php_swoole_cxx.h"
 
-#ifdef SW_COROUTINE
-#include "swoole_coroutine.h"
-#endif
-
 using namespace std;
 using namespace swoole;
 
@@ -83,12 +79,13 @@ const zend_function_entry swoole_server_port_methods[] =
     PHP_FE_END
 };
 
-void swoole_server_port_init(int module_number)
+void php_swoole_server_port_minit(int module_number)
 {
     SW_INIT_CLASS_ENTRY(swoole_server_port, "Swoole\\Server\\Port", "swoole_server_port", NULL, swoole_server_port_methods);
     SW_SET_CLASS_SERIALIZABLE(swoole_server_port, zend_class_serialize_deny, zend_class_unserialize_deny);
-    SW_SET_CLASS_CLONEABLE(swoole_server_port, zend_class_clone_deny);
-    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_server_port, zend_class_unset_property_deny);
+    SW_SET_CLASS_CLONEABLE(swoole_server_port, sw_zend_class_clone_deny);
+    SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_server_port, sw_zend_class_unset_property_deny);
+    SW_SET_CLASS_CREATE_WITH_ITS_OWN_HANDLERS(swoole_server_port);
 
     zend_declare_property_null(swoole_server_port_ce, ZEND_STRL("onConnect"), ZEND_ACC_PRIVATE);
     zend_declare_property_null(swoole_server_port_ce, ZEND_STRL("onReceive"), ZEND_ACC_PRIVATE);
@@ -110,9 +107,46 @@ void swoole_server_port_init(int module_number)
     zend_declare_property_null(swoole_server_port_ce, ZEND_STRL("connections"), ZEND_ACC_PUBLIC);
 }
 
+/**
+ * [Master-Process]
+ */
+static ssize_t php_swoole_server_length_func(swProtocol *protocol, swSocket *conn, char *data, uint32_t length)
+{
+    swServer *serv = (swServer *) protocol->private_data_2;
+    swServer_lock(serv);
+
+    zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache *) protocol->private_data;
+    zval zdata;
+    zval retval;
+    ssize_t ret = -1;
+
+    // TODO: reduce memory copy
+    ZVAL_STRINGL(&zdata, data, length);
+    if (UNEXPECTED(sw_zend_call_function_ex(NULL, fci_cache, 1, &zdata, &retval) != SUCCESS))
+    {
+        php_swoole_fatal_error(E_WARNING, "length function handler error");
+    }
+    else
+    {
+        ret = zval_get_long(&retval);
+        zval_ptr_dtor(&retval);
+    }
+    zval_ptr_dtor(&zdata);
+
+    swServer_unlock(serv);
+
+    /* the exception should only be thrown after unlocked */
+    if (UNEXPECTED(EG(exception)))
+    {
+        zend_exception_error(EG(exception), E_ERROR);
+    }
+
+    return ret;
+}
+
 static PHP_METHOD(swoole_server_port, __construct)
 {
-    swoole_php_fatal_error(E_ERROR, "please use the Swoole\\Server->listen method");
+    php_swoole_fatal_error(E_ERROR, "please use the Swoole\\Server->listen method");
     return;
 }
 
@@ -120,10 +154,10 @@ static PHP_METHOD(swoole_server_port, __destruct)
 {
     SW_PREVENT_USER_DESTRUCT();
 
-    swoole_server_port_property *property = (swoole_server_port_property *) swoole_get_property(getThis(), 0);
+    swoole_server_port_property *property = (swoole_server_port_property *) swoole_get_property(ZEND_THIS, 0);
+    swListenPort *port = property->port;
 
-    int j;
-    for (j = 0; j < PHP_SWOOLE_SERVER_PORT_CALLBACK_NUM; j++)
+    for (int j = 0; j < PHP_SWOOLE_SERVER_PORT_CALLBACK_NUM; j++)
     {
         if (property->caches[j])
         {
@@ -132,16 +166,23 @@ static PHP_METHOD(swoole_server_port, __destruct)
         }
     }
 
+    if (port->protocol.private_data)
+    {
+        sw_zend_fci_cache_discard((zend_fcall_info_cache *) port->protocol.private_data);
+        efree(port->protocol.private_data);
+        port->protocol.private_data = nullptr;
+    }
+
     efree(property);
-    swoole_set_property(getThis(), 0, NULL);
-    swoole_set_object(getThis(), NULL);
+    swoole_set_property(ZEND_THIS, 0, NULL);
+    swoole_set_object(ZEND_THIS, NULL);
 }
 
 static PHP_METHOD(swoole_server_port, set)
 {
     zval *zset = NULL;
     HashTable *vht;
-    zval *v;
+    zval *ztmp;
 
     ZEND_PARSE_PARAMETERS_START(1, 1)
         Z_PARAM_ARRAY(zset)
@@ -149,23 +190,25 @@ static PHP_METHOD(swoole_server_port, set)
 
     vht = Z_ARRVAL_P(zset);
 
-    swListenPort *port = (swListenPort *) swoole_get_object(getThis());
-    swoole_server_port_property *property = (swoole_server_port_property *) swoole_get_property(getThis(), 0);
+    swListenPort *port = (swListenPort *) swoole_get_object(ZEND_THIS);
+    swoole_server_port_property *property = (swoole_server_port_property *) swoole_get_property(ZEND_THIS, 0);
 
     if (port == NULL || property == NULL)
     {
-        swoole_php_fatal_error(E_ERROR, "please use the swoole_server->listen method");
+        php_swoole_fatal_error(E_ERROR, "please use the swoole_server->listen method");
         return;
     }
 
     //backlog
-    if (php_swoole_array_get_value(vht, "backlog", v))
+    if (php_swoole_array_get_value(vht, "backlog", ztmp))
     {
-        port->backlog = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->backlog = SW_MAX(0, SW_MIN(v, UINT16_MAX));
     }
-    if (php_swoole_array_get_value(vht, "socket_buffer_size", v))
+    if (php_swoole_array_get_value(vht, "socket_buffer_size", ztmp))
     {
-        port->socket_buffer_size = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->socket_buffer_size = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
         if (port->socket_buffer_size <= 0)
         {
             port->socket_buffer_size = INT_MAX;
@@ -174,9 +217,10 @@ static PHP_METHOD(swoole_server_port, set)
     /**
      * !!! Don't set this option, for tests only.
      */
-    if (php_swoole_array_get_value(vht, "kernel_socket_recv_buffer_size", v))
+    if (php_swoole_array_get_value(vht, "kernel_socket_recv_buffer_size", ztmp))
     {
-        port->kernel_socket_recv_buffer_size = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->kernel_socket_recv_buffer_size = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
         if (port->kernel_socket_recv_buffer_size <= 0)
         {
             port->kernel_socket_recv_buffer_size = INT_MAX;
@@ -185,90 +229,94 @@ static PHP_METHOD(swoole_server_port, set)
     /**
      * !!! Don't set this option, for tests only.
      */
-    if (php_swoole_array_get_value(vht, "kernel_socket_send_buffer_size", v))
+    if (php_swoole_array_get_value(vht, "kernel_socket_send_buffer_size", ztmp))
     {
-        port->kernel_socket_send_buffer_size = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->kernel_socket_send_buffer_size = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
         if (port->kernel_socket_send_buffer_size <= 0)
         {
             port->kernel_socket_send_buffer_size = INT_MAX;
         }
     }
-    if (php_swoole_array_get_value(vht, "buffer_high_watermark", v))
+    if (php_swoole_array_get_value(vht, "buffer_high_watermark", ztmp))
     {
-        port->buffer_high_watermark = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->buffer_high_watermark = SW_MAX(0, SW_MIN(v, UINT32_MAX));
     }
-    if (php_swoole_array_get_value(vht, "buffer_low_watermark", v))
+    if (php_swoole_array_get_value(vht, "buffer_low_watermark", ztmp))
     {
-        port->buffer_low_watermark = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->buffer_low_watermark = SW_MAX(0, SW_MIN(v, UINT32_MAX));
     }
     //server: tcp_nodelay
-    if (php_swoole_array_get_value(vht, "open_tcp_nodelay", v))
+    if (php_swoole_array_get_value(vht, "open_tcp_nodelay", ztmp))
     {
-        port->open_tcp_nodelay = zval_is_true(v);
+        port->open_tcp_nodelay = zval_is_true(ztmp);
     }
     else
     {
         port->open_tcp_nodelay = 1;
     }
     //tcp_defer_accept
-    if (php_swoole_array_get_value(vht, "tcp_defer_accept", v))
+    if (php_swoole_array_get_value(vht, "tcp_defer_accept", ztmp))
     {
-        port->tcp_defer_accept = (uint8_t) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->tcp_defer_accept = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
     }
     //tcp_keepalive
-    if (php_swoole_array_get_value(vht, "open_tcp_keepalive", v))
+    if (php_swoole_array_get_value(vht, "open_tcp_keepalive", ztmp))
     {
-        port->open_tcp_keepalive = zval_is_true(v);
+        port->open_tcp_keepalive = zval_is_true(ztmp);
     }
     //buffer: eof check
-    if (php_swoole_array_get_value(vht, "open_eof_check", v))
+    if (php_swoole_array_get_value(vht, "open_eof_check", ztmp))
     {
-        port->open_eof_check = zval_is_true(v);
+        port->open_eof_check = zval_is_true(ztmp);
     }
     //buffer: split package with eof
-    if (php_swoole_array_get_value(vht, "open_eof_split", v))
+    if (php_swoole_array_get_value(vht, "open_eof_split", ztmp))
     {
-        port->protocol.split_by_eof = zval_is_true(v);
+        port->protocol.split_by_eof = zval_is_true(ztmp);
         if (port->protocol.split_by_eof)
         {
             port->open_eof_check = 1;
         }
     }
     //package eof
-    if (php_swoole_array_get_value(vht, "package_eof", v))
+    if (php_swoole_array_get_value(vht, "package_eof", ztmp))
     {
-        zend::string str_v(v);
+        zend::string str_v(ztmp);
         port->protocol.package_eof_len = str_v.len();
         if (port->protocol.package_eof_len == 0)
         {
-            swoole_php_fatal_error(E_ERROR, "pacakge_eof cannot be an empty string");
+            php_swoole_fatal_error(E_ERROR, "pacakge_eof cannot be an empty string");
             RETURN_FALSE;
         }
         else if (port->protocol.package_eof_len > SW_DATA_EOF_MAXLEN)
         {
-            swoole_php_fatal_error(E_ERROR, "pacakge_eof max length is %d", SW_DATA_EOF_MAXLEN);
+            php_swoole_fatal_error(E_ERROR, "pacakge_eof max length is %d", SW_DATA_EOF_MAXLEN);
             RETURN_FALSE;
         }
         bzero(port->protocol.package_eof, SW_DATA_EOF_MAXLEN);
         memcpy(port->protocol.package_eof, str_v.val(), str_v.len());
     }
     //http_protocol
-    if (php_swoole_array_get_value(vht, "open_http_protocol", v))
+    if (php_swoole_array_get_value(vht, "open_http_protocol", ztmp))
     {
-        port->open_http_protocol = zval_is_true(v);
+        port->open_http_protocol = zval_is_true(ztmp);
     }
     //websocket protocol
-    if (php_swoole_array_get_value(vht, "open_websocket_protocol", v))
+    if (php_swoole_array_get_value(vht, "open_websocket_protocol", ztmp))
     {
-        port->open_websocket_protocol = zval_is_true(v);
+        port->open_websocket_protocol = zval_is_true(ztmp);
         if (port->open_websocket_protocol)
         {
             port->open_http_protocol = 1;
         }
     }
-    if (php_swoole_array_get_value(vht, "websocket_subprotocol", v))
+    if (php_swoole_array_get_value(vht, "websocket_subprotocol", ztmp))
     {
-        zend::string str_v(v);
+        zend::string str_v(ztmp);
         if (port->websocket_subprotocol)
         {
             sw_free(port->websocket_subprotocol);
@@ -276,15 +324,15 @@ static PHP_METHOD(swoole_server_port, set)
         port->websocket_subprotocol = str_v.dup();
         port->websocket_subprotocol_length = str_v.len();
     }
-    if (php_swoole_array_get_value(vht, "open_websocket_close_frame", v))
+    if (php_swoole_array_get_value(vht, "open_websocket_close_frame", ztmp))
     {
-        port->open_websocket_close_frame = zval_is_true(v);
+        port->open_websocket_close_frame = zval_is_true(ztmp);
     }
 #ifdef SW_USE_HTTP2
     //http2 protocol
-    if (php_swoole_array_get_value(vht, "open_http2_protocol", v))
+    if (php_swoole_array_get_value(vht, "open_http2_protocol", ztmp))
     {
-        port->open_http2_protocol = zval_is_true(v);
+        port->open_http2_protocol = zval_is_true(ztmp);
         if (port->open_http2_protocol)
         {
             port->open_http_protocol = 1;
@@ -292,100 +340,112 @@ static PHP_METHOD(swoole_server_port, set)
     }
 #endif
     //buffer: mqtt protocol
-    if (php_swoole_array_get_value(vht, "open_mqtt_protocol", v))
+    if (php_swoole_array_get_value(vht, "open_mqtt_protocol", ztmp))
     {
-        port->open_mqtt_protocol = zval_is_true(v);
+        port->open_mqtt_protocol = zval_is_true(ztmp);
     }
     //redis protocol
-    if (php_swoole_array_get_value(vht, "open_redis_protocol", v))
+    if (php_swoole_array_get_value(vht, "open_redis_protocol", ztmp))
     {
-        port->open_redis_protocol = zval_get_long(v);
+        port->open_redis_protocol = zval_get_long(ztmp);
     }
     //tcp_keepidle
-    if (php_swoole_array_get_value(vht, "tcp_keepidle", v))
+    if (php_swoole_array_get_value(vht, "tcp_keepidle", ztmp))
     {
-        port->tcp_keepidle = (uint16_t) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->tcp_keepidle = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
     }
     //tcp_keepinterval
-    if (php_swoole_array_get_value(vht, "tcp_keepinterval", v))
+    if (php_swoole_array_get_value(vht, "tcp_keepinterval", ztmp))
     {
-        port->tcp_keepinterval = (uint16_t) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->tcp_keepinterval = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
     }
     //tcp_keepcount
-    if (php_swoole_array_get_value(vht, "tcp_keepcount", v))
+    if (php_swoole_array_get_value(vht, "tcp_keepcount", ztmp))
     {
-        port->tcp_keepcount = (uint16_t) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->tcp_keepcount = SW_MAX(INT_MIN, SW_MIN(v, INT_MAX));
     }
     //tcp_fastopen
-    if (php_swoole_array_get_value(vht, "tcp_fastopen", v))
+    if (php_swoole_array_get_value(vht, "tcp_fastopen", ztmp))
     {
-        port->tcp_fastopen = zval_is_true(v);
+        port->tcp_fastopen = zval_is_true(ztmp);
     }
     //open length check
-    if (php_swoole_array_get_value(vht, "open_length_check", v))
+    if (php_swoole_array_get_value(vht, "open_length_check", ztmp))
     {
-        port->open_length_check = zval_is_true(v);
+        port->open_length_check = zval_is_true(ztmp);
     }
     //package length size
-    if (php_swoole_array_get_value(vht, "package_length_type", v))
+    if (php_swoole_array_get_value(vht, "package_length_type", ztmp))
     {
-        zend::string str_v(v);
+        zend::string str_v(ztmp);
         port->protocol.package_length_type = str_v.val()[0];
         port->protocol.package_length_size = swoole_type_size(port->protocol.package_length_type);
         if (port->protocol.package_length_size == 0)
         {
-            swoole_php_fatal_error(E_ERROR, "unknow package_length_type, see pack(). Link: http://php.net/pack");
+            php_swoole_fatal_error(E_ERROR, "unknow package_length_type, see pack(). Link: http://php.net/pack");
             RETURN_FALSE;
         }
     }
     //package length offset
-    if (php_swoole_array_get_value(vht, "package_length_offset", v))
+    if (php_swoole_array_get_value(vht, "package_length_offset", ztmp))
     {
-        port->protocol.package_length_offset = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->protocol.package_length_offset = SW_MAX(0, SW_MIN(v, UINT16_MAX));
         if (port->protocol.package_length_offset > SW_IPC_BUFFER_SIZE)
         {
-            swoole_php_fatal_error(E_ERROR, "'package_length_offset' value is too large");
+            php_swoole_fatal_error(E_ERROR, "'package_length_offset' value is too large");
         }
     }
     //package body start
-    if (php_swoole_array_get_value(vht, "package_body_offset", v) || php_swoole_array_get_value(vht, "package_body_start", v))
+    if (php_swoole_array_get_value(vht, "package_body_offset", ztmp) || php_swoole_array_get_value(vht, "package_body_start", ztmp))
     {
-        port->protocol.package_body_offset = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->protocol.package_body_offset = SW_MAX(0, SW_MIN(v, UINT16_MAX));
         if (port->protocol.package_body_offset > SW_IPC_BUFFER_SIZE)
         {
-            swoole_php_fatal_error(E_ERROR, "'package_body_offset' value is too large");
+            php_swoole_fatal_error(E_ERROR, "'package_body_offset' value is too large");
         }
     }
     //length function
-    if (php_swoole_array_get_value(vht, "package_length_func", v))
+    if (php_swoole_array_get_value(vht, "package_length_func", ztmp))
     {
         while(1)
         {
-            if (Z_TYPE_P(v) == IS_STRING)
+            if (Z_TYPE_P(ztmp) == IS_STRING)
             {
-                swProtocol_length_function func = (swProtocol_length_function) swoole_get_function(Z_STRVAL_P(v), Z_STRLEN_P(v));
+                swProtocol_length_function func = (swProtocol_length_function) swoole_get_function(Z_STRVAL_P(ztmp), Z_STRLEN_P(ztmp));
                 if (func != NULL)
                 {
                     port->protocol.get_package_length = func;
                     break;
                 }
             }
-
-            char *func_name = NULL;
-            if (!sw_zend_is_callable(v, 0, &func_name))
+#ifdef ZTS
+            swServer *serv = property->serv;
+            if (serv->factory_mode == SW_MODE_PROCESS && !serv->single_thread)
             {
-                swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
+                php_swoole_fatal_error(E_ERROR, "option [package_length_func] does not support with ZTS");
+            }
+#endif
+            char *func_name;
+            zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache *) ecalloc(1, sizeof(zend_fcall_info_cache));
+            if (!sw_zend_is_callable_ex(ztmp, NULL, 0, &func_name, NULL, fci_cache, NULL))
+            {
+                php_swoole_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
                 return;
             }
             efree(func_name);
-            port->protocol.get_package_length = php_swoole_length_func;
+            port->protocol.get_package_length = php_swoole_server_length_func;
             if (port->protocol.private_data)
             {
-                zval_ptr_dtor((zval *)port->protocol.private_data);
+                sw_zend_fci_cache_discard((zend_fcall_info_cache *) port->protocol.private_data);
                 efree(port->protocol.private_data);
             }
-            Z_TRY_ADDREF_P(v);
-            port->protocol.private_data = sw_zval_dup(v);
+            sw_zend_fci_cache_persist(fci_cache);
+            port->protocol.private_data = fci_cache;
             break;
         }
 
@@ -396,20 +456,21 @@ static PHP_METHOD(swoole_server_port, set)
     /**
      * package max length
      */
-    if (php_swoole_array_get_value(vht, "package_max_length", v))
+    if (php_swoole_array_get_value(vht, "package_max_length", ztmp))
     {
-        port->protocol.package_max_length = (int) zval_get_long(v);
+        zend_long v = zval_get_long(ztmp);
+        port->protocol.package_max_length = SW_MAX(0, SW_MIN(v, UINT32_MAX));
     }
 
 #ifdef SW_USE_OPENSSL
     if (port->ssl)
     {
-        if (php_swoole_array_get_value(vht, "ssl_cert_file", v))
+        if (php_swoole_array_get_value(vht, "ssl_cert_file", ztmp))
         {
-            zend::string str_v(v);
+            zend::string str_v(ztmp);
             if (access(str_v.val(), R_OK) < 0)
             {
-                swoole_php_fatal_error(E_ERROR, "ssl cert file[%s] not found", str_v.val());
+                php_swoole_fatal_error(E_ERROR, "ssl cert file[%s] not found", str_v.val());
                 return;
             }
             if (port->ssl_option.cert_file)
@@ -419,12 +480,12 @@ static PHP_METHOD(swoole_server_port, set)
             port->ssl_option.cert_file = str_v.dup();
             port->open_ssl_encrypt = 1;
         }
-        if (php_swoole_array_get_value(vht, "ssl_key_file", v))
+        if (php_swoole_array_get_value(vht, "ssl_key_file", ztmp))
         {
-            zend::string str_v(v);
+            zend::string str_v(ztmp);
             if (access(str_v.val(), R_OK) < 0)
             {
-                swoole_php_fatal_error(E_ERROR, "ssl key file[%s] not found", str_v.val());
+                php_swoole_fatal_error(E_ERROR, "ssl key file[%s] not found", str_v.val());
                 return;
             }
             if (port->ssl_option.key_file)
@@ -433,25 +494,36 @@ static PHP_METHOD(swoole_server_port, set)
             }
             port->ssl_option.key_file = str_v.dup();
         }
-        if (php_swoole_array_get_value(vht, "ssl_method", v))
+        if (php_swoole_array_get_value(vht, "ssl_method", ztmp))
         {
-            port->ssl_option.method = (int) zval_get_long(v);
+            zend_long v = zval_get_long(ztmp);
+            port->ssl_option.method = SW_MAX(0, SW_MIN(v, UINT8_MAX));
         }
-        if (php_swoole_array_get_value(vht, "ssl_verify_peer", v))
+        if (php_swoole_array_get_value(vht, "ssl_compress", ztmp))
         {
-            port->ssl_option.verify_peer = zval_is_true(v);
+            port->ssl_option.disable_compress = !zval_is_true(ztmp);
         }
-        if (php_swoole_array_get_value(vht, "ssl_allow_self_signed", v))
+        if (php_swoole_array_get_value(vht, "ssl_protocols", ztmp))
         {
-            port->ssl_option.allow_self_signed = zval_is_true(v);
+            zend_long v = zval_get_long(ztmp);
+            port->ssl_option.disable_protocols = (SW_SSL_SSLv2 | SW_SSL_SSLv3 | SW_SSL_TLSv1 | SW_SSL_TLSv1_1
+                    | SW_SSL_TLSv1_2) ^ v;
+        }
+        if (php_swoole_array_get_value(vht, "ssl_verify_peer", ztmp))
+        {
+            port->ssl_option.verify_peer = zval_is_true(ztmp);
+        }
+        if (php_swoole_array_get_value(vht, "ssl_allow_self_signed", ztmp))
+        {
+            port->ssl_option.allow_self_signed = zval_is_true(ztmp);
         }
         //verify client cert
-        if (php_swoole_array_get_value(vht, "ssl_client_cert_file", v))
+        if (php_swoole_array_get_value(vht, "ssl_client_cert_file", ztmp))
         {
-            zend::string str_v(v);
+            zend::string str_v(ztmp);
             if (access(str_v.val(), R_OK) < 0)
             {
-                swoole_php_fatal_error(E_ERROR, "ssl_client_cert_file[%s] not found", str_v.val());
+                php_swoole_fatal_error(E_ERROR, "ssl_client_cert_file[%s] not found", str_v.val());
                 return;
             }
             if (port->ssl_option.client_cert_file)
@@ -460,13 +532,14 @@ static PHP_METHOD(swoole_server_port, set)
             }
             port->ssl_option.client_cert_file = str_v.dup();
         }
-        if (php_swoole_array_get_value(vht, "ssl_verify_depth", v))
+        if (php_swoole_array_get_value(vht, "ssl_verify_depth", ztmp))
         {
-            port->ssl_option.verify_depth = (int) zval_get_long(v);
+            zend_long v = zval_get_long(ztmp);
+            port->ssl_option.verify_depth = SW_MAX(0, SW_MIN(v, UINT8_MAX));
         }
-        if (php_swoole_array_get_value(vht, "ssl_prefer_server_ciphers", v))
+        if (php_swoole_array_get_value(vht, "ssl_prefer_server_ciphers", ztmp))
         {
-            port->ssl_config.prefer_server_ciphers = zval_is_true(v);
+            port->ssl_config.prefer_server_ciphers = zval_is_true(ztmp);
         }
         //    if ((v = zend_hash_str_find(vht, ZEND_STRL("ssl_session_tickets"))))
         //    {
@@ -480,29 +553,29 @@ static PHP_METHOD(swoole_server_port, set)
         //    {
         //        port->ssl_config.stapling_verify = zval_is_true(v);
         //    }
-        if (php_swoole_array_get_value(vht, "ssl_ciphers", v))
+        if (php_swoole_array_get_value(vht, "ssl_ciphers", ztmp))
         {
             if (port->ssl_config.ciphers)
             {
                 sw_free(port->ssl_config.ciphers);
             }
-            port->ssl_config.ciphers = zend::string(v).dup();
+            port->ssl_config.ciphers = zend::string(ztmp).dup();
         }
-        if (php_swoole_array_get_value(vht, "ssl_ecdh_curve", v))
+        if (php_swoole_array_get_value(vht, "ssl_ecdh_curve", ztmp))
         {
             if (port->ssl_config.ecdh_curve)
             {
                 sw_free(port->ssl_config.ecdh_curve);
             }
-            port->ssl_config.ecdh_curve = zend::string(v).dup();
+            port->ssl_config.ecdh_curve = zend::string(ztmp).dup();
         }
-        if (php_swoole_array_get_value(vht, "ssl_dhparam", v))
+        if (php_swoole_array_get_value(vht, "ssl_dhparam", ztmp))
         {
             if (port->ssl_config.dhparam)
             {
                 sw_free(port->ssl_config.dhparam);
             }
-            port->ssl_config.dhparam = zend::string(v).dup();
+            port->ssl_config.dhparam = zend::string(ztmp).dup();
         }
         //    if ((v = zend_hash_str_find(vht, ZEND_STRL("ssl_session_cache"))))
         //    {
@@ -510,13 +583,13 @@ static PHP_METHOD(swoole_server_port, set)
         //    }
         if (swPort_enable_ssl_encrypt(port) < 0)
         {
-            swoole_php_fatal_error(E_ERROR, "swPort_enable_ssl_encrypt() failed");
+            php_swoole_fatal_error(E_ERROR, "swPort_enable_ssl_encrypt() failed");
             RETURN_FALSE;
         }
     }
 #endif
 
-    zval *zsetting = sw_zend_read_property_array(swoole_server_port_ce, getThis(), ZEND_STRL("setting"), 1);
+    zval *zsetting = sw_zend_read_and_convert_property_array(swoole_server_port_ce, ZEND_THIS, ZEND_STRL("setting"), 0);
     php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
     property->zsetting = zsetting;
 }
@@ -527,11 +600,11 @@ static PHP_METHOD(swoole_server_port, on)
     size_t len, i;
     zval *cb;
 
-    swoole_server_port_property *property = (swoole_server_port_property *) swoole_get_property(getThis(), 0);
+    swoole_server_port_property *property = (swoole_server_port_property *) swoole_get_property(ZEND_THIS, 0);
     swServer *serv = property->serv;
     if (serv->gs->start > 0)
     {
-        swoole_php_fatal_error(E_WARNING, "can't register event callback function after server started");
+        php_swoole_fatal_error(E_WARNING, "can't register event callback function after server started");
         RETURN_FALSE;
     }
 
@@ -544,7 +617,7 @@ static PHP_METHOD(swoole_server_port, on)
     zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache *) emalloc(sizeof(zend_fcall_info_cache));
     if (!sw_zend_is_callable_ex(cb, NULL, 0, &func_name, NULL, fci_cache, NULL))
     {
-        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
+        php_swoole_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
         return;
     }
     efree(func_name);
@@ -576,8 +649,8 @@ static PHP_METHOD(swoole_server_port, on)
         memcpy(property_name + 2, callback_name[i], len);
         l_property_name = len + 2;
         property_name[l_property_name] = '\0';
-        zend_update_property(swoole_server_port_ce, getThis(), property_name, l_property_name, cb);
-        property->callbacks[i] = sw_zend_read_property(swoole_server_port_ce, getThis(), property_name, l_property_name, 0);
+        zend_update_property(swoole_server_port_ce, ZEND_THIS, property_name, l_property_name, cb);
+        property->callbacks[i] = sw_zend_read_property(swoole_server_port_ce, ZEND_THIS, property_name, l_property_name, 0);
         sw_copy_to_stack(property->callbacks[i], property->_callbacks[i]);
         if (property->caches[i])
         {
@@ -614,7 +687,7 @@ static PHP_METHOD(swoole_server_port, on)
 
     if (l_property_name == 0)
     {
-        swoole_php_error(E_WARNING, "unknown event types[%s]", name);
+        php_swoole_error(E_WARNING, "unknown event types[%s]", name);
         efree(fci_cache);
         RETURN_FALSE;
     }
@@ -635,7 +708,7 @@ static PHP_METHOD(swoole_server_port, getCallback)
     if (i != server_port_event_map.end())
     {
         string property_name = "on" + i->second.name;
-        zval rv, *property = zend_read_property(swoole_server_port_ce, getThis(), property_name.c_str(), property_name.length(), 1, &rv);
+        zval rv, *property = zend_read_property(swoole_server_port_ce, ZEND_THIS, property_name.c_str(), property_name.length(), 1, &rv);
         if (!ZVAL_IS_NULL(property))
         {
             RETURN_ZVAL(property, 1, 0);
@@ -647,7 +720,7 @@ static PHP_METHOD(swoole_server_port, getCallback)
 #ifdef SWOOLE_SOCKETS_SUPPORT
 static PHP_METHOD(swoole_server_port, getSocket)
 {
-    swListenPort *port = (swListenPort *) swoole_get_object(getThis());
+    swListenPort *port = (swListenPort *) swoole_get_object(ZEND_THIS);
     php_socket *socket_object = swoole_convert_to_socket(port->sock);
     if (!socket_object)
     {
